@@ -27,7 +27,6 @@
 #include "kernel/osl/types.h"
 
 #include "kernel/integrator/state.h"
-#include "kernel/integrator/state_util.h"
 
 #include "kernel/geom/primitive.h"
 #include "kernel/geom/shader_data.h"
@@ -40,6 +39,7 @@
 #include "kernel/svm/bevel.h"
 
 #include "kernel/util/ies.h"
+#include "kernel/util/texture_3d.h"
 
 CCL_NAMESPACE_BEGIN
 
@@ -106,6 +106,7 @@ ustring OSLRenderServices::u_path_diffuse_depth("path:diffuse_depth");
 ustring OSLRenderServices::u_path_glossy_depth("path:glossy_depth");
 ustring OSLRenderServices::u_path_transparent_depth("path:transparent_depth");
 ustring OSLRenderServices::u_path_transmission_depth("path:transmission_depth");
+ustring OSLRenderServices::u_path_portal_depth("path:portal_depth");
 ustring OSLRenderServices::u_trace("trace");
 ustring OSLRenderServices::u_hit("hit");
 ustring OSLRenderServices::u_hitdist("hitdist");
@@ -134,7 +135,7 @@ OSLRenderServices::OSLRenderServices(OSL::TextureSystem *texture_system, const i
 OSLRenderServices::~OSLRenderServices()
 {
   if (m_texturesys) {
-    VLOG_INFO << "OSL texture system stats:\n" << m_texturesys->getstats();
+    LOG_INFO << "OSL texture system stats:\n" << m_texturesys->getstats();
   }
 }
 
@@ -144,6 +145,8 @@ int OSLRenderServices::supports(string_view feature) const
   if (feature == "OptiX") {
     return device_type_ == DEVICE_OPTIX;
   }
+#else
+  (void)feature;
 #endif
 
   return false;
@@ -151,7 +154,7 @@ int OSLRenderServices::supports(string_view feature) const
 
 bool OSLRenderServices::get_matrix(OSL::ShaderGlobals *sg,
                                    OSL::Matrix44 &result,
-                                   OSL::TransformationPtr xform,
+                                   OSL::TransformationPtr /*xform*/,
                                    const float time)
 {
   ShaderGlobals *globals = reinterpret_cast<ShaderGlobals *>(sg);
@@ -189,7 +192,7 @@ bool OSLRenderServices::get_matrix(OSL::ShaderGlobals *sg,
 
 bool OSLRenderServices::get_inverse_matrix(OSL::ShaderGlobals *sg,
                                            OSL::Matrix44 &result,
-                                           OSL::TransformationPtr xform,
+                                           OSL::TransformationPtr /*xform*/,
                                            const float time)
 {
   ShaderGlobals *globals = reinterpret_cast<ShaderGlobals *>(sg);
@@ -228,7 +231,7 @@ bool OSLRenderServices::get_inverse_matrix(OSL::ShaderGlobals *sg,
 bool OSLRenderServices::get_matrix(OSL::ShaderGlobals *sg,
                                    OSL::Matrix44 &result,
                                    OSLUStringHash from,
-                                   const float time)
+                                   const float /*time*/)
 {
   ShaderGlobals *globals = reinterpret_cast<ShaderGlobals *>(sg);
   const ThreadKernelGlobalsCPU *kg = globals->kg;
@@ -260,7 +263,7 @@ bool OSLRenderServices::get_matrix(OSL::ShaderGlobals *sg,
 bool OSLRenderServices::get_inverse_matrix(OSL::ShaderGlobals *sg,
                                            OSL::Matrix44 &result,
                                            OSLUStringHash to,
-                                           const float time)
+                                           const float /*time*/)
 {
   ShaderGlobals *globals = reinterpret_cast<ShaderGlobals *>(sg);
   const ThreadKernelGlobalsCPU *kg = globals->kg;
@@ -291,7 +294,7 @@ bool OSLRenderServices::get_inverse_matrix(OSL::ShaderGlobals *sg,
 
 bool OSLRenderServices::get_matrix(OSL::ShaderGlobals *sg,
                                    OSL::Matrix44 &result,
-                                   OSL::TransformationPtr xform)
+                                   OSL::TransformationPtr /*xform*/)
 {
   ShaderGlobals *globals = reinterpret_cast<ShaderGlobals *>(sg);
 
@@ -317,7 +320,7 @@ bool OSLRenderServices::get_matrix(OSL::ShaderGlobals *sg,
 
 bool OSLRenderServices::get_inverse_matrix(OSL::ShaderGlobals *sg,
                                            OSL::Matrix44 &result,
-                                           OSL::TransformationPtr xform)
+                                           OSL::TransformationPtr /*xform*/)
 {
   ShaderGlobals *globals = reinterpret_cast<ShaderGlobals *>(sg);
 
@@ -395,129 +398,74 @@ bool OSLRenderServices::get_inverse_matrix(OSL::ShaderGlobals *sg,
   return false;
 }
 
-bool OSLRenderServices::get_array_attribute(OSL::ShaderGlobals *sg,
+bool OSLRenderServices::get_array_attribute(OSL::ShaderGlobals * /*sg*/,
+                                            bool /* derivatives*/,
+                                            OSLUStringHash /* object*/,
+                                            const TypeDesc /* type*/,
+                                            OSLUStringHash /* name*/,
+                                            const int /* index*/,
+                                            void * /*val*/)
+{
+  return false;
+}
+
+ccl_device_template_spec bool set_attribute(const dual1 v,
+                                            TypeDesc type,
                                             bool derivatives,
-                                            OSLUStringHash object,
-                                            const TypeDesc type,
-                                            OSLUStringHash name,
-                                            const int index,
                                             void *val)
-{
-  return false;
-}
-
-template<typename T>
-inline bool set_attribute(
-    const T v, const T dx, const T dy, TypeDesc type, bool derivatives, void *val);
-
-inline void set_data_float(
-    const float v, const float dx, const float dy, bool derivatives, void *val)
-{
-  float *fval = static_cast<float *>(val);
-  fval[0] = v;
-  if (derivatives) {
-    fval[1] = dx;
-    fval[2] = dy;
-  }
-}
-
-inline void set_data_float3(
-    const float3 v, const float3 dx, const float3 dy, bool derivatives, void *val)
-{
-  float *fval = static_cast<float *>(val);
-  fval[0] = v.x;
-  fval[1] = v.y;
-  fval[2] = v.z;
-  if (derivatives) {
-    fval[3] = dx.x;
-    fval[4] = dx.y;
-    fval[5] = dx.z;
-    fval[6] = dy.x;
-    fval[7] = dy.y;
-    fval[8] = dy.z;
-  }
-}
-
-inline void set_data_float4(
-    const float4 v, const float4 dx, const float4 dy, bool derivatives, void *val)
-{
-  float *fval = static_cast<float *>(val);
-  fval[0] = v.x;
-  fval[1] = v.y;
-  fval[2] = v.z;
-  fval[3] = v.w;
-  if (derivatives) {
-    fval[4] = dx.x;
-    fval[5] = dx.y;
-    fval[6] = dx.z;
-    fval[7] = dx.w;
-    fval[8] = dy.x;
-    fval[9] = dy.y;
-    fval[10] = dy.z;
-    fval[11] = dy.w;
-  }
-}
-
-ccl_device_template_spec bool set_attribute(
-    const float v, const float dx, const float dy, TypeDesc type, bool derivatives, void *val)
 {
   if (type == TypeFloatArray4) {
-    set_data_float4(make_float4(v, v, v, 1.0f),
-                    make_float4(dx, dx, dx, 0.0f),
-                    make_float4(dy, dy, dy, 0.0f),
-                    derivatives,
-                    val);
+    set_data_float4(make_float4(make_float3(v)), derivatives, val);
     return true;
   }
   if (type == TypePoint || type == TypeVector || type == TypeNormal || type == TypeColor) {
-    set_data_float3(make_float3(v), make_float3(dx), make_float3(dy), derivatives, val);
+    set_data_float3(make_float3(v), derivatives, val);
     return true;
   }
   if (type == TypeFloat) {
-    set_data_float(v, dx, dy, derivatives, val);
+    set_data_float(v, derivatives, val);
     return true;
   }
 
   return false;
 }
 
-ccl_device_template_spec bool set_attribute(
-    const float2 v, const float2 dx, const float2 dy, TypeDesc type, bool derivatives, void *val)
+ccl_device_template_spec bool set_attribute(const dual2 v,
+                                            TypeDesc type,
+                                            bool derivatives,
+                                            void *val)
 {
   if (type == TypeFloatArray4) {
-    set_data_float4(make_float4(v.x, v.y, 0.0f, 1.0f),
-                    make_float4(dx.x, dx.y, 0.0f, 0.0f),
-                    make_float4(dy.x, dy.y, 0.0f, 0.0f),
-                    derivatives,
-                    val);
+    set_data_float4(make_float4(make_float3(v)), derivatives, val);
     return true;
   }
   if (type == TypePoint || type == TypeVector || type == TypeNormal || type == TypeColor) {
-    set_data_float3(make_float3(v), make_float3(dx), make_float3(dy), derivatives, val);
+    set_data_float3(make_float3(v), derivatives, val);
     return true;
   }
   if (type == TypeFloat) {
-    set_data_float(average(v), average(dx), average(dy), derivatives, val);
+    set_data_float(average(v), derivatives, val);
     return true;
   }
 
   return false;
 }
 
-ccl_device_template_spec bool set_attribute(
-    const float3 v, const float3 dx, const float3 dy, TypeDesc type, bool derivatives, void *val)
+ccl_device_template_spec bool set_attribute(const dual3 v,
+                                            TypeDesc type,
+                                            bool derivatives,
+                                            void *val)
 {
   if (type == TypeFloatArray4) {
-    set_data_float4(
-        make_float4(v, 1.0f), make_float4(dx, 0.0f), make_float4(dy, 0.0f), derivatives, val);
+    set_data_float4(make_float4(v), derivatives, val);
     return true;
   }
   if (type == TypePoint || type == TypeVector || type == TypeNormal || type == TypeColor) {
-    set_data_float3(v, dx, dy, derivatives, val);
+    set_data_float3(v, derivatives, val);
     return true;
   }
   if (type == TypeFloat) {
-    set_data_float(average(v), average(dx), average(dy), derivatives, val);
+    set_data_float(average(v), derivatives, val);
     return true;
   }
 
@@ -532,23 +480,21 @@ ccl_device_template_spec bool set_attribute(
  * this for the correct operation of the Attribute node.
  */
 
-ccl_device_template_spec bool set_attribute(
-    const float4 v, const float4 dx, const float4 dy, TypeDesc type, bool derivatives, void *val)
+ccl_device_template_spec bool set_attribute(const dual4 v,
+                                            TypeDesc type,
+                                            bool derivatives,
+                                            void *val)
 {
   if (type == TypeFloatArray4) {
-    set_data_float4(v, dx, dy, derivatives, val);
+    set_data_float4(v, derivatives, val);
     return true;
   }
   if (type == TypePoint || type == TypeVector || type == TypeNormal || type == TypeColor) {
-    set_data_float3(make_float3(v), make_float3(dx), make_float3(dy), derivatives, val);
+    set_data_float3(make_float3(v), derivatives, val);
     return true;
   }
   if (type == TypeFloat) {
-    set_data_float(average(make_float3(v)),
-                   average(make_float3(dx)),
-                   average(make_float3(dy)),
-                   derivatives,
-                   val);
+    set_data_float(average(make_float3(v)), derivatives, val);
     return true;
   }
   return false;
@@ -557,7 +503,7 @@ ccl_device_template_spec bool set_attribute(
 template<typename T>
 ccl_device_inline bool set_attribute(const T f, const TypeDesc type, bool derivatives, void *val)
 {
-  return set_attribute(f, make_zero<T>(), make_zero<T>(), type, derivatives, val);
+  return set_attribute(dual<T>(f), type, derivatives, val);
 }
 
 ccl_device_template_spec bool set_attribute(const int i,
@@ -588,12 +534,12 @@ ccl_device_template_spec bool set_attribute(ustring str,
   if (type.basetype == TypeDesc::STRING && type.aggregate == TypeDesc::SCALAR &&
       type.arraylen == 0)
   {
-    ustring *sval = (ustring *)val;
+    OSLUStringHash *sval = (OSLUStringHash *)val;
     sval[0] = str;
 
     if (derivatives) {
-      sval[1] = OSLRenderServices::u_empty;
-      sval[2] = OSLRenderServices::u_empty;
+      sval[1] = OSLUStringHash();
+      sval[2] = OSLUStringHash();
     }
 
     return true;
@@ -607,17 +553,9 @@ static bool set_attribute_float3_3(const float3 P[3], TypeDesc type, bool deriva
   if (type.vecsemantics == TypeDesc::POINT && type.arraylen >= 3) {
     float *fval = (float *)val;
 
-    fval[0] = P[0].x;
-    fval[1] = P[0].y;
-    fval[2] = P[0].z;
-
-    fval[3] = P[1].x;
-    fval[4] = P[1].y;
-    fval[5] = P[1].z;
-
-    fval[6] = P[2].x;
-    fval[7] = P[2].y;
-    fval[8] = P[2].z;
+    copy_v3_v3(fval, P[0]);
+    copy_v3_v3(fval + 3, P[1]);
+    copy_v3_v3(fval + 6, P[2]);
 
     if (type.arraylen > 3) {
       memset(fval + 3 * 3, 0, sizeof(float) * 3 * (type.arraylen - 3));
@@ -650,20 +588,17 @@ inline bool get_object_attribute_impl(const ThreadKernelGlobalsCPU *kg,
                                       bool derivatives,
                                       void *val)
 {
-  T v;
-  T dx = make_zero<T>();
-  T dy = make_zero<T>();
+  dual<T> data;
 #ifdef __VOLUME__
-  if (primitive_is_volume_attribute(sd, desc)) {
-    v = primitive_volume_attribute<T>(kg, sd, desc);
+  if (primitive_is_volume_attribute(sd)) {
+    data.val = primitive_volume_attribute<T>(kg, sd, desc, true);
   }
   else
 #endif
   {
-    v = primitive_surface_attribute<T>(
-        kg, sd, desc, derivatives ? &dx : nullptr, derivatives ? &dy : nullptr);
+    data = primitive_surface_attribute<T>(kg, sd, desc, derivatives, derivatives);
   }
-  return set_attribute(v, dx, dy, type, derivatives, val);
+  return set_attribute(data, type, derivatives, val);
 }
 
 static bool get_object_attribute(const ThreadKernelGlobalsCPU *kg,
@@ -676,16 +611,16 @@ static bool get_object_attribute(const ThreadKernelGlobalsCPU *kg,
   if (desc.type == NODE_ATTR_FLOAT) {
     return get_object_attribute_impl<float>(kg, sd, desc, type, derivatives, val);
   }
-  else if (desc.type == NODE_ATTR_FLOAT2) {
+  if (desc.type == NODE_ATTR_FLOAT2) {
     return get_object_attribute_impl<float2>(kg, sd, desc, type, derivatives, val);
   }
-  else if (desc.type == NODE_ATTR_FLOAT3) {
+  if (desc.type == NODE_ATTR_FLOAT3) {
     return get_object_attribute_impl<float3>(kg, sd, desc, type, derivatives, val);
   }
-  else if (desc.type == NODE_ATTR_FLOAT4 || desc.type == NODE_ATTR_RGBA) {
+  if (desc.type == NODE_ATTR_FLOAT4 || desc.type == NODE_ATTR_RGBA) {
     return get_object_attribute_impl<float4>(kg, sd, desc, type, derivatives, val);
   }
-  else if (desc.type == NODE_ATTR_MATRIX) {
+  if (desc.type == NODE_ATTR_MATRIX) {
     const Transform tfm = primitive_attribute_matrix(kg, desc);
     return set_attribute_matrix(tfm, type, val);
   }
@@ -829,7 +764,7 @@ bool OSLRenderServices::get_object_standard_attribute(
     return set_attribute(f, type, derivatives, val);
   }
   if (name == u_curve_tangent_normal) {
-    const float3 f = curve_tangent_normal(kg, sd);
+    const float3 f = curve_tangent_normal(sd);
     return set_attribute(f, type, derivatives, val);
   }
   if (name == u_curve_random) {
@@ -858,17 +793,22 @@ bool OSLRenderServices::get_object_standard_attribute(
 #endif
   if (name == u_normal_map_normal) {
     if (sd->type & PRIMITIVE_TRIANGLE) {
+      const AttributeDescriptor desc = find_attribute(
+          kg, sd->object, sd->prim, ATTR_STD_NORMAL_UNDISPLACED);
+      if (desc.offset != ATTR_STD_NOT_FOUND) {
+        return get_object_attribute(kg, sd, desc, type, derivatives, val);
+      }
       const float3 f = triangle_smooth_normal_unnormalized(kg, sd, sd->Ng, sd->prim, sd->u, sd->v);
       return set_attribute(f, type, derivatives, val);
     }
     return false;
   }
   if (name == u_bump_map_normal) {
-    float3 f[3];
+    dual3 f;
     if (!attribute_bump_map_normal(kg, sd, f)) {
       return false;
     }
-    return set_attribute(f[0], f[1], f[2], type, derivatives, val);
+    return set_attribute(f, type, derivatives, val);
   }
   return get_background_attribute(globals, name, type, derivatives, val);
 }
@@ -924,33 +864,33 @@ bool OSLRenderServices::get_background_attribute(
     const int f = READ_PATH_STATE(transparent_bounce);
     return set_attribute(f, type, derivatives, val);
   }
+  if (name == u_path_portal_depth) {
+    /* Portal Ray Depth */
+    const int f = READ_PATH_STATE(portal_bounce);
+    return set_attribute(f, type, derivatives, val);
+  }
 #undef READ_PATH_STATE
 
   if (name == u_ndc) {
     /* NDC coordinates with special exception for orthographic projection. */
-    float3 ndc[3];
+    dual3 ndc;
 
     if ((globals->raytype & PATH_RAY_CAMERA) && sd->object == OBJECT_NONE &&
         kernel_data.cam.type == CAMERA_ORTHOGRAPHIC)
     {
-      ndc[0] = camera_world_to_ndc(kg, sd, sd->ray_P);
-
-      if (derivatives) {
-        ndc[1] = zero_float3();
-        ndc[2] = zero_float3();
-      }
+      ndc.val = camera_world_to_ndc(kg, sd, sd->ray_P);
     }
     else {
-      ndc[0] = camera_world_to_ndc(kg, sd, sd->P);
+      ndc.val = camera_world_to_ndc(kg, sd, sd->P);
 
       if (derivatives) {
         const differential3 dP = differential_from_compact(sd->Ng, sd->dP);
-        ndc[1] = camera_world_to_ndc(kg, sd, sd->P + dP.dx) - ndc[0];
-        ndc[2] = camera_world_to_ndc(kg, sd, sd->P + dP.dy) - ndc[0];
+        ndc.dx = camera_world_to_ndc(kg, sd, sd->P + dP.dx) - ndc.val;
+        ndc.dy = camera_world_to_ndc(kg, sd, sd->P + dP.dy) - ndc.val;
       }
     }
 
-    return set_attribute(ndc[0], ndc[1], ndc[2], type, derivatives, val);
+    return set_attribute(ndc, type, derivatives, val);
   }
 
   return false;
@@ -964,23 +904,23 @@ bool OSLRenderServices::get_camera_attribute(
     const float2 sensor = make_float2(kernel_data.cam.sensorwidth, kernel_data.cam.sensorheight);
     return set_attribute(sensor, type, derivatives, val);
   }
-  else if (name == u_image_resolution) {
+  if (name == u_image_resolution) {
     const float2 image = make_float2(kernel_data.cam.width, kernel_data.cam.height);
     return set_attribute(image, type, derivatives, val);
   }
-  else if (name == u_aperture_aspect_ratio) {
+  if (name == u_aperture_aspect_ratio) {
     return set_attribute(1.0f / kernel_data.cam.inv_aperture_ratio, type, derivatives, val);
   }
-  else if (name == u_aperture_size) {
+  if (name == u_aperture_size) {
     return set_attribute(kernel_data.cam.aperturesize, type, derivatives, val);
   }
-  else if (name == u_aperture_position) {
+  if (name == u_aperture_position) {
     /* The random numbers for aperture sampling are packed into N. */
     const float2 rand_lens = make_float2(globals->N.x, globals->N.y);
     const float2 pos = camera_sample_aperture(&kernel_data.cam, rand_lens);
     return set_attribute(pos * kernel_data.cam.aperturesize, type, derivatives, val);
   }
-  else if (name == u_focal_distance) {
+  if (name == u_focal_distance) {
     return set_attribute(kernel_data.cam.focaldistance, type, derivatives, val);
   }
   return false;
@@ -1031,8 +971,11 @@ bool OSLRenderServices::get_attribute(OSL::ShaderGlobals *sg,
   return get_object_standard_attribute(globals, name, type, derivatives, val);
 }
 
-bool OSLRenderServices::get_userdata(
-    bool derivatives, OSLUStringHash name, const TypeDesc type, OSL::ShaderGlobals *sg, void *val)
+bool OSLRenderServices::get_userdata(bool /*derivatives*/,
+                                     OSLUStringHash /* name*/,
+                                     const TypeDesc /* type*/,
+                                     OSL::ShaderGlobals * /*sg*/,
+                                     void * /*val*/)
 {
   return false; /* disabled by lockgeom */
 }
@@ -1148,7 +1091,7 @@ bool OSLRenderServices::texture(OSLUStringHash filename,
                                 float *result,
                                 float *dresultds,
                                 float *dresultdt,
-                                OSLUStringHash *errormessage)
+                                OSLUStringHash * /*errormessage*/)
 {
   OSLTextureHandle *handle = (OSLTextureHandle *)texture_handle;
   const OSLTextureHandle::Type texture_type = (handle) ? handle->type : OSLTextureHandle::OIIO;
@@ -1329,7 +1272,7 @@ bool OSLRenderServices::texture3d(OSLUStringHash filename,
                                   float *dresultds,
                                   float *dresultdt,
                                   float *dresultdr,
-                                  OSLUStringHash *errormessage)
+                                  OSLUStringHash * /*errormessage*/)
 {
   OSLTextureHandle *handle = (OSLTextureHandle *)texture_handle;
   const OSLTextureHandle::Type texture_type = (handle) ? handle->type : OSLTextureHandle::OIIO;
@@ -1342,7 +1285,8 @@ bool OSLRenderServices::texture3d(OSLUStringHash filename,
       /* Packed texture. */
       const int slot = handle->svm_slots[0].y;
       const float3 P_float3 = make_float3(P.x, P.y, P.z);
-      float4 rgba = kernel_tex_image_interp_3d(kernel_globals, slot, P_float3, INTERPOLATION_NONE);
+      float4 rgba = kernel_tex_image_interp_3d(
+          kernel_globals, globals->sd, slot, P_float3, INTERPOLATION_NONE, false);
 
       result[0] = rgba[0];
       if (nchannels > 1) {
@@ -1438,7 +1382,7 @@ bool OSLRenderServices::environment(OSLUStringHash filename,
                                     float *result,
                                     float *dresultds,
                                     float *dresultdt,
-                                    OSLUStringHash *errormessage)
+                                    OSLUStringHash * /*errormessage*/)
 {
   OSLTextureHandle *handle = (OSLTextureHandle *)texture_handle;
   OSL::TextureSystem *ts = m_texturesys;
@@ -1519,45 +1463,46 @@ bool OSLRenderServices::get_texture_info(OSLUStringHash filename,
       to_ustring(filename), subimage, to_ustring(dataname), datatype, data);
 }
 
-int OSLRenderServices::pointcloud_search(OSL::ShaderGlobals *sg,
-                                         OSLUStringHash filename,
-                                         const OSL::Vec3 &center,
-                                         const float radius,
-                                         const int max_points,
-                                         bool sort,
+int OSLRenderServices::pointcloud_search(OSL::ShaderGlobals * /*sg*/,
+                                         OSLUStringHash /*filename*/,
+                                         const OSL::Vec3 & /*center*/,
+                                         const float /*radius*/,
+                                         const int /*max_points*/,
+                                         bool /*sort*/,
 #if OSL_LIBRARY_VERSION_CODE >= 11400
-                                         int *indices,
+                                         int * /*indices*/,
 #else
-                                         size_t *out_indices,
+                                         size_t * /*out_indices*/,
 #endif
-                                         float *out_distances,
-                                         const int derivs_offset)
+                                         float * /*out_distances*/,
+                                         const int /*derivs_offset*/)
 {
   return 0;
 }
 
-int OSLRenderServices::pointcloud_get(OSL::ShaderGlobals *sg,
-                                      OSLUStringHash filename,
+int OSLRenderServices::pointcloud_get(OSL::ShaderGlobals * /*sg*/
+                                      ,
+                                      OSLUStringHash /*filename*/,
 #if OSL_LIBRARY_VERSION_CODE >= 11400
-                                      const int *indices,
+                                      const int * /*indices*/,
 #else
-                                      size_t *indices,
+                                      size_t * /*indices*/,
 #endif
-                                      const int count,
-                                      OSLUStringHash attr_name,
-                                      const TypeDesc attr_type,
-                                      void *out_data)
+                                      const int /*count*/,
+                                      OSLUStringHash /*attr_name*/,
+                                      const TypeDesc /*attr_type*/,
+                                      void * /*out_data*/)
 {
   return 0;
 }
 
-bool OSLRenderServices::pointcloud_write(OSL::ShaderGlobals *sg,
-                                         OSLUStringHash filename,
-                                         const OSL::Vec3 &pos,
-                                         const int nattribs,
-                                         const OSLUStringRep *names,
-                                         const TypeDesc *types,
-                                         const void **data)
+bool OSLRenderServices::pointcloud_write(OSL::ShaderGlobals * /*sg*/,
+                                         OSLUStringHash /*filename*/,
+                                         const OSL::Vec3 & /*pos*/,
+                                         const int /*nattribs*/,
+                                         const OSLUStringRep * /*names*/,
+                                         const TypeDesc * /*types*/,
+                                         const void ** /*data*/)
 {
   return false;
 }
@@ -1669,17 +1614,17 @@ bool OSLRenderServices::getmessage(OSL::ShaderGlobals *sg,
       }
       if (name == u_P) {
         const differential3 dP = differential_from_compact(sd->Ng, sd->dP);
-        return set_attribute(sd->P, dP.dx, dP.dy, type, derivatives, val);
+        return set_attribute(dual3(sd->P, dP.dx, dP.dy), type, derivatives, val);
       }
       if (name == u_I) {
         const differential3 dI = differential_from_compact(sd->wi, sd->dI);
-        return set_attribute(sd->wi, dI.dx, dI.dy, type, derivatives, val);
+        return set_attribute(dual3(sd->wi, dI.dx, dI.dy), type, derivatives, val);
       }
       if (name == u_u) {
-        return set_attribute(sd->u, sd->du.dx, sd->du.dy, type, derivatives, val);
+        return set_attribute(dual1(sd->u, sd->du.dx, sd->du.dy), type, derivatives, val);
       }
       if (name == u_v) {
-        return set_attribute(sd->v, sd->dv.dx, sd->dv.dy, type, derivatives, val);
+        return set_attribute(dual1(sd->v, sd->dv.dx, sd->dv.dy), type, derivatives, val);
       }
 
       return get_attribute(sg, derivatives, u_empty, type, name, val);
