@@ -33,14 +33,17 @@ public:
     Object(ObjectType _type):m_type(_type), m_cache(NULL), m_internalPose(F_identity), m_updated(false) {};
     virtual ~Object(){};
 
-	virtual int addEndEffector(const std::string& name){return 0;};
+	virtual int addEndEffector(const std::string&  /*name*/){return 0;};
 	virtual bool finalize(){return true;};
-	virtual const KDL::Frame& getPose(const unsigned int end_effector=0){return m_internalPose;};
-    virtual const ObjectType getType(){return m_type;};
-    virtual const unsigned int getNrOfCoordinates(){return 0;};
-    virtual void updateKinematics(const Timestamp& timestamp)=0;
-    virtual void pushCache(const Timestamp& timestamp)=0;
-	virtual void initCache(Cache *_cache) = 0;
+	virtual const KDL::Frame& getPose(const unsigned int end_effector=0){
+		(void)end_effector;
+		return m_internalPose;
+	};
+    virtual ObjectType getType(){return m_type;};
+    virtual unsigned int getNrOfCoordinates(){return 0;};
+    virtual void updateKinematics(const Timestamp& /*timestamp*/)=0;
+    virtual void pushCache(const Timestamp& /*timestamp*/)=0;
+	virtual void initCache(Cache * /*cache*/) = 0;
 	bool updated() {return m_updated;};
 	void updated(bool val) {m_updated=val;};
 };
