@@ -6,6 +6,8 @@
  * \ingroup bke
  */
 
+#include <algorithm>
+
 #include "MEM_guardedalloc.h"
 
 #include "BLI_listbase.h"
@@ -16,17 +18,15 @@
 
 #include "DNA_armature_types.h"
 #include "DNA_constraint_types.h"
+#include "DNA_curve_types.h"
 #include "DNA_object_types.h"
-#include "DNA_scene_types.h"
 
-#include "BKE_action.h"
+#include "BKE_action.hh"
 #include "BKE_anim_path.h"
-#include "BKE_armature.h"
-#include "BKE_curve.h"
-#include "BKE_displist.h"
-#include "BKE_fcurve.h"
-#include "BKE_object.hh"
-#include "BKE_scene.h"
+#include "BKE_armature.hh"
+#include "BKE_curve.hh"
+#include "BKE_object_types.hh"
+#include "BKE_scene.hh"
 
 #include "BIK_api.h"
 
@@ -92,7 +92,8 @@ static void splineik_init_tree_from_pchan(Scene * /*scene*/,
   /* Find the root bone and the chain of bones from the root to the tip.
    * NOTE: this assumes that the bones are connected, but that may not be true... */
   for (pchan = pchan_tip; pchan && (segcount < ik_data->chainlen);
-       pchan = pchan->parent, segcount++) {
+       pchan = pchan->parent, segcount++)
+  {
     /* Store this segment in the chain. */
     pchan_chain[segcount] = pchan;
 
@@ -116,8 +117,7 @@ static void splineik_init_tree_from_pchan(Scene * /*scene*/,
       MEM_freeN(ik_data->points);
     }
     ik_data->numpoints = ik_data->chainlen + 1;
-    ik_data->points = static_cast<float *>(
-        MEM_mallocN(sizeof(float) * ik_data->numpoints, "Spline IK Binding"));
+    ik_data->points = MEM_malloc_arrayN<float>(size_t(ik_data->numpoints), "Spline IK Binding");
 
     /* Bind 'tip' of chain (i.e. first joint = tip of bone with the Spline IK Constraint). */
     ik_data->points[0] = 1.0f;
@@ -153,16 +153,14 @@ static void splineik_init_tree_from_pchan(Scene * /*scene*/,
    * since that would take precedence... */
   {
     /* Make a new tree. */
-    tSplineIK_Tree *tree = static_cast<tSplineIK_Tree *>(
-        MEM_callocN(sizeof(tSplineIK_Tree), "SplineIK Tree"));
+    tSplineIK_Tree *tree = MEM_callocN<tSplineIK_Tree>("SplineIK Tree");
     tree->type = CONSTRAINT_TYPE_SPLINEIK;
 
     tree->chainlen = segcount;
     tree->totlength = totlength;
 
     /* Copy over the array of links to bones in the chain (from tip to root). */
-    tree->chain = static_cast<bPoseChannel **>(
-        MEM_mallocN(sizeof(bPoseChannel *) * segcount, "SplineIK Chain"));
+    tree->chain = MEM_malloc_arrayN<bPoseChannel *>(size_t(segcount), "SplineIK Chain");
     memcpy(tree->chain, pchan_chain, sizeof(bPoseChannel *) * segcount);
 
     /* Store reference to joint position array. */
@@ -216,7 +214,7 @@ static bool splineik_evaluate_init(tSplineIK_Tree *tree, tSplineIk_EvalState *st
     return false;
   }
 
-  CurveCache *cache = ik_data->tar->runtime.curve_cache;
+  CurveCache *cache = ik_data->tar->runtime->curve_cache;
 
   if (ELEM(nullptr, cache, cache->anim_path_accum_length)) {
     return false;
@@ -253,11 +251,11 @@ static void apply_curve_transform(
    * unless the option to allow curve to be positioned elsewhere is activated (i.e. no root).
    */
   if ((ik_data->flag & CONSTRAINT_SPLINEIK_NO_ROOT) == 0) {
-    mul_m4_v3(ik_data->tar->object_to_world, r_vec);
+    mul_m4_v3(ik_data->tar->object_to_world().ptr(), r_vec);
   }
 
   /* Convert the position to pose-space. */
-  mul_m4_v3(ob->world_to_object, r_vec);
+  mul_m4_v3(ob->world_to_object().ptr(), r_vec);
 
   /* Set the new radius (it should be the average value). */
   *r_radius = (radius + *r_radius) / 2;
@@ -286,7 +284,7 @@ static int position_tail_on_spline(bSplineIKConstraint *ik_data,
   /* This is using the tessellated curve data.
    * So we are working with piece-wise linear curve segments.
    * The same method is used in #BKE_where_on_path to get curve location data. */
-  const CurveCache *cache = ik_data->tar->runtime.curve_cache;
+  const CurveCache *cache = ik_data->tar->runtime->curve_cache;
   const float *seg_accum_len = cache->anim_path_accum_length;
 
   int max_seg_idx = BKE_anim_path_get_array_size(cache) - 1;
@@ -337,7 +335,7 @@ static int position_tail_on_spline(bSplineIKConstraint *ik_data,
   }
 
   /* Calculate the intersection point using the secant root finding method */
-  float x0 = 0.0f, x1 = 1.0f, x2 = 0.5f;
+  float x0 = 0.0f, x1 = 1.0f;
   float x0_point[3], x1_point[3], start_p[3];
   float epsilon = max_fff(1.0f, len_v3(head_pos), len_v3(bp->vec)) * FLT_EPSILON;
 
@@ -363,7 +361,7 @@ static int position_tail_on_spline(bSplineIKConstraint *ik_data,
       break;
     }
 
-    x2 = x1 - f_x1 * (x1 - x0) / (f_x1 - f_x0);
+    const float x2 = x1 - f_x1 * (x1 - x0) / (f_x1 - f_x0);
     x0 = x1;
     x1 = x2;
   }
@@ -663,7 +661,7 @@ static void splineik_evaluate_bone(
           }
           if (bulge < 1.0f) {
             if (ik_data->flag & CONSTRAINT_SPLINEIK_USE_BULGE_MIN) {
-              float bulge_min = CLAMPIS(ik_data->bulge_min, 0.0f, 1.0f);
+              float bulge_min = std::clamp(ik_data->bulge_min, 0.0f, 1.0f);
               float hard = max_ff(bulge, bulge_min);
 
               float range = 1.0f - bulge_min;
@@ -674,7 +672,7 @@ static void splineik_evaluate_bone(
             }
           }
 
-          /* Compute scale factor for xz axes from this value. */
+          /* Compute scale factor for XZ axes from this value. */
           final_scale = sqrtf(bulge);
         }
         else {
@@ -794,8 +792,7 @@ void BKE_pose_pchan_index_rebuild(bPose *pose)
 {
   MEM_SAFE_FREE(pose->chan_array);
   const int num_channels = BLI_listbase_count(&pose->chanbase);
-  pose->chan_array = static_cast<bPoseChannel **>(
-      MEM_malloc_arrayN(num_channels, sizeof(bPoseChannel *), "pose->chan_array"));
+  pose->chan_array = MEM_malloc_arrayN<bPoseChannel *>(size_t(num_channels), "pose->chan_array");
   int pchan_index = 0;
   for (bPoseChannel *pchan = static_cast<bPoseChannel *>(pose->chanbase.first); pchan != nullptr;
        pchan = pchan->next)
@@ -828,7 +825,7 @@ void BKE_pose_eval_init(Depsgraph *depsgraph, Scene * /*scene*/, Object *object)
   BLI_assert((object->pose->flag & POSE_RECALC) == 0);
 
   /* world_to_object is needed for solvers. */
-  invert_m4_m4(object->world_to_object, object->object_to_world);
+  invert_m4_m4(object->runtime->world_to_object.ptr(), object->object_to_world().ptr());
 
   /* clear flags */
   for (bPoseChannel *pchan = static_cast<bPoseChannel *>(pose->chanbase.first); pchan != nullptr;
@@ -947,6 +944,9 @@ static void pose_channel_flush_to_orig_if_needed(Depsgraph *depsgraph,
 
 void BKE_pose_bone_done(Depsgraph *depsgraph, Object *object, int pchan_index)
 {
+  /* Note: tests in `armature_deform_test.cc` update pose matrices locally to avoid creating a full
+   * depsgraph. Keep these in sync if this function is changed! */
+
   const bArmature *armature = (bArmature *)object->data;
   if (armature->edbo != nullptr) {
     return;
