@@ -12,7 +12,7 @@
 #include "DNA_scene_types.h"
 
 #include "BLI_path_utils.hh"
-#include "BLI_string.h"
+#include "BLI_string_utf8.h"
 #include "BLI_utildefines.h"
 
 #include "IMB_colormanagement.hh"
@@ -22,22 +22,19 @@
 
 #include "BKE_colortools.hh"
 #include "BKE_image_format.hh"
+#include "BKE_path_templates.hh"
+
+namespace path_templates = blender::bke::path_templates;
 
 /* Init/Copy/Free */
 
-void BKE_image_format_init(ImageFormatData *imf, const bool render)
+void BKE_image_format_init(ImageFormatData *imf)
 {
   *imf = *DNA_struct_default_get(ImageFormatData);
 
   BKE_color_managed_display_settings_init(&imf->display_settings);
 
-  if (render) {
-    BKE_color_managed_view_settings_init_render(
-        &imf->view_settings, &imf->display_settings, "Filmic");
-  }
-  else {
-    BKE_color_managed_view_settings_init_default(&imf->view_settings, &imf->display_settings);
-  }
+  BKE_color_managed_view_settings_init(&imf->view_settings, &imf->display_settings, "AgX");
 
   BKE_color_managed_colorspace_settings_init(&imf->linear_colorspace_settings);
 }
@@ -73,7 +70,7 @@ void BKE_image_format_update_color_space_for_type(ImageFormatData *format)
   if (format->linear_colorspace_settings.name[0] == '\0' || image_requires_linear != is_linear) {
     const int role = image_requires_linear ? COLOR_ROLE_DEFAULT_FLOAT : COLOR_ROLE_DEFAULT_BYTE;
     const char *default_color_space = IMB_colormanagement_role_colorspace_name_get(role);
-    STRNCPY(format->linear_colorspace_settings.name, default_color_space);
+    STRNCPY_UTF8(format->linear_colorspace_settings.name, default_color_space);
   }
 }
 
@@ -87,13 +84,56 @@ void BKE_image_format_blend_write(BlendWriter *writer, ImageFormatData *imf)
   BKE_color_managed_view_settings_blend_write(writer, &imf->view_settings);
 }
 
+void BKE_image_format_media_type_set(ImageFormatData *format,
+                                     ID *owner_id,
+                                     const MediaType media_type)
+{
+  format->media_type = media_type;
+
+  switch (media_type) {
+    case MEDIA_TYPE_IMAGE:
+      if (!BKE_imtype_is_image(format->imtype)) {
+        BKE_image_format_set(format, owner_id, R_IMF_IMTYPE_PNG);
+      }
+      break;
+    case MEDIA_TYPE_MULTI_LAYER_IMAGE:
+      if (!BKE_imtype_is_multi_layer_image(format->imtype)) {
+        BKE_image_format_set(format, owner_id, R_IMF_IMTYPE_MULTILAYER);
+      }
+      break;
+    case MEDIA_TYPE_VIDEO:
+      if (!BKE_imtype_is_movie(format->imtype)) {
+        BKE_image_format_set(format, owner_id, R_IMF_IMTYPE_FFMPEG);
+      }
+      break;
+  }
+}
+
 void BKE_image_format_set(ImageFormatData *imf, ID *owner_id, const char imtype)
 {
   imf->imtype = imtype;
 
+  /* Update media type in case it doesn't match the new imtype. Note that normally, one would use
+   * the BKE_image_format_media_type_set function to set the media type, but that function itself
+   * calls this function to update the imtype, and while this wouldn't case recursion since the
+   * imtype is already conforming, it is better to err on the side of caution and set the media
+   * type manually. */
+  if (BKE_imtype_is_image(imf->imtype)) {
+    imf->media_type = MEDIA_TYPE_IMAGE;
+  }
+  else if (BKE_imtype_is_multi_layer_image(imf->imtype)) {
+    imf->media_type = MEDIA_TYPE_MULTI_LAYER_IMAGE;
+  }
+  else if (BKE_imtype_is_movie(imf->imtype)) {
+    imf->media_type = MEDIA_TYPE_VIDEO;
+  }
+  else {
+    BLI_assert_unreachable();
+  }
+
   const bool is_render = (owner_id && GS(owner_id->name) == ID_SCE);
   /* see note below on why this is */
-  const char chan_flag = BKE_imtype_valid_channels(imf->imtype, true) |
+  const char chan_flag = BKE_imtype_valid_channels(imf->imtype) |
                          (is_render ? IMA_CHAN_FLAG_BW : 0);
 
   /* ensure depth and color settings match */
@@ -118,6 +158,18 @@ void BKE_image_format_set(ImageFormatData *imf, ID *owner_id, const char imtype)
     MOV_validate_output_settings(rd, imf);
   }
 
+  /* Verify `imf->views_format`. */
+  if (imf->imtype == R_IMF_IMTYPE_MULTILAYER) {
+    if (imf->views_format == R_IMF_VIEWS_STEREO_3D) {
+      imf->views_format = R_IMF_VIEWS_MULTIVIEW;
+    }
+  }
+  else if (imf->imtype != R_IMF_IMTYPE_OPENEXR) {
+    if (imf->views_format == R_IMF_VIEWS_MULTIVIEW) {
+      imf->views_format = R_IMF_VIEWS_INDIVIDUAL;
+    }
+  }
+
   BKE_image_format_update_color_space_for_type(imf);
 }
 
@@ -135,7 +187,7 @@ int BKE_imtype_to_ftype(const char imtype, ImbFormatOptions *r_options)
     return IMB_FTYPE_TGA;
   }
   if (imtype == R_IMF_IMTYPE_IRIS) {
-    return IMB_FTYPE_IMAGIC;
+    return IMB_FTYPE_IRIS;
   }
   if (imtype == R_IMF_IMTYPE_RADHDR) {
     return IMB_FTYPE_RADHDR;
@@ -157,7 +209,7 @@ int BKE_imtype_to_ftype(const char imtype, ImbFormatOptions *r_options)
     r_options->quality = 90;
     return IMB_FTYPE_OPENEXR;
   }
-#ifdef WITH_CINEON
+#ifdef WITH_IMAGE_CINEON
   if (imtype == R_IMF_IMTYPE_CINEON) {
     return IMB_FTYPE_CINEON;
   }
@@ -165,14 +217,14 @@ int BKE_imtype_to_ftype(const char imtype, ImbFormatOptions *r_options)
     return IMB_FTYPE_DPX;
   }
 #endif
-#ifdef WITH_OPENJPEG
+#ifdef WITH_IMAGE_OPENJPEG
   if (imtype == R_IMF_IMTYPE_JP2) {
     r_options->flag |= JP2_JP2;
     r_options->quality = 90;
     return IMB_FTYPE_JP2;
   }
 #endif
-#ifdef WITH_WEBP
+#ifdef WITH_IMAGE_WEBP
   if (imtype == R_IMF_IMTYPE_WEBP) {
     r_options->quality = 90;
     return IMB_FTYPE_WEBP;
@@ -188,7 +240,7 @@ char BKE_ftype_to_imtype(const int ftype, const ImbFormatOptions *options)
   if (ftype == IMB_FTYPE_NONE) {
     return R_IMF_IMTYPE_TARGA;
   }
-  if (ftype == IMB_FTYPE_IMAGIC) {
+  if (ftype == IMB_FTYPE_IRIS) {
     return R_IMF_IMTYPE_IRIS;
   }
   if (ftype == IMB_FTYPE_RADHDR) {
@@ -209,7 +261,7 @@ char BKE_ftype_to_imtype(const int ftype, const ImbFormatOptions *options)
   if (ftype == IMB_FTYPE_OPENEXR) {
     return R_IMF_IMTYPE_OPENEXR;
   }
-#ifdef WITH_CINEON
+#ifdef WITH_IMAGE_CINEON
   if (ftype == IMB_FTYPE_CINEON) {
     return R_IMF_IMTYPE_CINEON;
   }
@@ -224,12 +276,12 @@ char BKE_ftype_to_imtype(const int ftype, const ImbFormatOptions *options)
 
     return R_IMF_IMTYPE_TARGA;
   }
-#ifdef WITH_OPENJPEG
+#ifdef WITH_IMAGE_OPENJPEG
   if (ftype == IMB_FTYPE_JP2) {
     return R_IMF_IMTYPE_JP2;
   }
 #endif
-#ifdef WITH_WEBP
+#ifdef WITH_IMAGE_WEBP
   if (ftype == IMB_FTYPE_WEBP) {
     return R_IMF_IMTYPE_WEBP;
   }
@@ -238,19 +290,19 @@ char BKE_ftype_to_imtype(const int ftype, const ImbFormatOptions *options)
   return R_IMF_IMTYPE_JPEG90;
 }
 
+bool BKE_imtype_is_image(const char imtype)
+{
+  return !BKE_imtype_is_multi_layer_image(imtype) && !BKE_imtype_is_movie(imtype);
+}
+
+bool BKE_imtype_is_multi_layer_image(const char imtype)
+{
+  return imtype == R_IMF_IMTYPE_MULTILAYER;
+}
+
 bool BKE_imtype_is_movie(const char imtype)
 {
-  switch (imtype) {
-    case R_IMF_IMTYPE_AVIRAW:
-    case R_IMF_IMTYPE_AVIJPEG:
-    case R_IMF_IMTYPE_FFMPEG:
-    case R_IMF_IMTYPE_H264:
-    case R_IMF_IMTYPE_THEORA:
-    case R_IMF_IMTYPE_XVID:
-    case R_IMF_IMTYPE_AV1:
-      return true;
-  }
-  return false;
+  return imtype == R_IMF_IMTYPE_FFMPEG;
 }
 
 bool BKE_imtype_supports_compress(const char imtype)
@@ -267,7 +319,6 @@ bool BKE_imtype_supports_quality(const char imtype)
   switch (imtype) {
     case R_IMF_IMTYPE_JPEG90:
     case R_IMF_IMTYPE_JP2:
-    case R_IMF_IMTYPE_AVIJPEG:
     case R_IMF_IMTYPE_WEBP:
       return true;
   }
@@ -287,17 +338,13 @@ bool BKE_imtype_requires_linear_float(const char imtype)
   return false;
 }
 
-char BKE_imtype_valid_channels(const char imtype, bool write_file)
+char BKE_imtype_valid_channels(const char imtype)
 {
   char chan_flag = IMA_CHAN_FLAG_RGB; /* Assume all support RGB. */
 
   /* Alpha. */
   switch (imtype) {
     case R_IMF_IMTYPE_BMP:
-      if (write_file) {
-        break;
-      }
-      ATTR_FALLTHROUGH;
     case R_IMF_IMTYPE_TARGA:
     case R_IMF_IMTYPE_RAWTGA:
     case R_IMF_IMTYPE_IRIS:
@@ -366,7 +413,7 @@ char BKE_imtype_valid_depths_with_video(char imtype, const ID *owner_id)
     const bool is_render_out = (owner_id && GS(owner_id->name) == ID_SCE);
     if (is_render_out) {
       const Scene *scene = (const Scene *)owner_id;
-      depths |= MOV_codec_valid_bit_depths(scene->r.ffcodecdata.codec);
+      depths |= MOV_codec_valid_bit_depths(scene->r.ffcodecdata.codec_id_get());
     }
   }
   return depths;
@@ -407,12 +454,6 @@ char BKE_imtype_from_arg(const char *imtype_arg)
   if (STREQ(imtype_arg, "RAWTGA")) {
     return R_IMF_IMTYPE_RAWTGA;
   }
-  if (STREQ(imtype_arg, "AVIRAW")) {
-    return R_IMF_IMTYPE_AVIRAW;
-  }
-  if (STREQ(imtype_arg, "AVIJPEG")) {
-    return R_IMF_IMTYPE_AVIJPEG;
-  }
   if (STREQ(imtype_arg, "PNG")) {
     return R_IMF_IMTYPE_PNG;
   }
@@ -425,7 +466,7 @@ char BKE_imtype_from_arg(const char *imtype_arg)
   if (STREQ(imtype_arg, "TIFF")) {
     return R_IMF_IMTYPE_TIFF;
   }
-#ifdef WITH_OPENEXR
+#ifdef WITH_IMAGE_OPENEXR
   if (STREQ(imtype_arg, "OPEN_EXR")) {
     return R_IMF_IMTYPE_OPENEXR;
   }
@@ -444,7 +485,7 @@ char BKE_imtype_from_arg(const char *imtype_arg)
     return R_IMF_IMTYPE_FFMPEG;
   }
 #endif
-#ifdef WITH_CINEON
+#ifdef WITH_IMAGE_CINEON
   if (STREQ(imtype_arg, "CINEON")) {
     return R_IMF_IMTYPE_CINEON;
   }
@@ -452,12 +493,12 @@ char BKE_imtype_from_arg(const char *imtype_arg)
     return R_IMF_IMTYPE_DPX;
   }
 #endif
-#ifdef WITH_OPENJPEG
+#ifdef WITH_IMAGE_OPENJPEG
   if (STREQ(imtype_arg, "JP2")) {
     return R_IMF_IMTYPE_JP2;
   }
 #endif
-#ifdef WITH_WEBP
+#ifdef WITH_IMAGE_WEBP
   if (STREQ(imtype_arg, "WEBP")) {
     return R_IMF_IMTYPE_WEBP;
   }
@@ -484,14 +525,7 @@ static int image_path_ext_from_imformat_impl(const char imtype,
   else if (imtype == R_IMF_IMTYPE_RADHDR) {
     r_ext[ext_num++] = ".hdr";
   }
-  else if (ELEM(imtype,
-                R_IMF_IMTYPE_PNG,
-                R_IMF_IMTYPE_FFMPEG,
-                R_IMF_IMTYPE_H264,
-                R_IMF_IMTYPE_THEORA,
-                R_IMF_IMTYPE_XVID,
-                R_IMF_IMTYPE_AV1))
-  {
+  else if (ELEM(imtype, R_IMF_IMTYPE_PNG, R_IMF_IMTYPE_FFMPEG)) {
     r_ext[ext_num++] = ".png";
   }
   else if (imtype == R_IMF_IMTYPE_DDS) {
@@ -510,12 +544,12 @@ static int image_path_ext_from_imformat_impl(const char imtype,
   else if (imtype == R_IMF_IMTYPE_PSD) {
     r_ext[ext_num++] = ".psd";
   }
-#ifdef WITH_OPENEXR
+#ifdef WITH_IMAGE_OPENEXR
   else if (ELEM(imtype, R_IMF_IMTYPE_OPENEXR, R_IMF_IMTYPE_MULTILAYER)) {
     r_ext[ext_num++] = ".exr";
   }
 #endif
-#ifdef WITH_CINEON
+#ifdef WITH_IMAGE_CINEON
   else if (imtype == R_IMF_IMTYPE_CINEON) {
     r_ext[ext_num++] = ".cin";
   }
@@ -523,7 +557,7 @@ static int image_path_ext_from_imformat_impl(const char imtype,
     r_ext[ext_num++] = ".dpx";
   }
 #endif
-#ifdef WITH_OPENJPEG
+#ifdef WITH_IMAGE_OPENJPEG
   else if (imtype == R_IMF_IMTYPE_JP2) {
     if (im_format) {
       if (im_format->jp2_codec == R_IMF_JP2_CODEC_JP2) {
@@ -541,13 +575,13 @@ static int image_path_ext_from_imformat_impl(const char imtype,
     }
   }
 #endif
-#ifdef WITH_WEBP
+#ifdef WITH_IMAGE_WEBP
   else if (imtype == R_IMF_IMTYPE_WEBP) {
     r_ext[ext_num++] = ".webp";
   }
 #endif
   else {
-    /* Handles: #R_IMF_IMTYPE_AVIRAW, #R_IMF_IMTYPE_AVIJPEG, #R_IMF_IMTYPE_JPEG90 etc. */
+    /* Handles: #R_IMF_IMTYPE_JPEG90 etc. */
     r_ext[ext_num++] = ".jpg";
     r_ext[ext_num++] = ".jpeg";
   }
@@ -601,20 +635,31 @@ int BKE_image_path_ext_from_imtype_ensure(char *filepath,
   return do_ensure_image_extension(filepath, filepath_maxncpy, imtype, nullptr);
 }
 
-static void do_makepicstring(char filepath[FILE_MAX],
-                             const char *base,
-                             const char *relbase,
-                             int frame,
-                             const char imtype,
-                             const ImageFormatData *im_format,
-                             const bool use_ext,
-                             const bool use_frames,
-                             const char *suffix)
+static blender::Vector<path_templates::Error> do_makepicstring(
+    char filepath[FILE_MAX],
+    const char *base,
+    const char *relbase,
+    const path_templates::VariableMap *template_variables,
+    int frame,
+    const char imtype,
+    const ImageFormatData *im_format,
+    const bool use_ext,
+    const bool use_frames,
+    const char *suffix)
 {
   if (filepath == nullptr) {
-    return;
+    return {};
   }
   BLI_strncpy(filepath, base, FILE_MAX - 10); /* weak assumption */
+
+  if (template_variables) {
+    const blender::Vector<path_templates::Error> variable_errors = BKE_path_apply_template(
+        filepath, FILE_MAX, *template_variables);
+    if (!variable_errors.is_empty()) {
+      return variable_errors;
+    }
+  }
+
   BLI_path_abs(filepath, relbase);
 
   if (use_frames) {
@@ -628,31 +673,54 @@ static void do_makepicstring(char filepath[FILE_MAX],
   if (use_ext) {
     do_ensure_image_extension(filepath, FILE_MAX, imtype, im_format);
   }
+
+  return {};
 }
 
-void BKE_image_path_from_imformat(char *filepath,
-                                  const char *base,
-                                  const char *relbase,
-                                  int frame,
-                                  const ImageFormatData *im_format,
-                                  const bool use_ext,
-                                  const bool use_frames,
-                                  const char *suffix)
+blender::Vector<path_templates::Error> BKE_image_path_from_imformat(
+    char *filepath,
+    const char *base,
+    const char *relbase,
+    const path_templates::VariableMap *template_variables,
+    int frame,
+    const ImageFormatData *im_format,
+    const bool use_ext,
+    const bool use_frames,
+    const char *suffix)
 {
-  do_makepicstring(
-      filepath, base, relbase, frame, im_format->imtype, im_format, use_ext, use_frames, suffix);
+  return do_makepicstring(filepath,
+                          base,
+                          relbase,
+                          template_variables,
+                          frame,
+                          im_format->imtype,
+                          im_format,
+                          use_ext,
+                          use_frames,
+                          suffix);
 }
 
-void BKE_image_path_from_imtype(char *filepath,
-                                const char *base,
-                                const char *relbase,
-                                int frame,
-                                const char imtype,
-                                const bool use_ext,
-                                const bool use_frames,
-                                const char *suffix)
+blender::Vector<path_templates::Error> BKE_image_path_from_imtype(
+    char *filepath,
+    const char *base,
+    const char *relbase,
+    const path_templates::VariableMap *template_variables,
+    int frame,
+    const char imtype,
+    const bool use_ext,
+    const bool use_frames,
+    const char *suffix)
 {
-  do_makepicstring(filepath, base, relbase, frame, imtype, nullptr, use_ext, use_frames, suffix);
+  return do_makepicstring(filepath,
+                          base,
+                          relbase,
+                          template_variables,
+                          frame,
+                          imtype,
+                          nullptr,
+                          use_ext,
+                          use_frames,
+                          suffix);
 }
 
 /* ImBuf Conversion */
@@ -668,19 +736,12 @@ void BKE_image_format_to_imbuf(ImBuf *ibuf, const ImageFormatData *imf)
   ibuf->foptions.flag = 0;
 
   if (imtype == R_IMF_IMTYPE_IRIS) {
-    ibuf->ftype = IMB_FTYPE_IMAGIC;
+    ibuf->ftype = IMB_FTYPE_IRIS;
   }
   else if (imtype == R_IMF_IMTYPE_RADHDR) {
     ibuf->ftype = IMB_FTYPE_RADHDR;
   }
-  else if (ELEM(imtype,
-                R_IMF_IMTYPE_PNG,
-                R_IMF_IMTYPE_FFMPEG,
-                R_IMF_IMTYPE_H264,
-                R_IMF_IMTYPE_THEORA,
-                R_IMF_IMTYPE_XVID,
-                R_IMF_IMTYPE_AV1))
-  {
+  else if (ELEM(imtype, R_IMF_IMTYPE_PNG, R_IMF_IMTYPE_FFMPEG)) {
     ibuf->ftype = IMB_FTYPE_PNG;
 
     if (imtype == R_IMF_IMTYPE_PNG) {
@@ -716,7 +777,7 @@ void BKE_image_format_to_imbuf(ImBuf *ibuf, const ImageFormatData *imf)
       ibuf->foptions.flag |= TIF_COMPRESS_PACKBITS;
     }
   }
-#ifdef WITH_OPENEXR
+#ifdef WITH_IMAGE_OPENEXR
   else if (ELEM(imtype, R_IMF_IMTYPE_OPENEXR, R_IMF_IMTYPE_MULTILAYER)) {
     ibuf->ftype = IMB_FTYPE_OPENEXR;
     if (imf->depth == R_IMF_CHAN_DEPTH_16) {
@@ -724,9 +785,12 @@ void BKE_image_format_to_imbuf(ImBuf *ibuf, const ImageFormatData *imf)
     }
     ibuf->foptions.flag |= (imf->exr_codec & OPENEXR_CODEC_MASK);
     ibuf->foptions.quality = quality;
+    if (imf->exr_flag & R_IMF_EXR_FLAG_MULTIPART) {
+      ibuf->foptions.flag |= OPENEXR_MULTIPART;
+    }
   }
 #endif
-#ifdef WITH_CINEON
+#ifdef WITH_IMAGE_CINEON
   else if (imtype == R_IMF_IMTYPE_CINEON) {
     ibuf->ftype = IMB_FTYPE_CINEON;
     if (imf->cineon_flag & R_IMF_CINEON_FLAG_LOG) {
@@ -765,7 +829,7 @@ void BKE_image_format_to_imbuf(ImBuf *ibuf, const ImageFormatData *imf)
     ibuf->ftype = IMB_FTYPE_TGA;
     ibuf->foptions.flag = RAWTGA;
   }
-#ifdef WITH_OPENJPEG
+#ifdef WITH_IMAGE_OPENJPEG
   else if (imtype == R_IMF_IMTYPE_JP2) {
     if (quality < 10) {
       quality = 90;
@@ -802,7 +866,7 @@ void BKE_image_format_to_imbuf(ImBuf *ibuf, const ImageFormatData *imf)
     }
   }
 #endif
-#ifdef WITH_WEBP
+#ifdef WITH_IMAGE_WEBP
   else if (imtype == R_IMF_IMTYPE_WEBP) {
     ibuf->ftype = IMB_FTYPE_WEBP;
     ibuf->foptions.quality = quality;
@@ -818,17 +882,57 @@ void BKE_image_format_to_imbuf(ImBuf *ibuf, const ImageFormatData *imf)
   }
 }
 
+static char imtype_best_depth(const ImBuf *ibuf, const char imtype)
+{
+  const char depth_ok = BKE_imtype_valid_depths(imtype);
+
+  if (ibuf->float_buffer.data) {
+    if (depth_ok & R_IMF_CHAN_DEPTH_32) {
+      return R_IMF_CHAN_DEPTH_32;
+    }
+    if (depth_ok & R_IMF_CHAN_DEPTH_24) {
+      return R_IMF_CHAN_DEPTH_24;
+    }
+    if (depth_ok & R_IMF_CHAN_DEPTH_16) {
+      return R_IMF_CHAN_DEPTH_16;
+    }
+    if (depth_ok & R_IMF_CHAN_DEPTH_12) {
+      return R_IMF_CHAN_DEPTH_12;
+    }
+    return R_IMF_CHAN_DEPTH_8;
+  }
+
+  if (depth_ok & R_IMF_CHAN_DEPTH_8) {
+    return R_IMF_CHAN_DEPTH_8;
+  }
+  if (depth_ok & R_IMF_CHAN_DEPTH_12) {
+    return R_IMF_CHAN_DEPTH_12;
+  }
+  if (depth_ok & R_IMF_CHAN_DEPTH_16) {
+    return R_IMF_CHAN_DEPTH_16;
+  }
+  if (depth_ok & R_IMF_CHAN_DEPTH_24) {
+    return R_IMF_CHAN_DEPTH_24;
+  }
+  if (depth_ok & R_IMF_CHAN_DEPTH_32) {
+    return R_IMF_CHAN_DEPTH_32;
+  }
+  return R_IMF_CHAN_DEPTH_8; /* fallback, should not get here */
+}
+
 void BKE_image_format_from_imbuf(ImageFormatData *im_format, const ImBuf *imbuf)
 {
   /* Read from ImBuf after file read. */
   int ftype = imbuf->ftype;
   int custom_flags = imbuf->foptions.flag;
   char quality = imbuf->foptions.quality;
+  bool is_depth_set = false;
 
-  BKE_image_format_init(im_format, false);
+  BKE_image_format_init(im_format);
+  im_format->media_type = MEDIA_TYPE_IMAGE;
 
   /* file type */
-  if (ftype == IMB_FTYPE_IMAGIC) {
+  if (ftype == IMB_FTYPE_IRIS) {
     im_format->imtype = R_IMF_IMTYPE_IRIS;
   }
   else if (ftype == IMB_FTYPE_RADHDR) {
@@ -839,6 +943,7 @@ void BKE_image_format_from_imbuf(ImageFormatData *im_format, const ImBuf *imbuf)
 
     if (custom_flags & PNG_16BIT) {
       im_format->depth = R_IMF_CHAN_DEPTH_16;
+      is_depth_set = true;
     }
 
     im_format->compress = quality;
@@ -853,6 +958,7 @@ void BKE_image_format_from_imbuf(ImageFormatData *im_format, const ImBuf *imbuf)
     im_format->imtype = R_IMF_IMTYPE_TIFF;
     if (custom_flags & TIF_16BIT) {
       im_format->depth = R_IMF_CHAN_DEPTH_16;
+      is_depth_set = true;
     }
     if (custom_flags & TIF_COMPRESS_NONE) {
       im_format->tiff_codec = R_IMF_TIFF_CODEC_NONE;
@@ -868,19 +974,28 @@ void BKE_image_format_from_imbuf(ImageFormatData *im_format, const ImBuf *imbuf)
     }
   }
 
-#ifdef WITH_OPENEXR
+#ifdef WITH_IMAGE_OPENEXR
   else if (ftype == IMB_FTYPE_OPENEXR) {
     im_format->imtype = R_IMF_IMTYPE_OPENEXR;
+    char exr_codec = custom_flags & OPENEXR_CODEC_MASK;
     if (custom_flags & OPENEXR_HALF) {
       im_format->depth = R_IMF_CHAN_DEPTH_16;
+      is_depth_set = true;
     }
-    if (custom_flags & OPENEXR_CODEC_MASK) {
-      im_format->exr_codec = R_IMF_EXR_CODEC_ZIP; /* Can't determine compression */
+    else if (ELEM(exr_codec, R_IMF_EXR_CODEC_B44, R_IMF_EXR_CODEC_B44A)) {
+      /* B44 and B44A are only selectable for half precision images, default to ZIP compression */
+      exr_codec = R_IMF_EXR_CODEC_ZIP;
+    }
+    if (exr_codec < R_IMF_EXR_CODEC_MAX) {
+      im_format->exr_codec = exr_codec;
+    }
+    if (custom_flags & OPENEXR_MULTIPART) {
+      im_format->exr_flag |= R_IMF_EXR_FLAG_MULTIPART;
     }
   }
 #endif
 
-#ifdef WITH_CINEON
+#ifdef WITH_IMAGE_CINEON
   else if (ftype == IMB_FTYPE_CINEON) {
     im_format->imtype = R_IMF_IMTYPE_CINEON;
   }
@@ -896,16 +1011,18 @@ void BKE_image_format_from_imbuf(ImageFormatData *im_format, const ImBuf *imbuf)
       im_format->imtype = R_IMF_IMTYPE_TARGA;
     }
   }
-#ifdef WITH_OPENJPEG
+#ifdef WITH_IMAGE_OPENJPEG
   else if (ftype == IMB_FTYPE_JP2) {
     im_format->imtype = R_IMF_IMTYPE_JP2;
     im_format->quality = quality;
 
     if (custom_flags & JP2_16BIT) {
       im_format->depth = R_IMF_CHAN_DEPTH_16;
+      is_depth_set = true;
     }
     else if (custom_flags & JP2_12BIT) {
       im_format->depth = R_IMF_CHAN_DEPTH_12;
+      is_depth_set = true;
     }
 
     if (custom_flags & JP2_YCC) {
@@ -930,7 +1047,7 @@ void BKE_image_format_from_imbuf(ImageFormatData *im_format, const ImBuf *imbuf)
     }
   }
 #endif
-#ifdef WITH_WEBP
+#ifdef WITH_IMAGE_WEBP
   else if (ftype == IMB_FTYPE_WEBP) {
     im_format->imtype = R_IMF_IMTYPE_WEBP;
     im_format->quality = quality;
@@ -942,6 +1059,11 @@ void BKE_image_format_from_imbuf(ImageFormatData *im_format, const ImBuf *imbuf)
     im_format->quality = quality;
   }
 
+  /* Default depth, accounting for float buffer and format support */
+  if (!is_depth_set) {
+    im_format->depth = imtype_best_depth(imbuf, im_format->imtype);
+  }
+
   /* planes */
   im_format->planes = imbuf->planes;
 }
@@ -969,18 +1091,26 @@ void BKE_image_format_color_management_copy_from_scene(ImageFormatData *imf, con
 
   BKE_color_managed_display_settings_copy(&imf->display_settings, &scene->display_settings);
   BKE_color_managed_view_settings_copy(&imf->view_settings, &scene->view_settings);
-  STRNCPY(imf->linear_colorspace_settings.name,
-          IMB_colormanagement_role_colorspace_name_get(COLOR_ROLE_SCENE_LINEAR));
+  STRNCPY_UTF8(imf->linear_colorspace_settings.name,
+               IMB_colormanagement_role_colorspace_name_get(COLOR_ROLE_SCENE_LINEAR));
 }
 
 /* Output */
 
 void BKE_image_format_init_for_write(ImageFormatData *imf,
                                      const Scene *scene_src,
-                                     const ImageFormatData *imf_src)
+                                     const ImageFormatData *imf_src,
+                                     const bool allow_video)
 {
   *imf = (imf_src) ? *imf_src : scene_src->r.im_format;
 
+  /* For image saving we can not have use media type video. */
+  if (!allow_video) {
+    if (scene_src && imf->media_type == MEDIA_TYPE_VIDEO) {
+      BKE_image_format_media_type_set(imf, const_cast<ID *>(&scene_src->id), MEDIA_TYPE_IMAGE);
+    }
+  }
+
   if (imf_src && imf_src->color_management == R_IMF_COLOR_MANAGEMENT_OVERRIDE) {
     /* Use settings specific to one node, image save operation, etc. */
     BKE_color_managed_display_settings_copy(&imf->display_settings, &imf_src->display_settings);
@@ -1001,7 +1131,7 @@ void BKE_image_format_init_for_write(ImageFormatData *imf,
     /* Use general scene settings also used for display. */
     BKE_color_managed_display_settings_copy(&imf->display_settings, &scene_src->display_settings);
     BKE_color_managed_view_settings_copy(&imf->view_settings, &scene_src->view_settings);
-    STRNCPY(imf->linear_colorspace_settings.name,
-            IMB_colormanagement_role_colorspace_name_get(COLOR_ROLE_SCENE_LINEAR));
+    STRNCPY_UTF8(imf->linear_colorspace_settings.name,
+                 IMB_colormanagement_role_colorspace_name_get(COLOR_ROLE_SCENE_LINEAR));
   }
 }
