@@ -19,7 +19,7 @@
 #include "BLI_math_geom.h"
 #include "BLI_math_matrix.h"
 #include "BLI_math_vector.h"
-#include "BLI_string.h"
+#include "BLI_string_utf8.h"
 #include "BLI_string_utils.hh"
 #include "BLI_utildefines.h"
 
@@ -27,6 +27,8 @@
 
 #include "DNA_defaults.h"
 #include "DNA_mask_types.h"
+#include "DNA_movieclip_types.h"
+#include "DNA_object_types.h"
 
 #include "BKE_animsys.h"
 #include "BKE_curve.hh"
@@ -47,7 +49,13 @@
 
 #include "BLO_read_write.hh"
 
-static CLG_LogRef LOG = {"bke.mask"};
+static CLG_LogRef LOG = {"mask"};
+
+/** Reset runtime mask fields when data-block is being initialized. */
+static void mask_runtime_reset(Mask *mask)
+{
+  mask->runtime.last_update = 0;
+}
 
 static void mask_copy_data(Main * /*bmain*/,
                            std::optional<Library *> /*owner_library*/,
@@ -63,8 +71,6 @@ static void mask_copy_data(Main * /*bmain*/,
   /* TODO: add unused flag to those as well. */
   BKE_mask_layer_copy_list(&mask_dst->masklayers, &mask_src->masklayers);
 
-  BLI_listbase_clear((ListBase *)&mask_dst->drawdata);
-
   /* enable fake user by default */
   id_fake_user_set(&mask_dst->id);
 }
@@ -75,8 +81,6 @@ static void mask_free_data(ID *id)
 
   /* free mask data */
   BKE_mask_layer_free_list(&mask->masklayers);
-
-  DRW_drawdata_free(id);
 }
 
 static void mask_foreach_id(ID *id, LibraryForeachIDData *data)
@@ -175,10 +179,12 @@ static void mask_blend_read_data(BlendDataReader *reader, ID *id)
     BLO_read_struct(reader, MaskSpline, &masklay->act_spline);
     masklay->act_point = act_point_search;
   }
+
+  mask_runtime_reset(mask);
 }
 
 IDTypeInfo IDType_ID_MSK = {
-    /*id_code*/ ID_MSK,
+    /*id_code*/ Mask::id_type,
     /*id_filter*/ FILTER_ID_MSK,
     /*dependencies_id_types*/ FILTER_ID_MC, /* WARNING! mask->parent.id, not typed. */
     /*main_listbase_index*/ INDEX_ID_MSK,
@@ -196,6 +202,7 @@ IDTypeInfo IDType_ID_MSK = {
     /*foreach_id*/ mask_foreach_id,
     /*foreach_cache*/ nullptr,
     /*foreach_path*/ nullptr,
+    /*foreach_working_space_color*/ nullptr,
     /*owner_pointer_get*/ nullptr,
 
     /*blend_write*/ mask_blend_write,
@@ -283,9 +290,9 @@ MaskSplinePoint *BKE_mask_spline_point_array_from_point(MaskSpline *spline,
 
 MaskLayer *BKE_mask_layer_new(Mask *mask, const char *name)
 {
-  MaskLayer *masklay = MEM_cnew<MaskLayer>(__func__);
+  MaskLayer *masklay = MEM_callocN<MaskLayer>(__func__);
 
-  STRNCPY(masklay->name, name && name[0] ? name : DATA_("MaskLayer"));
+  STRNCPY_UTF8(masklay->name, name && name[0] ? name : DATA_("MaskLayer"));
 
   BLI_addtail(&mask->masklayers, masklay);
 
@@ -337,7 +344,7 @@ void BKE_mask_layer_rename(Mask *mask,
                            const char *oldname,
                            const char *newname)
 {
-  STRNCPY(masklay->name, newname);
+  STRNCPY_UTF8(masklay->name, newname);
 
   BKE_mask_layer_unique_name(mask, masklay);
 
@@ -347,9 +354,9 @@ void BKE_mask_layer_rename(Mask *mask,
 
 MaskLayer *BKE_mask_layer_copy(const MaskLayer *masklay)
 {
-  MaskLayer *masklay_new = MEM_cnew<MaskLayer>("new mask layer");
+  MaskLayer *masklay_new = MEM_callocN<MaskLayer>("new mask layer");
 
-  STRNCPY(masklay_new->name, masklay->name);
+  STRNCPY_UTF8(masklay_new->name, masklay->name);
 
   masklay_new->alpha = masklay->alpha;
   masklay_new->blend = masklay->blend;
@@ -378,7 +385,7 @@ MaskLayer *BKE_mask_layer_copy(const MaskLayer *masklay)
   /* correct animation */
   if (masklay->splines_shapes.first) {
     LISTBASE_FOREACH (MaskLayerShape *, masklay_shape, &masklay->splines_shapes) {
-      MaskLayerShape *masklay_shape_new = MEM_cnew<MaskLayerShape>("new mask layer shape");
+      MaskLayerShape *masklay_shape_new = MEM_callocN<MaskLayerShape>("new mask layer shape");
 
       masklay_shape_new->data = static_cast<float *>(MEM_dupallocN(masklay_shape->data));
       masklay_shape_new->tot_vert = masklay_shape->tot_vert;
@@ -405,12 +412,12 @@ void BKE_mask_layer_copy_list(ListBase *masklayers_new, const ListBase *masklaye
 
 MaskSpline *BKE_mask_spline_add(MaskLayer *masklay)
 {
-  MaskSpline *spline = MEM_cnew<MaskSpline>("new mask spline");
+  MaskSpline *spline = MEM_callocN<MaskSpline>("new mask spline");
 
   BLI_addtail(&masklay->splines, spline);
 
   /* spline shall have one point at least */
-  spline->points = MEM_cnew<MaskSplinePoint>("new mask spline point");
+  spline->points = MEM_callocN<MaskSplinePoint>("new mask spline point");
   spline->tot_point = 1;
 
   /* cyclic shapes are more usually used */
@@ -872,7 +879,7 @@ MaskSplinePointUW *BKE_mask_point_sort_uw(MaskSplinePoint *point, MaskSplinePoin
 void BKE_mask_point_add_uw(MaskSplinePoint *point, float u, float w)
 {
   if (!point->uw) {
-    point->uw = MEM_cnew<MaskSplinePointUW>("mask point uw");
+    point->uw = MEM_callocN<MaskSplinePointUW>("mask point uw");
   }
   else {
     point->uw = static_cast<MaskSplinePointUW *>(
@@ -958,7 +965,7 @@ Mask *BKE_mask_new(Main *bmain, const char *name)
   Mask *mask;
   char mask_name[MAX_ID_NAME - 2];
 
-  STRNCPY(mask_name, (name && name[0]) ? name : DATA_("Mask"));
+  STRNCPY_UTF8(mask_name, (name && name[0]) ? name : DATA_("Mask"));
 
   mask = mask_alloc(bmain, mask_name);
 
@@ -1032,7 +1039,7 @@ static MaskSplinePoint *mask_spline_points_copy(const MaskSplinePoint *points, i
 
 MaskSpline *BKE_mask_spline_copy(const MaskSpline *spline)
 {
-  MaskSpline *nspline = MEM_cnew<MaskSpline>("new spline");
+  MaskSpline *nspline = MEM_callocN<MaskSpline>("new spline");
 
   *nspline = *spline;
 
@@ -1051,10 +1058,10 @@ MaskLayerShape *BKE_mask_layer_shape_alloc(MaskLayer *masklay, const int frame)
   MaskLayerShape *masklay_shape;
   int tot_vert = BKE_mask_layer_shape_totvert(masklay);
 
-  masklay_shape = MEM_cnew<MaskLayerShape>(__func__);
+  masklay_shape = MEM_callocN<MaskLayerShape>(__func__);
   masklay_shape->frame = frame;
   masklay_shape->tot_vert = tot_vert;
-  masklay_shape->data = MEM_cnew_array<float>(tot_vert * MASK_OBJECT_SHAPE_ELEM_SIZE, __func__);
+  masklay_shape->data = MEM_calloc_arrayN<float>(tot_vert * MASK_OBJECT_SHAPE_ELEM_SIZE, __func__);
 
   return masklay_shape;
 }
@@ -1474,7 +1481,7 @@ void BKE_mask_spline_ensure_deform(MaskSpline *spline)
       MEM_freeN(spline->points_deform);
     }
 
-    spline->points_deform = MEM_cnew_array<MaskSplinePoint>(spline->tot_point, __func__);
+    spline->points_deform = MEM_calloc_arrayN<MaskSplinePoint>(spline->tot_point, __func__);
   }
   else {
     // printf("alloc spline done\n");
@@ -1849,8 +1856,8 @@ void BKE_mask_layer_shape_changed_add(MaskLayer *masklay,
         float *data_resized;
 
         masklay_shape->tot_vert++;
-        data_resized = MEM_cnew_array<float>(masklay_shape->tot_vert * MASK_OBJECT_SHAPE_ELEM_SIZE,
-                                             __func__);
+        data_resized = MEM_calloc_arrayN<float>(
+            masklay_shape->tot_vert * MASK_OBJECT_SHAPE_ELEM_SIZE, __func__);
         if (index > 0) {
           memcpy(data_resized,
                  masklay_shape->data,
@@ -1909,8 +1916,8 @@ void BKE_mask_layer_shape_changed_remove(MaskLayer *masklay, int index, int coun
       float *data_resized;
 
       masklay_shape->tot_vert -= count;
-      data_resized = MEM_cnew_array<float>(masklay_shape->tot_vert * MASK_OBJECT_SHAPE_ELEM_SIZE,
-                                           __func__);
+      data_resized = MEM_calloc_arrayN<float>(
+          masklay_shape->tot_vert * MASK_OBJECT_SHAPE_ELEM_SIZE, __func__);
       if (index > 0) {
         memcpy(data_resized,
                masklay_shape->data,
@@ -1982,7 +1989,7 @@ void BKE_mask_clipboard_copy_from_layer(MaskLayer *mask_layer)
         if (point->parent.id) {
           if (!BLI_ghash_lookup(mask_clipboard.id_hash, point->parent.id)) {
             int len = strlen(point->parent.id->name);
-            char *name_copy = static_cast<char *>(MEM_mallocN(len + 1, "mask clipboard ID name"));
+            char *name_copy = MEM_malloc_arrayN<char>(size_t(len) + 1, "mask clipboard ID name");
             memcpy(name_copy, point->parent.id->name, len + 1);
             BLI_ghash_insert(mask_clipboard.id_hash, point->parent.id, name_copy);
           }
