@@ -32,6 +32,7 @@
 #include "DNA_particle_types.h"
 #include "DNA_pointcloud_types.h"
 #include "DNA_scene_types.h"
+#include "DNA_userdef_types.h"
 #include "DNA_volume_types.h"
 
 #include "BLI_array_utils.h"
@@ -49,7 +50,6 @@
 #include "BKE_curves.hh"
 #include "BKE_displist.h"
 #include "BKE_editmesh.hh"
-#include "BKE_gpencil_legacy.h"
 #include "BKE_grease_pencil.hh"
 #include "BKE_icons.h"
 #include "BKE_idtype.hh"
@@ -78,7 +78,7 @@
 
 #include "BLO_read_write.hh"
 
-static CLG_LogRef LOG = {"bke.material"};
+static CLG_LogRef LOG = {"material"};
 
 static void material_init_data(ID *id)
 {
@@ -87,8 +87,6 @@ static void material_init_data(ID *id)
   BLI_assert(MEMCMP_STRUCT_AFTER_IS_ZERO(material, id));
 
   MEMCPY_STRUCT_AFTER(material, DNA_struct_default_get(Material), id);
-
-  *((short *)id->name) = ID_MA;
 }
 
 static void material_copy_data(Main *bmain,
@@ -171,7 +169,6 @@ static void material_free_data(ID *id)
 static void material_foreach_id(ID *id, LibraryForeachIDData *data)
 {
   Material *material = reinterpret_cast<Material *>(id);
-  const int flag = BKE_lib_query_foreachid_process_flags_get(data);
 
   /* Node-trees **are owned by IDs**, treat them as mere sub-data and not real ID! */
   BKE_LIB_FOREACHID_PROCESS_FUNCTION_CALL(
@@ -183,9 +180,21 @@ static void material_foreach_id(ID *id, LibraryForeachIDData *data)
     BKE_LIB_FOREACHID_PROCESS_IDSUPER(data, material->gp_style->sima, IDWALK_CB_USER);
     BKE_LIB_FOREACHID_PROCESS_IDSUPER(data, material->gp_style->ima, IDWALK_CB_USER);
   }
+}
 
-  if (flag & IDWALK_DO_DEPRECATED_POINTERS) {
-    BKE_LIB_FOREACHID_PROCESS_ID_NOCHECK(data, material->ipo, IDWALK_CB_USER);
+static void material_foreach_working_space_color(ID *id,
+                                                 const IDTypeForeachColorFunctionCallback &fn)
+{
+  Material *material = reinterpret_cast<Material *>(id);
+
+  fn.single(&material->r);
+  fn.single(&material->specr);
+  fn.single(material->line_col);
+
+  if (material->gp_style) {
+    fn.single(material->gp_style->stroke_rgba);
+    fn.single(material->gp_style->fill_rgba);
+    fn.single(material->gp_style->mix_rgba);
   }
 }
 
@@ -197,6 +206,9 @@ static void material_blend_write(BlendWriter *writer, ID *id, const void *id_add
   ma->texpaintslot = nullptr;
   BLI_listbase_clear(&ma->gpumaterial);
 
+  /* Set deprecated #use_nodes for forward compatibility. */
+  ma->use_nodes = true;
+
   /* write LibData */
   BLO_write_id_struct(writer, Material, id_address, &ma->id);
   BKE_id_blend_write(writer, &ma->id);
@@ -232,7 +244,7 @@ static void material_blend_read_data(BlendDataReader *reader, ID *id)
 }
 
 IDTypeInfo IDType_ID_MA = {
-    /*id_code*/ ID_MA,
+    /*id_code*/ Material::id_type,
     /*id_filter*/ FILTER_ID_MA,
     /*dependencies_id_types*/ FILTER_ID_TE | FILTER_ID_GR,
     /*main_listbase_index*/ INDEX_ID_MA,
@@ -250,6 +262,7 @@ IDTypeInfo IDType_ID_MA = {
     /*foreach_id*/ material_foreach_id,
     /*foreach_cache*/ nullptr,
     /*foreach_path*/ nullptr,
+    /*foreach_working_space_color*/ material_foreach_working_space_color,
     /*owner_pointer_get*/ nullptr,
 
     /*blend_write*/ material_blend_write,
@@ -264,8 +277,7 @@ IDTypeInfo IDType_ID_MA = {
 void BKE_gpencil_material_attr_init(Material *ma)
 {
   if ((ma) && (ma->gp_style == nullptr)) {
-    ma->gp_style = static_cast<MaterialGPencilStyle *>(
-        MEM_callocN(sizeof(MaterialGPencilStyle), "Grease Pencil Material Settings"));
+    ma->gp_style = MEM_callocN<MaterialGPencilStyle>("Grease Pencil Material Settings");
 
     MaterialGPencilStyle *gp_style = ma->gp_style;
     /* set basic settings */
@@ -306,7 +318,7 @@ Material *BKE_material_add(Main *bmain, const char *name)
 {
   Material *ma;
 
-  ma = static_cast<Material *>(BKE_id_new(bmain, ID_MA, name));
+  ma = BKE_id_new<Material>(bmain, name);
 
   return ma;
 }
@@ -338,10 +350,6 @@ Material ***BKE_object_material_array_p(Object *ob)
     MetaBall *mb = static_cast<MetaBall *>(ob->data);
     return &(mb->mat);
   }
-  if (ob->type == OB_GPENCIL_LEGACY) {
-    bGPdata *gpd = static_cast<bGPdata *>(ob->data);
-    return &(gpd->mat);
-  }
   if (ob->type == OB_CURVES) {
     Curves *curves = static_cast<Curves *>(ob->data);
     return &(curves->mat);
@@ -375,10 +383,6 @@ short *BKE_object_material_len_p(Object *ob)
     MetaBall *mb = static_cast<MetaBall *>(ob->data);
     return &(mb->totcol);
   }
-  if (ob->type == OB_GPENCIL_LEGACY) {
-    bGPdata *gpd = static_cast<bGPdata *>(ob->data);
-    return &(gpd->totcol);
-  }
   if (ob->type == OB_CURVES) {
     Curves *curves = static_cast<Curves *>(ob->data);
     return &(curves->totcol);
@@ -565,6 +569,10 @@ void BKE_id_material_resize(Main *bmain, ID *id, short totcol, bool do_id_user)
   if (matar == nullptr) {
     return;
   }
+  if (totcol == *totcolp) {
+    /* Prevent depsgraph update and relations tag when nothing changed. */
+    return;
+  }
 
   if (do_id_user && totcol < (*totcolp)) {
     short i;
@@ -593,7 +601,7 @@ void BKE_id_material_append(Main *bmain, ID *id, Material *ma)
   Material ***matar = BKE_id_material_array_p(id);
   if (matar) {
     short *totcol = BKE_id_material_len_p(id);
-    Material **mat = MEM_cnew_array<Material *>((*totcol) + 1, "newmatar");
+    Material **mat = MEM_calloc_arrayN<Material *>((*totcol) + 1, "newmatar");
     if (*totcol) {
       memcpy(mat, *matar, sizeof(void *) * (*totcol));
     }
@@ -744,10 +752,15 @@ static const ID *get_evaluated_object_data_with_materials(const Object *ob)
 
 Material *BKE_object_material_get_eval(Object *ob, short act)
 {
-  BLI_assert(DEG_is_evaluated_object(ob));
-
   const ID *data = get_evaluated_object_data_with_materials(ob);
-  const int slots_num = BKE_object_material_count_eval(ob);
+  return const_cast<Material *>(BKE_object_material_get_eval(*ob, *data, act));
+}
+
+const Material *BKE_object_material_get_eval(const Object &ob, const ID &data, const short act)
+{
+  BLI_assert(DEG_is_evaluated(&ob));
+
+  const int slots_num = BKE_object_material_count_eval(ob, data);
 
   if (slots_num == 0) {
     return nullptr;
@@ -755,13 +768,13 @@ Material *BKE_object_material_get_eval(Object *ob, short act)
 
   /* Clamp to number of slots if index is out of range, same convention as used for rendering. */
   const int slot_index = clamp_i(act - 1, 0, slots_num - 1);
-  const int tot_slots_object = ob->totcol;
+  const int tot_slots_object = ob.totcol;
 
   /* Check if slot is overwritten by object. */
   if (slot_index < tot_slots_object) {
-    if (ob->matbits) {
-      if (ob->matbits[slot_index]) {
-        Material *material = ob->mat[slot_index];
+    if (ob.matbits) {
+      if (ob.matbits[slot_index]) {
+        Material *material = ob.mat[slot_index];
         if (material != nullptr) {
           return material;
         }
@@ -770,12 +783,12 @@ Material *BKE_object_material_get_eval(Object *ob, short act)
   }
 
   /* Otherwise use data from object-data. */
-  const short *data_slots_num_ptr = BKE_id_material_len_p(const_cast<ID *>(data));
+  const short *data_slots_num_ptr = BKE_id_material_len_p(const_cast<ID *>(&data));
   if (!data_slots_num_ptr) {
     return nullptr;
   }
   const int data_slots_num = *data_slots_num_ptr;
-  Material **data_materials = *BKE_id_material_array_p(const_cast<ID *>(data));
+  Material **data_materials = *BKE_id_material_array_p(const_cast<ID *>(&data));
   if (slot_index < data_slots_num) {
     Material *material = data_materials[slot_index];
     return material;
@@ -785,7 +798,7 @@ Material *BKE_object_material_get_eval(Object *ob, short act)
 
 int BKE_object_material_count_eval(const Object *ob)
 {
-  BLI_assert(DEG_is_evaluated_object(ob));
+  BLI_assert(DEG_is_evaluated(ob));
   if (ob->type == OB_EMPTY) {
     return 0;
   }
@@ -795,6 +808,17 @@ int BKE_object_material_count_eval(const Object *ob)
   return std::max(ob->totcol, len_p ? *len_p : 0);
 }
 
+int BKE_object_material_count_eval(const Object &ob, const ID &data)
+{
+  BLI_assert(DEG_is_evaluated(&ob));
+  if (ob.type == OB_EMPTY) {
+    return 0;
+  }
+  BLI_assert(ob.data != nullptr);
+  const short *len_p = BKE_id_material_len_p(const_cast<ID *>(&data));
+  return std::max(ob.totcol, len_p ? *len_p : 0);
+}
+
 std::optional<int> BKE_id_material_index_max_eval(const ID &id)
 {
   switch (GS(id.name)) {
@@ -822,6 +846,14 @@ std::optional<int> BKE_id_material_index_max_eval(const ID &id)
   return 0;
 }
 
+int BKE_id_material_used_eval(const ID &id)
+{
+  if (std::optional<int> max_index = BKE_id_material_index_max_eval(id)) {
+    return *max_index + 1;
+  }
+  return 0;
+}
+
 int BKE_id_material_used_with_fallback_eval(const ID &id)
 {
   const int max_material_index = std::max(0, BKE_id_material_index_max_eval(id).value_or(0));
@@ -917,6 +949,8 @@ Material *BKE_gpencil_material(Object *ob, short act)
     return ma;
   }
 
+  /* XXX FIXME This is critical abuse of the 'default material' feature, these IDs should never be
+   * used/returned as 'regular' data. */
   return BKE_material_default_gpencil();
 }
 
@@ -934,8 +968,26 @@ MaterialGPencilStyle *BKE_gpencil_material_settings(Object *ob, short act)
   return BKE_material_default_gpencil()->gp_style;
 }
 
+/**
+ * Ensure a valid active index.
+ * When materials are assigned, the active material must be in the range of `1..totcol`.
+ * see #139182 for details.
+ */
+static void object_material_active_index_sanitize(Object *ob)
+{
+  if (ob->totcol && ob->actcol == 0) {
+    ob->actcol = 1;
+  }
+  ob->actcol = std::min(ob->actcol, ob->totcol);
+}
+
 void BKE_object_material_resize(Main *bmain, Object *ob, const short totcol, bool do_id_user)
 {
+  if (totcol == ob->totcol) {
+    /* Prevent depsgraph update and relations tag when nothing changed. */
+    return;
+  }
+
   Material **newmatar;
   char *newmatbits;
 
@@ -954,8 +1006,8 @@ void BKE_object_material_resize(Main *bmain, Object *ob, const short totcol, boo
     }
   }
   else if (ob->totcol < totcol) {
-    newmatar = MEM_cnew_array<Material *>(totcol, "newmatar");
-    newmatbits = MEM_cnew_array<char>(totcol, "newmatbits");
+    newmatar = MEM_calloc_arrayN<Material *>(totcol, "newmatar");
+    newmatbits = MEM_calloc_arrayN<char>(totcol, "newmatbits");
     if (ob->totcol) {
       memcpy(newmatar, ob->mat, sizeof(void *) * ob->totcol);
       memcpy(newmatbits, ob->matbits, sizeof(char) * ob->totcol);
@@ -968,10 +1020,6 @@ void BKE_object_material_resize(Main *bmain, Object *ob, const short totcol, boo
   /* XXX(@ideasman42): why not realloc on shrink? */
 
   ob->totcol = totcol;
-  if (ob->totcol && ob->actcol == 0) {
-    ob->actcol = 1;
-  }
-  ob->actcol = std::min(ob->actcol, ob->totcol);
 
   DEG_id_tag_update(&ob->id, ID_RECALC_SYNC_TO_EVAL | ID_RECALC_GEOMETRY);
   DEG_relations_tag_update(bmain);
@@ -995,6 +1043,7 @@ void BKE_object_materials_sync_length(Main *bmain, Object *ob, ID *id)
   else {
     /* Normal case: the use the obdata amount of materials slots to update the object's one. */
     BKE_object_material_resize(bmain, ob, *totcol, false);
+    object_material_active_index_sanitize(ob);
   }
 }
 
@@ -1014,6 +1063,7 @@ void BKE_objects_materials_sync_length_all(Main *bmain, ID *id)
   {
     if (ob->data == id) {
       BKE_object_material_resize(bmain, ob, *totcol, false);
+      object_material_active_index_sanitize(ob);
       processed_objects++;
       BLI_assert(processed_objects <= id->us && processed_objects > 0);
       if (processed_objects == id->us) {
@@ -1044,7 +1094,7 @@ void BKE_id_material_assign(Main *bmain, ID *id, Material *ma, short act)
   }
 
   if (act > *totcolp) {
-    matar = MEM_cnew_array<Material *>(act, "matarray1");
+    matar = MEM_calloc_arrayN<Material *>(act, "matarray1");
 
     if (*totcolp) {
       memcpy(matar, *matarar, sizeof(void *) * (*totcolp));
@@ -1067,6 +1117,8 @@ void BKE_id_material_assign(Main *bmain, ID *id, Material *ma, short act)
   }
 
   BKE_objects_materials_sync_length_all(bmain, id);
+  DEG_id_tag_update(id, ID_RECALC_SYNC_TO_EVAL | ID_RECALC_GEOMETRY);
+  DEG_relations_tag_update(bmain);
 }
 
 static void object_material_assign(
@@ -1091,7 +1143,7 @@ static void object_material_assign(
   }
 
   if (act > *totcolp) {
-    matar = MEM_cnew_array<Material *>(act, "matarray1");
+    matar = MEM_calloc_arrayN<Material *>(act, "matarray1");
 
     if (*totcolp) {
       memcpy(matar, *matarar, sizeof(void *) * (*totcolp));
@@ -1162,6 +1214,9 @@ static void object_material_assign(
   if (ma) {
     id_us_plus(&ma->id);
   }
+
+  DEG_id_tag_update(&ob->id, ID_RECALC_SYNC_TO_EVAL | ID_RECALC_GEOMETRY);
+  DEG_relations_tag_update(bmain);
 }
 
 void BKE_object_material_assign(Main *bmain, Object *ob, Material *ma, short act, int assign_type)
@@ -1275,11 +1330,11 @@ void BKE_object_material_from_eval_data(Main *bmain, Object *ob_orig, const ID *
 
   /* Create new material slots based on materials on evaluated geometry. */
   *orig_totcol = *eval_totcol;
-  *orig_mat = *eval_totcol > 0 ? MEM_cnew_array<Material *>(*eval_totcol, __func__) : nullptr;
+  *orig_mat = *eval_totcol > 0 ? MEM_calloc_arrayN<Material *>(*eval_totcol, __func__) : nullptr;
   for (int i = 0; i < *eval_totcol; i++) {
     Material *material_eval = (*eval_mat)[i];
     if (material_eval != nullptr) {
-      Material *material_orig = (Material *)DEG_get_original_id(&material_eval->id);
+      Material *material_orig = DEG_get_original(material_eval);
       (*orig_mat)[i] = material_orig;
       id_us_plus(&material_orig->id);
     }
@@ -1386,7 +1441,7 @@ bool BKE_object_material_slot_remove(Main *bmain, Object *ob)
   }
 
   /* can happen on face selection in editmode */
-  ob->actcol = std::min(ob->actcol, ob->totcol);
+  object_material_active_index_sanitize(ob);
 
   /* we delete the actcol */
   mao = (*matarar)[ob->actcol - 1];
@@ -1425,7 +1480,7 @@ bool BKE_object_material_slot_remove(Main *bmain, Object *ob)
         obt->matbits[a - 1] = obt->matbits[a];
       }
       obt->totcol--;
-      obt->actcol = std::min(obt->actcol, obt->totcol);
+      object_material_active_index_sanitize(ob);
 
       if (obt->totcol == 0) {
         MEM_freeN(obt->mat);
@@ -1637,10 +1692,9 @@ void BKE_texpaint_slot_refresh_cache(Scene *scene, Material *ma, const Object *o
       ma->paint_clone_slot = 0;
     }
     else {
-      ma->texpaintslot = static_cast<TexPaintSlot *>(
-          MEM_callocN(sizeof(TexPaintSlot) * count, "texpaint_slots"));
+      ma->texpaintslot = MEM_calloc_arrayN<TexPaintSlot>(count, "texpaint_slots");
 
-      bNode *active_node = blender::bke::node_get_active_paint_canvas(ma->nodetree);
+      bNode *active_node = blender::bke::node_get_active_paint_canvas(*ma->nodetree);
 
       fill_texpaint_slots_recursive(ma->nodetree, active_node, ob, ma, count, slot_filter);
 
@@ -1987,135 +2041,139 @@ void BKE_material_eval(Depsgraph *depsgraph, Material *material)
  * Used for rendering when objects have no materials assigned, and initializing
  * default shader nodes. */
 
-static Material default_material_empty;
-static Material default_material_holdout;
-static Material default_material_surface;
-static Material default_material_volume;
-static Material default_material_gpencil;
+static Material *default_material_empty = nullptr;
+static Material *default_material_holdout = nullptr;
+static Material *default_material_surface = nullptr;
+static Material *default_material_volume = nullptr;
+static Material *default_material_gpencil = nullptr;
 
-static Material *default_materials[] = {&default_material_empty,
-                                        &default_material_holdout,
-                                        &default_material_surface,
-                                        &default_material_volume,
-                                        &default_material_gpencil,
-                                        nullptr};
+static Material **default_materials[] = {&default_material_empty,
+                                         &default_material_holdout,
+                                         &default_material_surface,
+                                         &default_material_volume,
+                                         &default_material_gpencil,
+                                         nullptr};
 
-static void material_default_gpencil_init(Material *ma)
+static Material *material_default_create(Material **ma_p, const char *name)
 {
-  BLI_strncpy(ma->id.name + 2, "Default GPencil", MAX_NAME);
+  *ma_p = BKE_id_new_nomain<Material>(name);
+  return *ma_p;
+}
+
+static void material_default_gpencil_init(Material **ma_p)
+{
+  Material *ma = material_default_create(ma_p, "Default GPencil");
+
   BKE_gpencil_material_attr_init(ma);
   add_v3_fl(&ma->gp_style->stroke_rgba[0], 0.6f);
 }
 
-static void material_default_surface_init(Material *ma)
+static void material_default_surface_init(Material **ma_p)
 {
-  BLI_strncpy(ma->id.name + 2, "Default Surface", MAX_NAME);
+  Material *ma = material_default_create(ma_p, "Default Surface");
 
   bNodeTree *ntree = blender::bke::node_tree_add_tree_embedded(
       nullptr, &ma->id, "Shader Nodetree", ntreeType_Shader->idname);
-  ma->use_nodes = true;
 
-  bNode *principled = blender::bke::node_add_static_node(nullptr, ntree, SH_NODE_BSDF_PRINCIPLED);
-  bNodeSocket *base_color = blender::bke::node_find_socket(principled, SOCK_IN, "Base Color");
+  bNode *principled = blender::bke::node_add_static_node(nullptr, *ntree, SH_NODE_BSDF_PRINCIPLED);
+  bNodeSocket *base_color = blender::bke::node_find_socket(*principled, SOCK_IN, "Base Color");
   copy_v3_v3(((bNodeSocketValueRGBA *)base_color->default_value)->value, &ma->r);
 
-  bNode *output = blender::bke::node_add_static_node(nullptr, ntree, SH_NODE_OUTPUT_MATERIAL);
+  bNode *output = blender::bke::node_add_static_node(nullptr, *ntree, SH_NODE_OUTPUT_MATERIAL);
 
-  blender::bke::node_add_link(ntree,
-                              principled,
-                              blender::bke::node_find_socket(principled, SOCK_OUT, "BSDF"),
-                              output,
-                              blender::bke::node_find_socket(output, SOCK_IN, "Surface"));
+  blender::bke::node_add_link(*ntree,
+                              *principled,
+                              *blender::bke::node_find_socket(*principled, SOCK_OUT, "BSDF"),
+                              *output,
+                              *blender::bke::node_find_socket(*output, SOCK_IN, "Surface"));
 
-  principled->location[0] = 10.0f;
-  principled->location[1] = 300.0f;
-  output->location[0] = 300.0f;
-  output->location[1] = 300.0f;
+  principled->location[0] = -200.0f;
+  principled->location[1] = 100.0f;
+  output->location[0] = 200.0f;
+  output->location[1] = 100.0f;
 
-  blender::bke::node_set_active(ntree, output);
+  blender::bke::node_set_active(*ntree, *output);
 }
 
-static void material_default_volume_init(Material *ma)
+static void material_default_volume_init(Material **ma_p)
 {
-  BLI_strncpy(ma->id.name + 2, "Default Volume", MAX_NAME);
+  Material *ma = material_default_create(ma_p, "Default Volume");
 
   bNodeTree *ntree = blender::bke::node_tree_add_tree_embedded(
       nullptr, &ma->id, "Shader Nodetree", ntreeType_Shader->idname);
-  ma->use_nodes = true;
 
   bNode *principled = blender::bke::node_add_static_node(
-      nullptr, ntree, SH_NODE_VOLUME_PRINCIPLED);
-  bNode *output = blender::bke::node_add_static_node(nullptr, ntree, SH_NODE_OUTPUT_MATERIAL);
+      nullptr, *ntree, SH_NODE_VOLUME_PRINCIPLED);
+  bNode *output = blender::bke::node_add_static_node(nullptr, *ntree, SH_NODE_OUTPUT_MATERIAL);
 
-  blender::bke::node_add_link(ntree,
-                              principled,
-                              blender::bke::node_find_socket(principled, SOCK_OUT, "Volume"),
-                              output,
-                              blender::bke::node_find_socket(output, SOCK_IN, "Volume"));
+  blender::bke::node_add_link(*ntree,
+                              *principled,
+                              *blender::bke::node_find_socket(*principled, SOCK_OUT, "Volume"),
+                              *output,
+                              *blender::bke::node_find_socket(*output, SOCK_IN, "Volume"));
 
-  principled->location[0] = 10.0f;
-  principled->location[1] = 300.0f;
-  output->location[0] = 300.0f;
-  output->location[1] = 300.0f;
+  principled->location[0] = -200.0f;
+  principled->location[1] = 100.0f;
+  output->location[0] = 200.0f;
+  output->location[1] = 100.0f;
 
-  blender::bke::node_set_active(ntree, output);
+  blender::bke::node_set_active(*ntree, *output);
 }
 
-static void material_default_holdout_init(Material *ma)
+static void material_default_holdout_init(Material **ma_p)
 {
-  BLI_strncpy(ma->id.name + 2, "Default Holdout", MAX_NAME);
+  Material *ma = material_default_create(ma_p, "Default Holdout");
 
   bNodeTree *ntree = blender::bke::node_tree_add_tree_embedded(
       nullptr, &ma->id, "Shader Nodetree", ntreeType_Shader->idname);
-  ma->use_nodes = true;
 
-  bNode *holdout = blender::bke::node_add_static_node(nullptr, ntree, SH_NODE_HOLDOUT);
-  bNode *output = blender::bke::node_add_static_node(nullptr, ntree, SH_NODE_OUTPUT_MATERIAL);
+  bNode *holdout = blender::bke::node_add_static_node(nullptr, *ntree, SH_NODE_HOLDOUT);
+  bNode *output = blender::bke::node_add_static_node(nullptr, *ntree, SH_NODE_OUTPUT_MATERIAL);
 
-  blender::bke::node_add_link(ntree,
-                              holdout,
-                              blender::bke::node_find_socket(holdout, SOCK_OUT, "Holdout"),
-                              output,
-                              blender::bke::node_find_socket(output, SOCK_IN, "Surface"));
+  blender::bke::node_add_link(*ntree,
+                              *holdout,
+                              *blender::bke::node_find_socket(*holdout, SOCK_OUT, "Holdout"),
+                              *output,
+                              *blender::bke::node_find_socket(*output, SOCK_IN, "Surface"));
 
   holdout->location[0] = 10.0f;
   holdout->location[1] = 300.0f;
   output->location[0] = 300.0f;
   output->location[1] = 300.0f;
 
-  blender::bke::node_set_active(ntree, output);
+  blender::bke::node_set_active(*ntree, *output);
 }
 
 Material *BKE_material_default_empty()
 {
-  return &default_material_empty;
+  return default_material_empty;
 }
 
 Material *BKE_material_default_holdout()
 {
-  return &default_material_holdout;
+  return default_material_holdout;
 }
 
 Material *BKE_material_default_surface()
 {
-  return &default_material_surface;
+  return default_material_surface;
 }
 
 Material *BKE_material_default_volume()
 {
-  return &default_material_volume;
+  return default_material_volume;
 }
 
 Material *BKE_material_default_gpencil()
 {
-  return &default_material_gpencil;
+  return default_material_gpencil;
 }
 
 void BKE_material_defaults_free_gpu()
 {
   for (int i = 0; default_materials[i]; i++) {
-    Material *ma = default_materials[i];
-    if (ma->gpumaterial.first) {
+    Material *ma = *default_materials[i];
+    if (ma && ma->gpumaterial.first) {
       GPU_material_free(&ma->gpumaterial);
     }
   }
@@ -2126,9 +2184,12 @@ void BKE_material_defaults_free_gpu()
 void BKE_materials_init()
 {
   for (int i = 0; default_materials[i]; i++) {
-    material_init_data(&default_materials[i]->id);
+    BLI_assert_msg(*default_materials[i] == nullptr,
+                   "Default material pointers should always be null when initializing them, maybe "
+                   "missing a call to `BKE_materials_exit` first?");
   }
 
+  material_default_create(&default_material_empty, "Default Empty");
   material_default_surface_init(&default_material_surface);
   material_default_volume_init(&default_material_volume);
   material_default_holdout_init(&default_material_holdout);
@@ -2138,6 +2199,10 @@ void BKE_materials_init()
 void BKE_materials_exit()
 {
   for (int i = 0; default_materials[i]; i++) {
-    material_free_data(&default_materials[i]->id);
+    Material *ma = *default_materials[i];
+    *default_materials[i] = nullptr;
+    if (ma) {
+      BKE_id_free(nullptr, &ma->id);
+    }
   }
 }
