@@ -6,6 +6,7 @@
  * \ingroup bke
  */
 
+#include <algorithm>
 #include <cstring>
 
 #include "MEM_guardedalloc.h"
@@ -22,15 +23,12 @@
 #include "DNA_meshdata_types.h"
 #include "DNA_modifier_types.h"
 #include "DNA_particle_types.h"
-#include "DNA_scene_types.h"
 
-#include "BKE_customdata.h"
-#include "BKE_global.h"
-#include "BKE_lib_id.h"
+#include "BKE_customdata.hh"
+#include "BKE_global.hh"
+#include "BKE_lib_id.hh"
 #include "BKE_mesh.hh"
 #include "BKE_mesh_legacy_convert.hh"
-#include "BKE_mesh_runtime.hh"
-#include "BKE_object.hh"
 #include "BKE_particle.h"
 
 #include "DEG_depsgraph_query.hh"
@@ -40,7 +38,7 @@ static void alloc_child_particles(ParticleSystem *psys, int tot)
   if (psys->child) {
     /* only re-allocate if we have to */
     if (psys->part->childtype && psys->totchild == tot) {
-      memset(psys->child, 0, tot * sizeof(ChildParticle));
+      std::fill_n(psys->child, tot, ChildParticle{});
       return;
     }
 
@@ -52,8 +50,7 @@ static void alloc_child_particles(ParticleSystem *psys, int tot)
   if (psys->part->childtype) {
     psys->totchild = tot;
     if (psys->totchild) {
-      psys->child = static_cast<ChildParticle *>(
-          MEM_callocN(psys->totchild * sizeof(ChildParticle), "child_particles"));
+      psys->child = MEM_calloc_arrayN<ChildParticle>(psys->totchild, "child_particles");
     }
   }
 }
@@ -100,7 +97,7 @@ static void distribute_grid(Mesh *mesh, ParticleSystem *psys)
   ParticleData *pa = nullptr;
   float min[3], max[3], delta[3], d;
   const blender::Span<blender::float3> positions = mesh->vert_positions();
-  int totvert = mesh->totvert, from = psys->part->from;
+  int totvert = mesh->verts_num, from = psys->part->from;
   int i, j, k, p, res = psys->part->grid_res, size[3], axis;
 
   /* find bounding box of dm */
@@ -127,12 +124,12 @@ static void distribute_grid(Mesh *mesh, ParticleSystem *psys)
   size[(axis + 2) % 3] = int(ceil(delta[(axis + 2) % 3] / d));
 
   /* float errors grrr. */
-  size[(axis + 1) % 3] = MIN2(size[(axis + 1) % 3], res);
-  size[(axis + 2) % 3] = MIN2(size[(axis + 2) % 3], res);
+  size[(axis + 1) % 3] = std::min(size[(axis + 1) % 3], res);
+  size[(axis + 2) % 3] = std::min(size[(axis + 2) % 3], res);
 
-  size[0] = MAX2(size[0], 1);
-  size[1] = MAX2(size[1], 1);
-  size[2] = MAX2(size[2], 1);
+  size[0] = std::max(size[0], 1);
+  size[1] = std::max(size[1], 1);
+  size[2] = std::max(size[2], 1);
 
   /* no full offset for flat/thin objects */
   min[0] += d < delta[0] ? d / 2.0f : delta[0] / 2.0f;
@@ -350,9 +347,9 @@ static void init_mv_jit(float *jit, int num, int seed2, float amount)
     return;
   }
 
-  rad1 = float(1.0f / sqrtf(float(num)));
-  rad2 = float(1.0f / float(num));
-  rad3 = float(sqrtf(float(num)) / float(num));
+  rad1 = (1.0f / sqrtf(float(num)));
+  rad2 = (1.0f / float(num));
+  rad3 = (sqrtf(float(num)) / float(num));
 
   rng = BLI_rng_new(31415926 + num + seed2);
   x = 0;
@@ -362,19 +359,20 @@ static void init_mv_jit(float *jit, int num, int seed2, float amount)
     jit[i] = x + amount * rad1 * (0.5f - BLI_rng_get_float(rng));
     jit[i + 1] = i / (2.0f * num) + amount * rad1 * (0.5f - BLI_rng_get_float(rng));
 
-    jit[i] -= float(floor(jit[i]));
-    jit[i + 1] -= float(floor(jit[i + 1]));
+    jit[i] -= floor(jit[i]);
+    jit[i + 1] -= floor(jit[i + 1]);
 
     x += rad3;
-    x -= float(floor(x));
+    x -= floor(x);
   }
 
-  jit2 = static_cast<float *>(MEM_mallocN(12 + sizeof(float[2]) * num, "initjit"));
+  /* FIXME: The `+ 3` number of items does not seem to be required? */
+  jit2 = MEM_malloc_arrayN<float>(3 + 2 * size_t(num), "initjit");
 
   for (i = 0; i < 4; i++) {
-    BLI_jitterate1((float(*)[2])jit, (float(*)[2])jit2, num, rad1);
-    BLI_jitterate1((float(*)[2])jit, (float(*)[2])jit2, num, rad1);
-    BLI_jitterate2((float(*)[2])jit, (float(*)[2])jit2, num, rad2);
+    BLI_jitterate1((float (*)[2])jit, (float (*)[2])jit2, num, rad1);
+    BLI_jitterate1((float (*)[2])jit, (float (*)[2])jit2, num, rad1);
+    BLI_jitterate2((float (*)[2])jit, (float (*)[2])jit2, num, rad2);
   }
   MEM_freeN(jit2);
   BLI_rng_free(rng);
@@ -475,7 +473,7 @@ static void distribute_from_verts_exec(ParticleTask *thread, ParticleData *pa, i
 
   zero_v4(pa->fuv);
 
-  if (pa->num != DMCACHE_NOTFOUND && pa->num < ctx->mesh->totvert) {
+  if (pa->num != DMCACHE_NOTFOUND && pa->num < ctx->mesh->verts_num) {
 
     /* This finds the first face to contain the emitting vertex,
      * this is not ideal, but is mostly fine as UV seams generally
@@ -619,8 +617,8 @@ static void distribute_from_volume_exec(ParticleTask *thread, ParticleData *pa,
   tot = mesh->totface_legacy;
 
   psys_interpolate_face(mesh,
-                        reinterpret_cast<const float(*)[3]>(positions.data()),
-                        reinterpret_cast<const float(*)[3]>(mesh->vert_normals().data()),
+                        reinterpret_cast<const float (*)[3]>(positions.data()),
+                        reinterpret_cast<const float (*)[3]>(mesh->vert_normals().data()),
                         mface,
                         nullptr,
                         nullptr,
@@ -837,8 +835,8 @@ static int distribute_compare_orig_index(const void *p1, const void *p2, void *u
     return -1;
   }
   if (index1 == index2) {
-    /* this pointer comparison appears to make qsort stable for glibc,
-     * and apparently on solaris too, makes the renders reproducible */
+    /* This pointer comparison appears to make #qsort stable for GLIBC,
+     * and apparently on SOLARIS too, makes the renders reproducible. */
     if (p1 < p2) {
       return -1;
     }
@@ -1022,9 +1020,9 @@ static int psys_thread_context_init_distribute(ParticleThreadContext *ctx,
 
     if (from == PART_FROM_VERT) {
       const blender::Span<blender::float3> positions = mesh->vert_positions();
-      const float(*orcodata)[3] = static_cast<const float(*)[3]>(
+      const float (*orcodata)[3] = static_cast<const float (*)[3]>(
           CustomData_get_layer(&mesh->vert_data, CD_ORCO));
-      int totvert = mesh->totvert;
+      int totvert = mesh->verts_num;
 
       tree = BLI_kdtree_3d_new(totvert);
 
@@ -1044,7 +1042,7 @@ static int psys_thread_context_init_distribute(ParticleThreadContext *ctx,
   }
 
   /* Get total number of emission elements and allocate needed arrays */
-  totelem = (from == PART_FROM_VERT) ? mesh->totvert : mesh->totface_legacy;
+  totelem = (from == PART_FROM_VERT) ? mesh->verts_num : mesh->totface_legacy;
 
   if (totelem == 0) {
     distribute_invalid(sim, children ? PART_FROM_CHILD : 0);
@@ -1063,19 +1061,16 @@ static int psys_thread_context_init_distribute(ParticleThreadContext *ctx,
     return 0;
   }
 
-  element_weight = static_cast<float *>(
-      MEM_callocN(sizeof(float) * totelem, "particle_distribution_weights"));
-  particle_element = static_cast<int *>(
-      MEM_callocN(sizeof(int) * totpart, "particle_distribution_indexes"));
-  jitter_offset = static_cast<float *>(
-      MEM_callocN(sizeof(float) * totelem, "particle_distribution_jitoff"));
+  element_weight = MEM_calloc_arrayN<float>(totelem, "particle_distribution_weights");
+  particle_element = MEM_calloc_arrayN<int>(totpart, "particle_distribution_indexes");
+  jitter_offset = MEM_calloc_arrayN<float>(totelem, "particle_distribution_jitoff");
 
   /* Calculate weights from face areas */
   if ((part->flag & PART_EDISTR || children) && from != PART_FROM_VERT) {
     float totarea = 0.0f, co1[3], co2[3], co3[3], co4[3];
-    const float(*orcodata)[3];
+    const float (*orcodata)[3];
 
-    orcodata = static_cast<const float(*)[3]>(CustomData_get_layer(&mesh->vert_data, CD_ORCO));
+    orcodata = static_cast<const float (*)[3]>(CustomData_get_layer(&mesh->vert_data, CD_ORCO));
 
     MFace *mfaces = (MFace *)CustomData_get_layer_for_write(
         &mesh->fdata_legacy, CD_MFACE, mesh->totface_legacy);
@@ -1107,9 +1102,7 @@ static int psys_thread_context_init_distribute(ParticleThreadContext *ctx,
 
       cur = mf->v4 ? area_quad_v3(co1, co2, co3, co4) : area_tri_v3(co1, co2, co3);
 
-      if (cur > maxweight) {
-        maxweight = cur;
-      }
+      maxweight = std::max(cur, maxweight);
 
       element_weight[i] = cur;
       totarea += cur;
@@ -1122,7 +1115,7 @@ static int psys_thread_context_init_distribute(ParticleThreadContext *ctx,
     maxweight /= totarea;
   }
   else {
-    float min = 1.0f / float(MIN2(totelem, totpart));
+    float min = 1.0f / float(std::min(totelem, totpart));
     for (i = 0; i < totelem; i++) {
       element_weight[i] = min;
     }
@@ -1190,9 +1183,8 @@ static int psys_thread_context_init_distribute(ParticleThreadContext *ctx,
    * This simplifies greatly the filtering of zero-weighted items - and can be much more efficient
    * especially in random case (reducing a lot the size of binary-searched array)...
    */
-  float *element_sum = static_cast<float *>(
-      MEM_mallocN(sizeof(*element_sum) * totmapped, __func__));
-  int *element_map = static_cast<int *>(MEM_mallocN(sizeof(*element_map) * totmapped, __func__));
+  float *element_sum = MEM_malloc_arrayN<float>(size_t(totmapped), __func__);
+  int *element_map = MEM_malloc_arrayN<int>(size_t(totmapped), __func__);
   int i_mapped = 0;
 
   for (i = 0; i < totelem && element_weight[i] == 0.0f; i++) {
@@ -1260,7 +1252,7 @@ static int psys_thread_context_init_distribute(ParticleThreadContext *ctx,
     const int *orig_index = nullptr;
 
     if (from == PART_FROM_VERT) {
-      if (mesh->totvert) {
+      if (mesh->verts_num) {
         orig_index = static_cast<const int *>(
             CustomData_get_layer(&mesh->vert_data, CD_ORIGINDEX));
       }
@@ -1290,12 +1282,10 @@ static int psys_thread_context_init_distribute(ParticleThreadContext *ctx,
       if (part->flag & PART_EDISTR) {
         jitlevel *= 2; /* looks better in general, not very scientific */
       }
-      if (jitlevel < 3) {
-        jitlevel = 3;
-      }
+      jitlevel = std::max(jitlevel, 3);
     }
 
-    jit = static_cast<float *>(MEM_callocN((2 + jitlevel * 2) * sizeof(float), "jit"));
+    jit = MEM_calloc_arrayN<float>(2 + size_t(jitlevel * 2), "jit");
 
     /* for small amounts of particles we use regular jitter since it looks
      * a bit better, for larger amounts we switch to hammersley sequence
@@ -1345,30 +1335,25 @@ static void psys_task_init_distribute(ParticleTask *task, ParticleSimulationData
 
 static void distribute_particles_on_dm(ParticleSimulationData *sim, int from)
 {
-  TaskPool *task_pool;
   ParticleThreadContext ctx;
-  ParticleTask *tasks;
   Mesh *final_mesh = sim->psmd->mesh_final;
-  int i, totpart, numtasks;
 
   /* create a task pool for distribution tasks */
   if (!psys_thread_context_init_distribute(&ctx, sim, from)) {
     return;
   }
 
-  task_pool = BLI_task_pool_create(&ctx, TASK_PRIORITY_HIGH);
+  TaskPool *task_pool = BLI_task_pool_create(&ctx, TASK_PRIORITY_HIGH);
 
-  totpart = (from == PART_FROM_CHILD ? sim->psys->totchild : sim->psys->totpart);
-  psys_tasks_create(&ctx, 0, totpart, &tasks, &numtasks);
-  for (i = 0; i < numtasks; i++) {
-    ParticleTask *task = &tasks[i];
-
-    psys_task_init_distribute(task, sim);
+  const int totpart = (from == PART_FROM_CHILD ? sim->psys->totchild : sim->psys->totpart);
+  blender::Vector<ParticleTask> tasks = psys_tasks_create(&ctx, 0, totpart);
+  for (ParticleTask &task : tasks) {
+    psys_task_init_distribute(&task, sim);
     if (from == PART_FROM_CHILD) {
-      BLI_task_pool_push(task_pool, exec_distribute_child, task, false, nullptr);
+      BLI_task_pool_push(task_pool, exec_distribute_child, &task, false, nullptr);
     }
     else {
-      BLI_task_pool_push(task_pool, exec_distribute_parent, task, false, nullptr);
+      BLI_task_pool_push(task_pool, exec_distribute_parent, &task, false, nullptr);
     }
   }
   BLI_task_pool_work_and_wait(task_pool);
@@ -1381,7 +1366,7 @@ static void distribute_particles_on_dm(ParticleSimulationData *sim, int from)
     BKE_id_free(nullptr, ctx.mesh);
   }
 
-  psys_tasks_free(tasks, numtasks);
+  psys_tasks_free(tasks);
 
   psys_thread_context_free(&ctx);
 }
