@@ -10,6 +10,7 @@
 
 #include "BLI_math_rotation.hh"
 #include "BLI_simd.hh"
+#include "BLI_task.hh"
 
 #include <Eigen/Core>
 #include <Eigen/Dense>
@@ -282,8 +283,8 @@ template double4x4 pseudo_invert(const double4x4 &mat, double epsilon);
  * Right polar decomposition:
  *     M = UP
  *
- * U is the 'rotation'-like component, the closest orthogonal matrix to M.
- * P is the 'scaling'-like component, defined in U space.
+ * U is the *rotation*-like component, the closest orthogonal matrix to M.
+ * P is the *scaling*-like component, defined in U space.
  *
  * See https://en.wikipedia.org/wiki/Polar_decomposition for more.
  */
@@ -315,7 +316,7 @@ static void polar_decompose(const MatBase<T, 3, 3> &mat3,
 
     Eigen::Map<MatrixT>(W.base_ptr()) = svd.matrixU();
     (Eigen::Map<VectorT>(S_val)) = svd.singularValues();
-    Map<MatrixT>(V.base_ptr()) = svd.matrixV();
+    Eigen::Map<MatrixT>(V.base_ptr()) = svd.matrixV();
   }
 
   MatBase<T, 3, 3> S = from_scale<MatBase<T, 3, 3>>(S_val);
@@ -517,4 +518,140 @@ template float4x4 perspective_infinite(
 
 /** \} */
 
+/**
+ * Check that each column is orthogonal to the others, and that each column is the same length.
+ * In other words, there is no shear, and any scaling is uniform.
+ */
+template<typename T>
+bool is_similarity_transform(const MatBase<T, 3, 3> &matrix, const T &epsilon = 1e-6)
+{
+  if (math::abs(math::dot(matrix[0], matrix[1])) > epsilon) {
+    return false;
+  }
+  if (math::abs(math::dot(matrix[0], matrix[2])) > epsilon) {
+    return false;
+  }
+  if (math::abs(math::dot(matrix[1], matrix[2])) > epsilon) {
+    return false;
+  }
+  const float length_0 = math::length_squared(matrix[0]);
+  const float length_1 = math::length_squared(matrix[1]);
+  const float length_2 = math::length_squared(matrix[2]);
+  if (math::abs(length_0 - length_1) > epsilon) {
+    return false;
+  }
+  if (math::abs(length_0 - length_2) > epsilon) {
+    return false;
+  }
+  if (math::abs(length_1 - length_2) > epsilon) {
+    return false;
+  }
+  return true;
+}
+
+void transform_normals(const float3x3 &transform, MutableSpan<float3> normals)
+{
+  if (math::is_equal(transform, float3x3::identity(), 1e-6f)) {
+    return;
+  }
+  const float3x3 normal_transform = math::transpose(math::invert(transform));
+  if (is_similarity_transform(normal_transform)) {
+    const float3x3 normalized_transform = math::normalize(normal_transform);
+    threading::parallel_for(normals.index_range(), 1024, [&](const IndexRange range) {
+      for (float3 &normal : normals.slice(range)) {
+        normal = normalized_transform * normal;
+      }
+    });
+  }
+  else {
+    threading::parallel_for(normals.index_range(), 1024, [&](const IndexRange range) {
+      for (float3 &normal : normals.slice(range)) {
+        normal = math::normalize(normal_transform * normal);
+      }
+    });
+  }
+}
+
+void transform_normals(Span<float3> src, const float3x3 &transform, MutableSpan<float3> dst)
+{
+  if (math::is_equal(transform, float3x3::identity(), 1e-6f)) {
+    dst.copy_from(src);
+    return;
+  }
+  const float3x3 normal_transform = math::transpose(math::invert(transform));
+  if (is_similarity_transform(normal_transform)) {
+    const float3x3 normalized_transform = math::normalize(normal_transform);
+    threading::parallel_for(src.index_range(), 1024, [&](const IndexRange range) {
+      for (const int i : range) {
+        dst[i] = normalized_transform * src[i];
+      }
+    });
+  }
+  else {
+    threading::parallel_for(src.index_range(), 1024, [&](const IndexRange range) {
+      for (const int i : range) {
+        dst[i] = math::normalize(normal_transform * src[i]);
+      }
+    });
+  }
+}
+
+static bool skip_transform(const float4x4 &transform)
+{
+  return math::is_equal(transform, float4x4::identity(), 1e-6f);
+}
+
+static void transform_points_no_threading(const Span<float3> src,
+                                          const float4x4 &transform,
+                                          MutableSpan<float3> dst)
+{
+  for (const int64_t i : src.index_range()) {
+    dst[i] = math::transform_point(transform, src[i]);
+  }
+}
+
+void transform_points(const Span<float3> src,
+                      const float4x4 &transform,
+                      MutableSpan<float3> dst,
+                      const bool use_threading)
+{
+  if (skip_transform(transform)) {
+    dst.copy_from(src);
+  }
+  else {
+    if (use_threading) {
+      threading::parallel_for(src.index_range(), 1024, [&](const IndexRange range) {
+        transform_points_no_threading(src.slice(range), transform, dst.slice(range));
+      });
+    }
+    else {
+      transform_points_no_threading(src, transform, dst);
+    }
+  }
+}
+
+static void transform_points_no_threading(const float4x4 &transform, MutableSpan<float3> points)
+{
+  for (float3 &position : points) {
+    position = math::transform_point(transform, position);
+  }
+}
+
+void transform_points(const float4x4 &transform,
+                      MutableSpan<float3> points,
+                      const bool use_threading)
+{
+  if (skip_transform(transform)) {
+    return;
+  }
+  if (use_threading) {
+    threading::parallel_for(points.index_range(), 1024, [&](const IndexRange range) {
+      transform_points_no_threading(transform, points.slice(range));
+    });
+  }
+  else {
+    transform_points_no_threading(transform, points);
+  }
+}
+
 }  // namespace blender::math
