@@ -23,7 +23,9 @@
 
 using blender::Array;
 using blender::float3;
+using blender::float4x4;
 using blender::MutableSpan;
+using blender::Span;
 
 const BMAllocTemplate bm_mesh_allocsize_default = {512, 1024, 2048, 512};
 const BMAllocTemplate bm_mesh_chunksize_default = {512, 1024, 2048, 512};
@@ -130,7 +132,7 @@ void BM_mesh_elem_toolflags_clear(BMesh *bm)
 BMesh *BM_mesh_create(const BMAllocTemplate *allocsize, const BMeshCreateParams *params)
 {
   /* allocate the structure */
-  BMesh *bm = static_cast<BMesh *>(MEM_callocN(sizeof(BMesh), __func__));
+  BMesh *bm = MEM_callocN<BMesh>(__func__);
 
   /* allocate the memory pools for the mesh elements */
   bm_mempool_init(bm, allocsize, params->use_toolflags);
@@ -201,10 +203,10 @@ void BM_mesh_data_free(BMesh *bm)
   }
 
   /* free custom data */
-  CustomData_free(&bm->vdata, 0);
-  CustomData_free(&bm->edata, 0);
-  CustomData_free(&bm->ldata, 0);
-  CustomData_free(&bm->pdata, 0);
+  CustomData_free(&bm->vdata);
+  CustomData_free(&bm->edata);
+  CustomData_free(&bm->ldata);
+  CustomData_free(&bm->pdata);
 
   /* destroy element pools */
   BLI_mempool_destroy(bm->vpool);
@@ -246,7 +248,7 @@ void BM_mesh_clear(BMesh *bm)
 
   /* free old mesh */
   BM_mesh_data_free(bm);
-  memset(bm, 0, sizeof(BMesh));
+  *bm = BMesh{};
 
   /* allocate the memory pools for the mesh elements */
   bm_mempool_init(bm, &bm_mesh_allocsize_default, use_toolflags);
@@ -766,11 +768,8 @@ void BM_mesh_remap(BMesh *bm, const uint *vert_idx, const uint *edge_idx, const
 
     /* Make a copy of all vertices. */
     verts_pool = bm->vtable;
-    verts_copy = static_cast<BMVert *>(
-        MEM_mallocN(sizeof(BMVert) * totvert, "BM_mesh_remap verts copy"));
-    void **pyptrs = (cd_vert_pyptr != -1) ?
-                        static_cast<void **>(MEM_mallocN(sizeof(void *) * totvert, __func__)) :
-                        nullptr;
+    verts_copy = MEM_malloc_arrayN<BMVert>(totvert, "BM_mesh_remap verts copy");
+    void **pyptrs = (cd_vert_pyptr != -1) ? MEM_malloc_arrayN<void *>(totvert, __func__) : nullptr;
     for (i = totvert, ve = verts_copy + totvert - 1, vep = verts_pool + totvert - 1; i--;
          ve--, vep--)
     {
@@ -823,11 +822,8 @@ void BM_mesh_remap(BMesh *bm, const uint *vert_idx, const uint *edge_idx, const
 
     /* Make a copy of all vertices. */
     edges_pool = bm->etable;
-    edges_copy = static_cast<BMEdge *>(
-        MEM_mallocN(sizeof(BMEdge) * totedge, "BM_mesh_remap edges copy"));
-    void **pyptrs = (cd_edge_pyptr != -1) ?
-                        static_cast<void **>(MEM_mallocN(sizeof(void *) * totedge, __func__)) :
-                        nullptr;
+    edges_copy = MEM_malloc_arrayN<BMEdge>(totedge, "BM_mesh_remap edges copy");
+    void **pyptrs = (cd_edge_pyptr != -1) ? MEM_malloc_arrayN<void *>(totedge, __func__) : nullptr;
     for (i = totedge, ed = edges_copy + totedge - 1, edp = edges_pool + totedge - 1; i--;
          ed--, edp--)
     {
@@ -879,11 +875,8 @@ void BM_mesh_remap(BMesh *bm, const uint *vert_idx, const uint *edge_idx, const
 
     /* Make a copy of all vertices. */
     faces_pool = bm->ftable;
-    faces_copy = static_cast<BMFace *>(
-        MEM_mallocN(sizeof(BMFace) * totface, "BM_mesh_remap faces copy"));
-    void **pyptrs = (cd_poly_pyptr != -1) ?
-                        static_cast<void **>(MEM_mallocN(sizeof(void *) * totface, __func__)) :
-                        nullptr;
+    faces_copy = MEM_malloc_arrayN<BMFace>(totface, "BM_mesh_remap faces copy");
+    void **pyptrs = (cd_poly_pyptr != -1) ? MEM_malloc_arrayN<void *>(totface, __func__) : nullptr;
     for (i = totface, fa = faces_copy + totface - 1, fap = faces_pool + totface - 1; i--;
          fa--, fap--)
     {
@@ -1051,17 +1044,13 @@ void BM_mesh_rebuild(BMesh *bm,
   const char remap = (vpool_dst ? BM_VERT : 0) | (epool_dst ? BM_EDGE : 0) |
                      (lpool_dst ? BM_LOOP : 0) | (fpool_dst ? BM_FACE : 0);
 
-  BMVert **vtable_dst = (remap & BM_VERT) ? static_cast<BMVert **>(MEM_mallocN(
-                                                sizeof(BMVert *) * bm->totvert, __func__)) :
+  BMVert **vtable_dst = (remap & BM_VERT) ? MEM_malloc_arrayN<BMVert *>(bm->totvert, __func__) :
                                             nullptr;
-  BMEdge **etable_dst = (remap & BM_EDGE) ? static_cast<BMEdge **>(MEM_mallocN(
-                                                sizeof(BMEdge *) * bm->totedge, __func__)) :
+  BMEdge **etable_dst = (remap & BM_EDGE) ? MEM_malloc_arrayN<BMEdge *>(bm->totedge, __func__) :
                                             nullptr;
-  BMLoop **ltable_dst = (remap & BM_LOOP) ? static_cast<BMLoop **>(MEM_mallocN(
-                                                sizeof(BMLoop *) * bm->totloop, __func__)) :
+  BMLoop **ltable_dst = (remap & BM_LOOP) ? MEM_malloc_arrayN<BMLoop *>(bm->totloop, __func__) :
                                             nullptr;
-  BMFace **ftable_dst = (remap & BM_FACE) ? static_cast<BMFace **>(MEM_mallocN(
-                                                sizeof(BMFace *) * bm->totface, __func__)) :
+  BMFace **ftable_dst = (remap & BM_FACE) ? MEM_malloc_arrayN<BMFace *>(bm->totface, __func__) :
                                             nullptr;
 
   const bool use_toolflags = params->use_toolflags;
@@ -1361,7 +1350,7 @@ Array<float3> BM_mesh_vert_coords_alloc(BMesh *bm)
   return positions;
 }
 
-void BM_mesh_vert_coords_apply(BMesh *bm, const float (*vert_coords)[3])
+void BM_mesh_vert_coords_apply(BMesh *bm, const Span<float3> vert_coords)
 {
   BMIter iter;
   BMVert *v;
@@ -1372,14 +1361,14 @@ void BM_mesh_vert_coords_apply(BMesh *bm, const float (*vert_coords)[3])
 }
 
 void BM_mesh_vert_coords_apply_with_mat4(BMesh *bm,
-                                         const float (*vert_coords)[3],
-                                         const float mat[4][4])
+                                         const Span<float3> vert_coords,
+                                         const float4x4 &transform)
 {
   BMIter iter;
   BMVert *v;
   int i;
   BM_ITER_MESH_INDEX (v, &iter, bm, BM_VERTS_OF_MESH, i) {
-    mul_v3_m4v3(v->co, mat, vert_coords[i]);
+    mul_v3_m4v3(v->co, transform.ptr(), vert_coords[i]);
   }
 }
 
