@@ -8,19 +8,19 @@
  * Creates a solid wireframe from connected faces.
  */
 
+#include <algorithm>
+
 #include "MEM_guardedalloc.h"
 
-#include "DNA_meshdata_types.h"
-
-#include "bmesh.h"
+#include "bmesh.hh"
 
 #include "BLI_math_geom.h"
 #include "BLI_math_vector.h"
 
-#include "BKE_customdata.h"
-#include "BKE_deform.h"
+#include "BKE_customdata.hh"
+#include "BKE_deform.hh"
 
-#include "bmesh_wireframe.h"
+#include "bmesh_wireframe.hh"
 
 static BMLoop *bm_edge_tag_faceloop(BMEdge *e)
 {
@@ -154,7 +154,7 @@ void BM_mesh_wireframe(BMesh *bm,
                        const int defgrp_index,
                        const bool defgrp_invert,
                        const short mat_offset,
-                       const short mat_max,
+                       const int mat_max,
                        /* for operators */
                        const bool use_tag)
 {
@@ -176,12 +176,9 @@ void BM_mesh_wireframe(BMesh *bm,
   BMIter itersub;
 
   /* filled only with boundary verts */
-  BMVert **verts_src = static_cast<BMVert **>(
-      MEM_mallocN(sizeof(BMVert *) * totvert_orig, __func__));
-  BMVert **verts_neg = static_cast<BMVert **>(
-      MEM_mallocN(sizeof(BMVert *) * totvert_orig, __func__));
-  BMVert **verts_pos = static_cast<BMVert **>(
-      MEM_mallocN(sizeof(BMVert *) * totvert_orig, __func__));
+  BMVert **verts_src = MEM_malloc_arrayN<BMVert *>(totvert_orig, __func__);
+  BMVert **verts_neg = MEM_malloc_arrayN<BMVert *>(totvert_orig, __func__);
+  BMVert **verts_pos = MEM_malloc_arrayN<BMVert *>(totvert_orig, __func__);
 
   /* Will over-allocate, but makes for easy lookups by index to keep aligned. */
   BMVert **verts_boundary = static_cast<BMVert **>(
@@ -306,7 +303,7 @@ void BM_mesh_wireframe(BMesh *bm,
     BM_mesh_elem_hflag_disable_all(bm, BM_VERT, BM_ELEM_TAG, false);
   }
 
-  verts_loop = static_cast<BMVert **>(MEM_mallocN(sizeof(BMVert *) * verts_loop_tot, __func__));
+  verts_loop = MEM_malloc_arrayN<BMVert *>(verts_loop_tot, __func__);
   verts_loop_tot = 0; /* count up again */
 
   BM_ITER_MESH (f_src, &iter, bm, BM_FACES_OF_MESH) {
@@ -423,28 +420,28 @@ void BM_mesh_wireframe(BMesh *bm,
 
       f_new = BM_face_create_quad_tri(bm, v_l1, v_l2, v_neg2, v_neg1, f_src, BM_CREATE_NOP);
       if (mat_offset) {
-        f_new->mat_nr = CLAMPIS(f_new->mat_nr + mat_offset, 0, mat_max);
+        f_new->mat_nr = std::clamp(f_new->mat_nr + mat_offset, 0, mat_max);
       }
       BM_elem_flag_enable(f_new, BM_ELEM_TAG);
       l_new = BM_FACE_FIRST_LOOP(f_new);
 
-      BM_elem_attrs_copy(bm, bm, l, l_new);
-      BM_elem_attrs_copy(bm, bm, l, l_new->prev);
-      BM_elem_attrs_copy(bm, bm, l_next, l_new->next);
-      BM_elem_attrs_copy(bm, bm, l_next, l_new->next->next);
+      BM_elem_attrs_copy(bm, l, l_new);
+      BM_elem_attrs_copy(bm, l, l_new->prev);
+      BM_elem_attrs_copy(bm, l_next, l_new->next);
+      BM_elem_attrs_copy(bm, l_next, l_new->next->next);
 
       f_new = BM_face_create_quad_tri(bm, v_l2, v_l1, v_pos1, v_pos2, f_src, BM_CREATE_NOP);
 
       if (mat_offset) {
-        f_new->mat_nr = CLAMPIS(f_new->mat_nr + mat_offset, 0, mat_max);
+        f_new->mat_nr = std::clamp(f_new->mat_nr + mat_offset, 0, mat_max);
       }
       BM_elem_flag_enable(f_new, BM_ELEM_TAG);
       l_new = BM_FACE_FIRST_LOOP(f_new);
 
-      BM_elem_attrs_copy(bm, bm, l_next, l_new);
-      BM_elem_attrs_copy(bm, bm, l_next, l_new->prev);
-      BM_elem_attrs_copy(bm, bm, l, l_new->next);
-      BM_elem_attrs_copy(bm, bm, l, l_new->next->next);
+      BM_elem_attrs_copy(bm, l_next, l_new);
+      BM_elem_attrs_copy(bm, l_next, l_new->prev);
+      BM_elem_attrs_copy(bm, l, l_new->next);
+      BM_elem_attrs_copy(bm, l, l_new->next->next);
 
       if (use_boundary) {
         if (BM_elem_flag_test(l->e, BM_ELEM_TAG)) {
@@ -455,27 +452,27 @@ void BM_mesh_wireframe(BMesh *bm,
 
           f_new = BM_face_create_quad_tri(bm, v_b2, v_b1, v_neg1, v_neg2, f_src, BM_CREATE_NOP);
           if (mat_offset) {
-            f_new->mat_nr = CLAMPIS(f_new->mat_nr + mat_offset, 0, mat_max);
+            f_new->mat_nr = std::clamp(f_new->mat_nr + mat_offset, 0, mat_max);
           }
           BM_elem_flag_enable(f_new, BM_ELEM_TAG);
           l_new = BM_FACE_FIRST_LOOP(f_new);
 
-          BM_elem_attrs_copy(bm, bm, l_next, l_new);
-          BM_elem_attrs_copy(bm, bm, l_next, l_new->prev);
-          BM_elem_attrs_copy(bm, bm, l, l_new->next);
-          BM_elem_attrs_copy(bm, bm, l, l_new->next->next);
+          BM_elem_attrs_copy(bm, l_next, l_new);
+          BM_elem_attrs_copy(bm, l_next, l_new->prev);
+          BM_elem_attrs_copy(bm, l, l_new->next);
+          BM_elem_attrs_copy(bm, l, l_new->next->next);
 
           f_new = BM_face_create_quad_tri(bm, v_b1, v_b2, v_pos2, v_pos1, f_src, BM_CREATE_NOP);
           if (mat_offset) {
-            f_new->mat_nr = CLAMPIS(f_new->mat_nr + mat_offset, 0, mat_max);
+            f_new->mat_nr = std::clamp(f_new->mat_nr + mat_offset, 0, mat_max);
           }
           BM_elem_flag_enable(f_new, BM_ELEM_TAG);
           l_new = BM_FACE_FIRST_LOOP(f_new);
 
-          BM_elem_attrs_copy(bm, bm, l, l_new);
-          BM_elem_attrs_copy(bm, bm, l, l_new->prev);
-          BM_elem_attrs_copy(bm, bm, l_next, l_new->next);
-          BM_elem_attrs_copy(bm, bm, l_next, l_new->next->next);
+          BM_elem_attrs_copy(bm, l, l_new);
+          BM_elem_attrs_copy(bm, l, l_new->prev);
+          BM_elem_attrs_copy(bm, l_next, l_new->next);
+          BM_elem_attrs_copy(bm, l_next, l_new->next->next);
 
           if (use_crease) {
             BMEdge *e_new;
