@@ -6,6 +6,7 @@
  * \ingroup edcurves
  */
 
+#include "BLI_listbase.h"
 #include "BLI_math_base_safe.h"
 #include "BLI_rand.hh"
 
@@ -75,7 +76,7 @@ void ensure_surface_deformation_node_exists(bContext &C, Object &curves_ob)
   nmd.node_group = bke::node_tree_add_tree(bmain, DATA_("Surface Deform"), "GeometryNodeTree");
 
   if (!nmd.node_group->geometry_node_asset_traits) {
-    nmd.node_group->geometry_node_asset_traits = MEM_cnew<GeometryNodeAssetTraits>(__func__);
+    nmd.node_group->geometry_node_asset_traits = MEM_callocN<GeometryNodeAssetTraits>(__func__);
   }
 
   nmd.node_group->geometry_node_asset_traits->flag |= GEO_NODE_ASSET_MODIFIER;
@@ -85,22 +86,22 @@ void ensure_surface_deformation_node_exists(bContext &C, Object &curves_ob)
       "Geometry", "", "NodeSocketGeometry", NODE_INTERFACE_SOCKET_OUTPUT, nullptr);
   ntree->tree_interface.add_socket(
       "Geometry", "", "NodeSocketGeometry", NODE_INTERFACE_SOCKET_INPUT, nullptr);
-  bNode *group_input = bke::node_add_static_node(&C, ntree, NODE_GROUP_INPUT);
-  bNode *group_output = bke::node_add_static_node(&C, ntree, NODE_GROUP_OUTPUT);
-  bNode *deform_node = bke::node_add_static_node(&C, ntree, GEO_NODE_DEFORM_CURVES_ON_SURFACE);
+  bNode *group_input = bke::node_add_static_node(&C, *ntree, NODE_GROUP_INPUT);
+  bNode *group_output = bke::node_add_static_node(&C, *ntree, NODE_GROUP_OUTPUT);
+  bNode *deform_node = bke::node_add_static_node(&C, *ntree, GEO_NODE_DEFORM_CURVES_ON_SURFACE);
 
   BKE_main_ensure_invariants(*bmain, nmd.node_group->id);
 
-  bke::node_add_link(ntree,
-                     group_input,
-                     static_cast<bNodeSocket *>(group_input->outputs.first),
-                     deform_node,
-                     bke::node_find_socket(deform_node, SOCK_IN, "Curves"));
-  bke::node_add_link(ntree,
-                     deform_node,
-                     bke::node_find_socket(deform_node, SOCK_OUT, "Curves"),
-                     group_output,
-                     static_cast<bNodeSocket *>(group_output->inputs.first));
+  bke::node_add_link(*ntree,
+                     *group_input,
+                     *static_cast<bNodeSocket *>(group_input->outputs.first),
+                     *deform_node,
+                     *bke::node_find_socket(*deform_node, SOCK_IN, "Curves"));
+  bke::node_add_link(*ntree,
+                     *deform_node,
+                     *bke::node_find_socket(*deform_node, SOCK_OUT, "Curves"),
+                     *group_output,
+                     *static_cast<bNodeSocket *>(group_output->inputs.first));
 
   group_input->location[0] = -200;
   group_output->location[0] = 200;
