@@ -9,6 +9,8 @@
 #include <algorithm>
 
 #include "BKE_curves.hh"
+
+#include "BLI_listbase.h"
 #include "BLI_map.hh"
 #include "BLI_math_vector_types.hh"
 #include "BLI_utildefines.h"
@@ -319,7 +321,7 @@ static void append_frame_to_key_edit_data(KeyframeEditData *ked,
                                           const int frame_number,
                                           const GreasePencilFrame &frame)
 {
-  CfraElem *ce = MEM_cnew<CfraElem>(__func__);
+  CfraElem *ce = MEM_callocN<CfraElem>(__func__);
   ce->cfra = float(frame_number);
   ce->sel = frame.is_selected();
   BLI_addtail(&ked->list, ce);
@@ -373,7 +375,7 @@ bool ensure_active_keyframe(const Scene &scene,
   return true;
 }
 
-static int insert_blank_frame_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus insert_blank_frame_exec(bContext *C, wmOperator *op)
 {
   using namespace blender::bke::greasepencil;
   Scene *scene = CTX_data_scene(C);
@@ -503,7 +505,7 @@ static bool curves_geometry_is_equal(const bke::CurvesGeometry &curves_a,
   return true;
 }
 
-static int frame_clean_duplicate_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus frame_clean_duplicate_exec(bContext *C, wmOperator *op)
 {
   using namespace blender::bke::greasepencil;
   Object *object = CTX_data_active_object(C);
@@ -576,7 +578,7 @@ static void GREASE_PENCIL_OT_insert_blank_frame(wmOperatorType *ot)
 
   /* callbacks */
   ot->exec = insert_blank_frame_exec;
-  ot->poll = active_grease_pencil_poll;
+  ot->poll = editable_grease_pencil_poll;
 
   ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 
@@ -598,7 +600,7 @@ static void GREASE_PENCIL_OT_frame_clean_duplicate(wmOperatorType *ot)
 
   /* callbacks */
   ot->exec = frame_clean_duplicate_exec;
-  ot->poll = active_grease_pencil_poll;
+  ot->poll = editable_grease_pencil_poll;
 
   ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 
@@ -624,7 +626,7 @@ bool grease_pencil_copy_keyframes(bAnimContext *ac, KeyframeClipboard &clipboard
 
   LISTBASE_FOREACH (bAnimListElem *, ale, &anim_data) {
     /* This function only deals with grease pencil layer frames.
-     * This check is needed in the case of a call from the main dopesheet. */
+     * This check is needed in the case of a call from the main dope-sheet. */
     if (ale->type != ANIMTYPE_GREASE_PENCIL_LAYER) {
       continue;
     }
@@ -712,7 +714,7 @@ bool grease_pencil_paste_keyframes(bAnimContext *ac,
   const bool from_single_channel = clipboard.copy_buffer.size() == 1;
 
   LISTBASE_FOREACH (bAnimListElem *, ale, &anim_data) {
-    /* Only deal with GPlayers (case of calls from general dopesheet). */
+    /* Only deal with GPlayers (case of calls from general dope-sheet). */
     if (ale->type != ANIMTYPE_GREASE_PENCIL_LAYER) {
       continue;
     }
@@ -794,7 +796,7 @@ bool grease_pencil_paste_keyframes(bAnimContext *ac,
   return true;
 }
 
-static int grease_pencil_frame_duplicate_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus grease_pencil_frame_duplicate_exec(bContext *C, wmOperator *op)
 {
   using namespace blender::bke::greasepencil;
   Scene *scene = CTX_data_scene(C);
@@ -846,7 +848,7 @@ static void GREASE_PENCIL_OT_frame_duplicate(wmOperatorType *ot)
 
   /* callback */
   ot->exec = grease_pencil_frame_duplicate_exec;
-  ot->poll = active_grease_pencil_poll;
+  ot->poll = editable_grease_pencil_poll;
 
   /* flags */
   ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -855,7 +857,7 @@ static void GREASE_PENCIL_OT_frame_duplicate(wmOperatorType *ot)
       ot->srna, "all", false, "Duplicate all", "Duplicate active keyframes of all layer");
 }
 
-static int grease_pencil_active_frame_delete_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus grease_pencil_active_frame_delete_exec(bContext *C, wmOperator *op)
 {
   using namespace blender::bke::greasepencil;
   Scene *scene = CTX_data_scene(C);
@@ -904,7 +906,7 @@ static void GREASE_PENCIL_OT_active_frame_delete(wmOperatorType *ot)
 
   /* callback */
   ot->exec = grease_pencil_active_frame_delete_exec;
-  ot->poll = active_grease_pencil_poll;
+  ot->poll = editable_grease_pencil_poll;
 
   /* flags */
   ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -931,7 +933,8 @@ static bool grease_pencil_active_breakdown_frame_poll(bContext *C)
   return false;
 }
 
-static int grease_pencil_delete_breakdown_frames_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus grease_pencil_delete_breakdown_frames_exec(bContext *C,
+                                                                   wmOperator * /*op*/)
 {
   const Object &ob = *CTX_data_active_object(C);
   const Scene &scene = *CTX_data_scene(C);
