@@ -17,13 +17,18 @@
 #include "BLI_utildefines.h"
 
 #include "BKE_context.hh"
+#include "BKE_idtype.hh"
 #include "BKE_layer.hh"
+#include "BKE_lib_id.hh"
+#include "BKE_library.hh"
 #include "BKE_modifier.hh"
 #include "BKE_object.hh"
 #include "BKE_paint.hh"
+#include "BKE_paint_types.hh"
 #include "BKE_report.hh"
 
 #include "BLI_math_vector.h"
+#include "BLI_string.h"
 
 #include "WM_api.hh"
 #include "WM_types.hh"
@@ -152,7 +157,7 @@ bool mode_compat_set(bContext *C, Object *ob, eObjectMode mode, ReportList *repo
   if (!ELEM(ob->mode, mode, OB_MODE_OBJECT)) {
     const char *opstring = object_mode_op_string(eObjectMode(ob->mode));
 
-    WM_operator_name_call(C, opstring, WM_OP_EXEC_REGION_WIN, nullptr, nullptr);
+    WM_operator_name_call(C, opstring, wm::OpCallContext::ExecRegionWin, nullptr, nullptr);
     ok = ELEM(ob->mode, mode, OB_MODE_OBJECT);
     if (!ok) {
       wmOperatorType *ot = WM_operatortype_find(opstring, false);
@@ -203,12 +208,40 @@ bool mode_set_ex(bContext *C, eObjectMode mode, bool use_undo, ReportList *repor
   if (!use_undo) {
     wm->op_undo_depth++;
   }
-  WM_operator_name_call_ptr(C, ot, WM_OP_EXEC_REGION_WIN, nullptr, nullptr);
+  WM_operator_name_call_ptr(C, ot, wm::OpCallContext::ExecRegionWin, nullptr, nullptr);
   if (!use_undo) {
     wm->op_undo_depth--;
   }
 
   if (ob->mode != mode) {
+    /* Give more specific error messages for cases that are known to fail (like linked and packed
+     * object-data). */
+    if (ob->data && !ID_IS_EDITABLE(ob->data)) {
+      const ID &obdata_id = *static_cast<ID *>(ob->data);
+      char obdata_idtype_name_lower[MAX_ID_NAME];
+      STRNCPY(obdata_idtype_name_lower, BKE_idtype_idcode_to_name(GS(obdata_id.name)));
+      BLI_str_tolower_ascii(obdata_idtype_name_lower, strlen(obdata_idtype_name_lower));
+
+      if (ID_IS_PACKED(static_cast<ID *>(ob->data))) {
+        BKE_reportf(reports,
+                    RPT_ERROR,
+                    "The '%s' %s data-block is packed and not editable. Use \"Make Local\" to "
+                    "make it editable.",
+                    BKE_id_name(obdata_id),
+                    obdata_idtype_name_lower);
+        return false;
+      }
+      if (ID_IS_LINKED(ob->data)) {
+        BKE_reportf(reports,
+                    RPT_ERROR,
+                    "The '%s' %s data-block is linked and not editable. Use \"Make Local\" to "
+                    "make it editable.",
+                    BKE_id_name(obdata_id),
+                    obdata_idtype_name_lower);
+        return false;
+      }
+    }
+
     BKE_reportf(reports, RPT_ERROR, "Unable to execute '%s', error changing modes", ot->name);
     return false;
   }
@@ -394,24 +427,63 @@ static bool object_transfer_mode_poll(bContext *C)
 
 /* Update the viewport rotation origin to the mouse cursor. */
 static void object_transfer_mode_reposition_view_pivot(ARegion *region,
-                                                       Scene *scene,
+                                                       Paint *paint,
                                                        const int mval[2])
 {
   float global_loc[3];
   if (!ED_view3d_autodist_simple(region, mval, global_loc, 0, nullptr)) {
     return;
   }
-  UnifiedPaintSettings *ups = &scene->toolsettings->unified_paint_settings;
-  copy_v3_v3(ups->average_stroke_accum, global_loc);
-  ups->average_stroke_counter = 1;
-  ups->last_stroke_valid = true;
+  bke::PaintRuntime *paint_runtime = paint->runtime;
+  copy_v3_v3(paint_runtime->average_stroke_accum, global_loc);
+  paint_runtime->average_stroke_counter = 1;
+  paint_runtime->last_stroke_valid = true;
+}
+
+constexpr float mode_transfer_flash_length = 0.55f;
+
+static auto &mode_transfer_overlay_start_times()
+{
+  static Map<std::string, double> map;
+  return map;
+}
+
+static float alpha_from_time_get(const float anim_time)
+{
+  if (anim_time < 0.0f) {
+    return 0.0f;
+  }
+  return (1.0f - (anim_time / mode_transfer_flash_length));
+}
+
+Map<std::string, float, 1> mode_transfer_overlay_current_state()
+{
+  const double now = BLI_time_now_seconds();
+
+  /* Protect against possible concurrent access from multiple renderers or viewports. */
+  static Mutex mutex;
+  std::scoped_lock lock(mutex);
+
+  /* Remove finished animations form the global map. */
+  Map<std::string, double> &start_times = mode_transfer_overlay_start_times();
+  start_times.remove_if(
+      [&](const auto &item) { return (now - item.value) > mode_transfer_flash_length; });
+
+  Map<std::string, float, 1> factors;
+  for (const auto &item : start_times.items()) {
+    const float alpha = alpha_from_time_get(now - item.value);
+    if (alpha > 0.0f) {
+      factors.add_new(item.key, alpha);
+    }
+  }
+  return factors;
 }
 
 static void object_overlay_mode_transfer_animation_start(bContext *C, Object *ob_dst)
 {
   Depsgraph *depsgraph = CTX_data_depsgraph_pointer(C);
-  Object *ob_dst_eval = DEG_get_evaluated_object(depsgraph, ob_dst);
-  ob_dst_eval->runtime->overlay_mode_transfer_start_time = BLI_time_now_seconds();
+  Object *ob_dst_eval = DEG_get_evaluated(depsgraph, ob_dst);
+  mode_transfer_overlay_start_times().add_as(ob_dst_eval->id.name, BLI_time_now_seconds());
 }
 
 static bool object_transfer_mode_to_base(bContext *C,
@@ -455,7 +527,9 @@ static bool object_transfer_mode_to_base(bContext *C,
   return mode_transferred;
 }
 
-static int object_transfer_mode_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus object_transfer_mode_invoke(bContext *C,
+                                                    wmOperator *op,
+                                                    const wmEvent *event)
 {
   Scene *scene = CTX_data_scene(C);
   ARegion *region = CTX_wm_region(C);
@@ -512,7 +586,8 @@ static int object_transfer_mode_invoke(bContext *C, wmOperator *op, const wmEven
 
   WM_toolsystem_update_from_context_view3d(C);
   if (mode_src & OB_MODE_ALL_PAINT) {
-    object_transfer_mode_reposition_view_pivot(region, scene, event->mval);
+    Paint *paint = BKE_paint_get_active_from_context(C);
+    object_transfer_mode_reposition_view_pivot(region, paint, event->mval);
   }
 
   return OPERATOR_FINISHED;
@@ -527,7 +602,7 @@ void OBJECT_OT_transfer_mode(wmOperatorType *ot)
       "Switches the active object and assigns the same mode to a new one under the mouse cursor, "
       "leaving the active mode in the current one";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = object_transfer_mode_invoke;
   ot->poll = object_transfer_mode_poll;
 
