@@ -14,6 +14,7 @@
 
 #include "MEM_guardedalloc.h"
 
+#include "BLI_bounds.hh"
 #include "BLI_math_geom.h"
 #include "BLI_math_matrix.h"
 #include "BLI_string_utf8.h"
@@ -28,6 +29,7 @@
 #include "BKE_context.hh"
 #include "BKE_global.hh"
 #include "BKE_lib_id.hh"
+#include "BKE_library.hh"
 #include "BKE_main.hh"
 #include "BKE_mesh.hh"
 #include "BKE_mesh_mirror.hh"
@@ -105,7 +107,7 @@ static bool object_remesh_poll(bContext *C)
   return ED_operator_object_active_editable_mesh(C);
 }
 
-static int voxel_remesh_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus voxel_remesh_exec(bContext *C, wmOperator *op)
 {
   const Scene &scene = *CTX_data_scene(C);
   Object *ob = CTX_data_active_object(C);
@@ -158,12 +160,13 @@ static int voxel_remesh_exec(bContext *C, wmOperator *op)
   }
 
   BKE_mesh_nomain_to_mesh(new_mesh, mesh, ob);
+  /* Spatially organize the mesh after remesh. */
+  blender::bke::mesh_apply_spatial_organization(*mesh);
 
   if (ob->mode == OB_MODE_SCULPT) {
     sculpt_paint::undo::geometry_end(*ob);
     BKE_sculptsession_free_pbvh(*ob);
   }
-
   BKE_mesh_batch_cache_dirty_tag(static_cast<Mesh *>(ob->data), BKE_MESH_BATCH_DIRTY_ALL);
   DEG_id_tag_update(&ob->id, ID_RECALC_GEOMETRY);
   WM_event_add_notifier(C, NC_GEOM | ND_DATA, ob->data);
@@ -180,7 +183,7 @@ void OBJECT_OT_voxel_remesh(wmOperatorType *ot)
       "will be lost";
   ot->idname = "OBJECT_OT_voxel_remesh";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->poll = object_remesh_poll;
   ot->exec = voxel_remesh_exec;
 
@@ -270,7 +273,8 @@ static void voxel_size_edit_draw(const bContext *C, ARegion * /*region*/, void *
   GPU_blend(GPU_BLEND_ALPHA);
   GPU_line_smooth(true);
 
-  uint pos3d = GPU_vertformat_attr_add(immVertexFormat(), "pos", GPU_COMP_F32, 3, GPU_FETCH_FLOAT);
+  uint pos3d = GPU_vertformat_attr_add(
+      immVertexFormat(), "pos", blender::gpu::VertAttrType::SFLOAT_32_32_32);
   immBindBuiltinProgram(GPU_SHADER_3D_UNIFORM_COLOR);
   GPU_matrix_push();
   GPU_matrix_mul(cd->active_object->object_to_world().ptr());
@@ -350,7 +354,7 @@ static void voxel_size_edit_cancel(bContext *C, wmOperator *op)
 
   ED_region_draw_cb_exit(region->runtime->type, cd->draw_handle);
 
-  MEM_freeN(op->customdata);
+  MEM_freeN(cd);
 
   ED_workspace_status_text(C, nullptr);
 }
@@ -365,7 +369,7 @@ static void voxel_size_edit_update_header(wmOperator *op, bContext *C)
   status.item_bool(IFACE_("Precision Mode"), cd->slow_mode, ICON_EVENT_SHIFT);
 }
 
-static int voxel_size_edit_modal(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus voxel_size_edit_modal(bContext *C, wmOperator *op, const wmEvent *event)
 {
   ARegion *region = CTX_wm_region(C);
   VoxelSizeEditCustomData *cd = static_cast<VoxelSizeEditCustomData *>(op->customdata);
@@ -388,7 +392,7 @@ static int voxel_size_edit_modal(bContext *C, wmOperator *op, const wmEvent *eve
   {
     ED_region_draw_cb_exit(region->runtime->type, cd->draw_handle);
     mesh->remesh_voxel_size = cd->voxel_size;
-    MEM_freeN(op->customdata);
+    MEM_freeN(cd);
     ED_region_tag_redraw(region);
     ED_workspace_status_text(C, nullptr);
     WM_event_add_notifier(C, NC_GEOM | ND_DATA, nullptr);
@@ -431,13 +435,13 @@ static int voxel_size_edit_modal(bContext *C, wmOperator *op, const wmEvent *eve
   return OPERATOR_RUNNING_MODAL;
 }
 
-static int voxel_size_edit_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus voxel_size_edit_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
   ARegion *region = CTX_wm_region(C);
   Object *active_object = CTX_data_active_object(C);
   Mesh *mesh = (Mesh *)active_object->data;
 
-  VoxelSizeEditCustomData *cd = MEM_cnew<VoxelSizeEditCustomData>(
+  VoxelSizeEditCustomData *cd = MEM_callocN<VoxelSizeEditCustomData>(
       "Voxel Size Edit OP Custom Data");
 
   /* Initial operator Custom Data setup. */
@@ -446,15 +450,12 @@ static int voxel_size_edit_invoke(bContext *C, wmOperator *op, const wmEvent *ev
   cd->active_object = active_object;
   cd->init_mval[0] = event->mval[0];
   cd->init_mval[1] = event->mval[1];
-  cd->init_voxel_size = mesh->remesh_voxel_size;
-  cd->voxel_size = mesh->remesh_voxel_size;
   cd->slow_mode = false;
   op->customdata = cd;
 
   /* Select the front facing face of the mesh bounding box. */
   const Bounds<float3> bounds = *mesh->bounds_min_max();
-  BoundBox bb;
-  BKE_boundbox_init_from_minmax(&bb, bounds.min, bounds.max);
+  const std::array<float3, 8> bounds_box = bounds::corners(bounds);
 
   /* Indices of the Bounding Box faces. */
   const int BB_faces[6][4] = {
@@ -466,10 +467,10 @@ static int voxel_size_edit_invoke(bContext *C, wmOperator *op, const wmEvent *ev
       {2, 3, 7, 6},
   };
 
-  copy_v3_v3(cd->preview_plane[0], bb.vec[BB_faces[0][0]]);
-  copy_v3_v3(cd->preview_plane[1], bb.vec[BB_faces[0][1]]);
-  copy_v3_v3(cd->preview_plane[2], bb.vec[BB_faces[0][2]]);
-  copy_v3_v3(cd->preview_plane[3], bb.vec[BB_faces[0][3]]);
+  copy_v3_v3(cd->preview_plane[0], bounds_box[BB_faces[0][0]]);
+  copy_v3_v3(cd->preview_plane[1], bounds_box[BB_faces[0][1]]);
+  copy_v3_v3(cd->preview_plane[2], bounds_box[BB_faces[0][2]]);
+  copy_v3_v3(cd->preview_plane[3], bounds_box[BB_faces[0][3]]);
 
   RegionView3D *rv3d = CTX_wm_region_view3d(C);
 
@@ -493,16 +494,18 @@ static int voxel_size_edit_invoke(bContext *C, wmOperator *op, const wmEvent *ev
 
   /* Check if there is a face that is more aligned towards the view. */
   for (int i = 0; i < 6; i++) {
-    normal_tri_v3(
-        current_normal, bb.vec[BB_faces[i][0]], bb.vec[BB_faces[i][1]], bb.vec[BB_faces[i][2]]);
+    normal_tri_v3(current_normal,
+                  bounds_box[BB_faces[i][0]],
+                  bounds_box[BB_faces[i][1]],
+                  bounds_box[BB_faces[i][2]]);
     current_dot = dot_v3v3(current_normal, view_normal);
 
     if (current_dot < min_dot) {
       min_dot = current_dot;
-      copy_v3_v3(cd->preview_plane[0], bb.vec[BB_faces[i][0]]);
-      copy_v3_v3(cd->preview_plane[1], bb.vec[BB_faces[i][1]]);
-      copy_v3_v3(cd->preview_plane[2], bb.vec[BB_faces[i][2]]);
-      copy_v3_v3(cd->preview_plane[3], bb.vec[BB_faces[i][3]]);
+      copy_v3_v3(cd->preview_plane[0], bounds_box[BB_faces[i][0]]);
+      copy_v3_v3(cd->preview_plane[1], bounds_box[BB_faces[i][1]]);
+      copy_v3_v3(cd->preview_plane[2], bounds_box[BB_faces[i][2]]);
+      copy_v3_v3(cd->preview_plane[3], bounds_box[BB_faces[i][3]]);
     }
   }
 
@@ -512,6 +515,9 @@ static int voxel_size_edit_invoke(bContext *C, wmOperator *op, const wmEvent *ev
                               len_v3v3(cd->preview_plane[3], cd->preview_plane[0])) *
                        0.5f;
   cd->voxel_size_min = cd->voxel_size_max / VOXEL_SIZE_EDIT_MAX_GRIDS_LINES;
+  cd->init_voxel_size = clamp_f(
+      mesh->remesh_voxel_size, max_ff(cd->voxel_size_min, 0.0001f), cd->voxel_size_max);
+  cd->voxel_size = cd->init_voxel_size;
 
   /* Matrix calculation to position the text in 3D space. */
   float text_pos[3];
@@ -612,7 +618,7 @@ void OBJECT_OT_voxel_size_edit(wmOperatorType *ot)
   ot->description = "Modify the mesh voxel size interactively used in the voxel remesher";
   ot->idname = "OBJECT_OT_voxel_size_edit";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->poll = voxel_size_edit_poll;
   ot->invoke = voxel_size_edit_invoke;
   ot->modal = voxel_size_edit_modal;
@@ -635,6 +641,13 @@ enum {
   QUADRIFLOW_REMESH_FACES,
 };
 
+enum eQuadreFlowStatus {
+  QUADRIFLOW_STATUS_SUCCESS = 1,
+  QUADRIFLOW_STATUS_FAIL = 0,
+  QUADRIFLOW_STATUS_CANCELED = -1,
+  QUADRIFLOW_STATUS_NONMANIFOLD = -2,
+};
+
 enum eSymmetryAxes {
   SYMMETRY_AXES_X = (1 << 0),
   SYMMETRY_AXES_Y = (1 << 1),
@@ -644,8 +657,7 @@ enum eSymmetryAxes {
 struct QuadriFlowJob {
   /* from wmJob */
   Object *owner;
-  bool *stop, *do_update;
-  float *progress;
+  wmJobWorkerStatus *worker_status;
 
   const wmOperator *op;
   Scene *scene;
@@ -661,7 +673,7 @@ struct QuadriFlowJob {
   bool preserve_attributes;
   bool smooth_normals;
 
-  int success;
+  eQuadreFlowStatus status;
   bool is_nonblocking_job;
 };
 
@@ -677,9 +689,8 @@ static bool mesh_is_manifold_consistent(Mesh *mesh)
   const Span<int> corner_edges = mesh->corner_edges();
 
   bool is_manifold_consistent = true;
-  char *edge_faces = (char *)MEM_callocN(mesh->edges_num * sizeof(char), "remesh_manifold_check");
-  int *edge_vert = (int *)MEM_malloc_arrayN(
-      mesh->edges_num, sizeof(uint), "remesh_consistent_check");
+  char *edge_faces = MEM_calloc_arrayN<char>(mesh->edges_num, "remesh_manifold_check");
+  int *edge_vert = MEM_malloc_arrayN<int>(mesh->edges_num, "remesh_consistent_check");
 
   for (uint i = 0; i < mesh->edges_num; i++) {
     edge_vert[i] = -1;
@@ -738,13 +749,14 @@ static int quadriflow_break_job(void *customdata)
   // return *(qj->stop);
 
   /* this is not nice yet, need to make the jobs list template better
-   * for identifying/acting upon various different jobs */
+   * for identifying/acting upon various different jobs canceled */
   /* but for now we'll reuse the render break... */
   bool should_break = false;
+
   if (qj->is_nonblocking_job) {
     bool should_break = (G.is_break);
     if (should_break) {
-      qj->success = -1;
+      qj->status = QUADRIFLOW_STATUS_CANCELED;
     }
   }
 
@@ -763,8 +775,8 @@ static void quadriflow_update_job(void *customdata, float progress, int *cancel)
     *cancel = 0;
   }
 
-  *(qj->do_update) = true;
-  *(qj->progress) = progress;
+  qj->worker_status->do_update = true;
+  qj->worker_status->progress = progress;
 }
 
 static Mesh *remesh_symmetry_bisect(Mesh *mesh, eSymmetryAxes symmetry_axes)
@@ -833,10 +845,8 @@ static void quadriflow_start_job(void *customdata, wmJobWorkerStatus *worker_sta
 {
   QuadriFlowJob *qj = static_cast<QuadriFlowJob *>(customdata);
 
-  qj->stop = &worker_status->stop;
-  qj->do_update = &worker_status->do_update;
-  qj->progress = &worker_status->progress;
-  qj->success = 1;
+  qj->worker_status = worker_status;
+  qj->status = QUADRIFLOW_STATUS_SUCCESS;
 
   if (qj->is_nonblocking_job) {
     G.is_break = false; /* XXX shared with render - replace with job 'stop' switch */
@@ -850,7 +860,7 @@ static void quadriflow_start_job(void *customdata, wmJobWorkerStatus *worker_sta
 
   /* Check if the mesh is manifold. Quadriflow requires manifold meshes */
   if (!mesh_is_manifold_consistent(mesh)) {
-    qj->success = -2;
+    qj->status = QUADRIFLOW_STATUS_NONMANIFOLD;
     return;
   }
 
@@ -879,9 +889,9 @@ static void quadriflow_start_job(void *customdata, wmJobWorkerStatus *worker_sta
   if (new_mesh == nullptr) {
     worker_status->do_update = true;
     worker_status->stop = false;
-    if (qj->success == 1) {
+    if (qj->status == QUADRIFLOW_STATUS_SUCCESS) {
       /* This is not a user cancellation event. */
-      qj->success = 0;
+      qj->status = QUADRIFLOW_STATUS_FAIL;
     }
     return;
   }
@@ -905,7 +915,6 @@ static void quadriflow_start_job(void *customdata, wmJobWorkerStatus *worker_sta
     sculpt_paint::undo::geometry_end(*ob);
     BKE_sculptsession_free_pbvh(*ob);
   }
-
   BKE_mesh_batch_cache_dirty_tag(static_cast<Mesh *>(ob->data), BKE_MESH_BATCH_DIRTY_ALL);
 
   worker_status->do_update = true;
@@ -919,31 +928,35 @@ static void quadriflow_end_job(void *customdata)
   Object *ob = qj->owner;
 
   if (qj->is_nonblocking_job) {
-    WM_set_locked_interface(static_cast<wmWindowManager *>(G_MAIN->wm.first), false);
+    WM_locked_interface_set(static_cast<wmWindowManager *>(G_MAIN->wm.first), false);
   }
 
-  switch (qj->success) {
-    case 1:
+  ReportList *reports = qj->worker_status->reports;
+  switch (qj->status) {
+    case QUADRIFLOW_STATUS_SUCCESS:
+      /* Spatially organize the mesh after remesh. */
+      bke::mesh_apply_spatial_organization(*static_cast<Mesh *>(ob->data));
       DEG_id_tag_update(&ob->id, ID_RECALC_GEOMETRY);
-      WM_reportf(RPT_INFO, "QuadriFlow: Remeshing completed");
+      BKE_reportf(reports, RPT_INFO, "QuadriFlow: Remeshing completed");
       break;
-    case 0:
-      WM_reportf(RPT_ERROR, "QuadriFlow: Remeshing failed");
+    case QUADRIFLOW_STATUS_FAIL:
+      BKE_reportf(reports, RPT_ERROR, "QuadriFlow: Remeshing failed");
       break;
-    case -1:
-      WM_report(RPT_WARNING, "QuadriFlow: Remeshing canceled");
+    case QUADRIFLOW_STATUS_CANCELED:
+      BKE_report(reports, RPT_WARNING, "QuadriFlow: Remeshing canceled");
       break;
-    case -2:
-      WM_report(RPT_WARNING,
-                "QuadriFlow: The mesh needs to be manifold and have face normals that point in a "
-                "consistent direction");
+    case QUADRIFLOW_STATUS_NONMANIFOLD:
+      BKE_report(reports,
+                 RPT_WARNING,
+                 "QuadriFlow: The mesh needs to be manifold and have face normals that point in a "
+                 "consistent direction");
       break;
   }
 }
 
-static int quadriflow_remesh_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus quadriflow_remesh_exec(bContext *C, wmOperator *op)
 {
-  QuadriFlowJob *job = (QuadriFlowJob *)MEM_mallocN(sizeof(QuadriFlowJob), "QuadriFlowJob");
+  QuadriFlowJob *job = MEM_mallocN<QuadriFlowJob>("QuadriFlowJob");
 
   job->op = op;
   job->owner = CTX_data_active_object(C);
@@ -981,11 +994,15 @@ static int quadriflow_remesh_exec(bContext *C, wmOperator *op)
     job->symmetry_axes = (eSymmetryAxes)0;
   }
 
-  if (op->flag == 0) {
+  eQuadreFlowStatus status = QUADRIFLOW_STATUS_SUCCESS;
+  if ((op->flag & OP_IS_INVOKE) == 0) {
     /* This is called directly from the exec operator, this operation is now blocking */
     job->is_nonblocking_job = false;
     wmJobWorkerStatus worker_status = {};
+    worker_status.reports = op->reports;
     quadriflow_start_job(job, &worker_status);
+
+    status = job->status;
     quadriflow_end_job(job);
     quadriflow_free_job(job);
   }
@@ -996,7 +1013,7 @@ static int quadriflow_remesh_exec(bContext *C, wmOperator *op)
     wmJob *wm_job = WM_jobs_get(CTX_wm_manager(C),
                                 CTX_wm_window(C),
                                 CTX_data_scene(C),
-                                "QuadriFlow Remesh",
+                                "Remeshing with QuadriFlow...",
                                 WM_JOB_PROGRESS,
                                 WM_JOB_TYPE_QUADRIFLOW_REMESH);
 
@@ -1004,11 +1021,16 @@ static int quadriflow_remesh_exec(bContext *C, wmOperator *op)
     WM_jobs_timer(wm_job, 0.1, NC_GEOM | ND_DATA, NC_GEOM | ND_DATA);
     WM_jobs_callbacks(wm_job, quadriflow_start_job, nullptr, nullptr, quadriflow_end_job);
 
-    WM_set_locked_interface(CTX_wm_manager(C), true);
+    WM_locked_interface_set(CTX_wm_manager(C), true);
 
     WM_jobs_start(CTX_wm_manager(C), wm_job);
   }
-  return OPERATOR_FINISHED;
+
+  if (status == QUADRIFLOW_STATUS_SUCCESS) {
+    return OPERATOR_FINISHED;
+  }
+  /* Only ever runs with immediate execution. */
+  return OPERATOR_CANCELLED;
 }
 
 static bool quadriflow_check(bContext *C, wmOperator *op)
@@ -1095,7 +1117,7 @@ static const EnumPropertyItem mode_type_items[] = {
     {0, nullptr, 0, nullptr, nullptr},
 };
 
-static int quadriflow_remesh_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus quadriflow_remesh_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
   return WM_operator_props_popup_confirm_ex(
       C, op, event, IFACE_("QuadriFlow Remesh the Selected Mesh"), IFACE_("Remesh"));
@@ -1110,7 +1132,7 @@ void OBJECT_OT_quadriflow_remesh(wmOperatorType *ot)
       "layers will be lost";
   ot->idname = "OBJECT_OT_quadriflow_remesh";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->poll = object_remesh_poll;
   ot->poll_property = quadriflow_poll_property;
   ot->check = quadriflow_check;
