@@ -17,6 +17,7 @@
 #include <algorithm>
 
 #include "BLI_math_matrix.h"
+#include "BLI_math_vector.h"
 #include "BLI_math_vector_types.hh"
 #include "BLI_sort_utils.h"
 
@@ -95,8 +96,10 @@ static void gizmo_axis_draw(const bContext *C, wmGizmo *gz)
   GPU_matrix_mul(matrix_screen);
 
   GPUVertFormat *format = immVertexFormat();
-  const uint pos_id = GPU_vertformat_attr_add(format, "pos", GPU_COMP_F32, 3, GPU_FETCH_FLOAT);
-  const uint color_id = GPU_vertformat_attr_add(format, "color", GPU_COMP_F32, 4, GPU_FETCH_FLOAT);
+  const uint pos_id = GPU_vertformat_attr_add(
+      format, "pos", blender::gpu::VertAttrType::SFLOAT_32_32_32);
+  const uint color_id = GPU_vertformat_attr_add(
+      format, "color", blender::gpu::VertAttrType::SFLOAT_32_32_32_32);
   float viewport_size[4];
   GPU_viewport_size_get_f(viewport_size);
 
@@ -359,7 +362,7 @@ void VIEW3D_GT_navigate_rotate(wmGizmoType *gzt)
   /* identifiers */
   gzt->idname = "VIEW3D_GT_navigate_rotate";
 
-  /* api callbacks */
+  /* API callbacks. */
   gzt->draw = gizmo_axis_draw;
   gzt->test_select = gizmo_axis_test_select;
   gzt->cursor_get = gizmo_axis_cursor_get;
