@@ -34,6 +34,7 @@
 #include "DEG_depsgraph_build.hh"
 
 #include "ANIM_action.hh"
+#include "ANIM_armature.hh"
 #include "ANIM_bone_collections.hh"
 #include "ANIM_keyframing.hh"
 #include "ANIM_rna.hh"
@@ -45,6 +46,8 @@
 /* Own include. */
 #include "transform_convert.hh"
 
+namespace blender::ed::transform {
+
 struct BoneInitData {
   EditBone *bone;
   float tail[3];
@@ -61,7 +64,7 @@ struct BoneInitData {
  * and we will insert a keyframe at the end of transform. */
 static bool motionpath_need_update_pose(Scene *scene, Object *ob)
 {
-  if (blender::animrig::autokeyframe_cfra_can_key(scene, &ob->id)) {
+  if (animrig::autokeyframe_cfra_can_key(scene, &ob->id)) {
     return (ob->pose->avs.path_bakeflag & MOTIONPATH_BAKE_HAS_PATHS) != 0;
   }
 
@@ -263,7 +266,7 @@ static short pose_grab_with_ik(Main *bmain, Object *ob)
    * (but they must be selected, and only one ik-solver per chain should get added). */
   LISTBASE_FOREACH (bPoseChannel *, pchan, &ob->pose->chanbase) {
     if (BKE_pose_is_bonecoll_visible(arm, pchan)) {
-      if (pchan->bone->flag & (BONE_SELECTED | BONE_TRANSFORM_MIRROR)) {
+      if ((pchan->flag & POSE_SELECTED) || (pchan->bone->flag & BONE_TRANSFORM_MIRROR)) {
         /* Rule: no IK for solitary (unconnected) bones. */
         for (bonec = static_cast<Bone *>(pchan->bone->childbase.first); bonec; bonec = bonec->next)
         {
@@ -280,7 +283,7 @@ static short pose_grab_with_ik(Main *bmain, Object *ob)
           /* Only adds if there's no IK yet (and no parent bone was selected). */
           bPoseChannel *parent;
           for (parent = pchan->parent; parent; parent = parent->parent) {
-            if (parent->bone->flag & (BONE_SELECTED | BONE_TRANSFORM_MIRROR)) {
+            if ((parent->flag & POSE_SELECTED) || (parent->bone->flag & BONE_TRANSFORM_MIRROR)) {
               break;
             }
           }
@@ -316,7 +319,7 @@ struct PoseInitData_Mirror {
   bPoseChannel *pchan;
   struct {
     float loc[3];
-    float size[3];
+    float scale[3];
     union {
       float eul[3];
       float quat[4];
@@ -341,7 +344,7 @@ static void pose_mirror_info_init(PoseInitData_Mirror *pid,
 {
   pid->pchan = pchan;
   copy_v3_v3(pid->orig.loc, pchan->loc);
-  copy_v3_v3(pid->orig.size, pchan->size);
+  copy_v3_v3(pid->orig.scale, pchan->scale);
   pid->orig.curve_in_x = pchan->curve_in_x;
   pid->orig.curve_out_x = pchan->curve_out_x;
   pid->orig.roll1 = pchan->roll1;
@@ -386,22 +389,28 @@ static void pose_mirror_info_init(PoseInitData_Mirror *pid,
 /** \name Convert Armature
  * \{ */
 
-static void add_pose_transdata(TransInfo *t, bPoseChannel *pchan, Object *ob, TransData *td)
+static void add_pose_transdata(
+    TransInfo *t, bPoseChannel *pchan, Object *ob, TransData *td, TransDataExtension *td_ext)
 {
   Bone *bone = pchan->bone;
   float pmat[3][3], omat[3][3];
   float cmat[3][3], tmat[3][3];
-  float vec[3];
 
-  copy_v3_v3(vec, pchan->pose_mat[3]);
-  copy_v3_v3(td->center, vec);
+  const bArmature *arm = static_cast<bArmature *>(ob->data);
+  BKE_pose_channel_transform_location(arm, pchan, td->center);
+  if (pchan->flag & POSE_TRANSFORM_AROUND_CUSTOM_TX) {
+    copy_v3_v3(td_ext->center_no_override, pchan->pose_mat[3]);
+  }
+  else {
+    copy_v3_v3(td_ext->center_no_override, td->center);
+  }
 
   td->flag = TD_SELECTED;
-  if (bone->flag & BONE_HINGE_CHILD_TRANSFORM) {
+  if (pchan->runtime.flag & POSE_RUNTIME_HINGE_CHILD_TRANSFORM) {
     td->flag |= TD_NOCENTER;
   }
 
-  if (bone->flag & BONE_TRANSFORM_CHILD) {
+  if (pchan->runtime.flag & POSE_RUNTIME_TRANSFORM_CHILD) {
     td->flag |= TD_NOCENTER;
     td->flag |= TD_NO_LOC;
   }
@@ -412,44 +421,45 @@ static void add_pose_transdata(TransInfo *t, bPoseChannel *pchan, Object *ob, Tr
   td->loc = pchan->loc;
   copy_v3_v3(td->iloc, pchan->loc);
 
-  td->ext->size = pchan->size;
-  copy_v3_v3(td->ext->isize, pchan->size);
+  td_ext->scale = pchan->scale;
+  copy_v3_v3(td_ext->iscale, pchan->scale);
 
   if (pchan->rotmode > 0) {
-    td->ext->rot = pchan->eul;
-    td->ext->rotAxis = nullptr;
-    td->ext->rotAngle = nullptr;
-    td->ext->quat = nullptr;
+    td_ext->rot = pchan->eul;
+    td_ext->rotAxis = nullptr;
+    td_ext->rotAngle = nullptr;
+    td_ext->quat = nullptr;
 
-    copy_v3_v3(td->ext->irot, pchan->eul);
+    copy_v3_v3(td_ext->irot, pchan->eul);
   }
   else if (pchan->rotmode == ROT_MODE_AXISANGLE) {
-    td->ext->rot = nullptr;
-    td->ext->rotAxis = pchan->rotAxis;
-    td->ext->rotAngle = &pchan->rotAngle;
-    td->ext->quat = nullptr;
+    td_ext->rot = nullptr;
+    td_ext->rotAxis = pchan->rotAxis;
+    td_ext->rotAngle = &pchan->rotAngle;
+    td_ext->quat = nullptr;
 
-    td->ext->irotAngle = pchan->rotAngle;
-    copy_v3_v3(td->ext->irotAxis, pchan->rotAxis);
+    td_ext->irotAngle = pchan->rotAngle;
+    copy_v3_v3(td_ext->irotAxis, pchan->rotAxis);
   }
   else {
-    td->ext->rot = nullptr;
-    td->ext->rotAxis = nullptr;
-    td->ext->rotAngle = nullptr;
-    td->ext->quat = pchan->quat;
+    td_ext->rot = nullptr;
+    td_ext->rotAxis = nullptr;
+    td_ext->rotAngle = nullptr;
+    td_ext->quat = pchan->quat;
 
-    copy_qt_qt(td->ext->iquat, pchan->quat);
+    copy_qt_qt(td_ext->iquat, pchan->quat);
   }
-  td->ext->rotOrder = pchan->rotmode;
+  td_ext->rotOrder = pchan->rotmode;
 
   /* Proper way to get parent transform + our own transform + constraints transform. */
   copy_m3_m4(omat, ob->object_to_world().ptr());
 
-  /* New code, using "generic" BKE_bone_parent_transform_calc_from_pchan(). */
   {
     BoneParentTransform bpt;
     float rpmat[3][3];
 
+    /* Not using the pchan->custom_tx here because we need the transformation to be
+     * relative to the actual bone being modified, not it's visual representation.  */
     BKE_bone_parent_transform_calc_from_pchan(pchan, &bpt);
     if (t->mode == TFM_TRANSLATION) {
       copy_m3_m4(pmat, bpt.loc_mat);
@@ -468,13 +478,13 @@ static void add_pose_transdata(TransInfo *t, bPoseChannel *pchan, Object *ob, Tr
       copy_m3_m4(tmat, pchan->constinv);
       invert_m3_m3(cmat, tmat);
       mul_m3_series(td->mtx, cmat, omat, pmat);
-      mul_m3_series(td->ext->r_mtx, cmat, omat, rpmat);
+      mul_m3_series(td_ext->r_mtx, cmat, omat, rpmat);
     }
     else {
       mul_m3_series(td->mtx, omat, pmat);
-      mul_m3_series(td->ext->r_mtx, omat, rpmat);
+      mul_m3_series(td_ext->r_mtx, omat, rpmat);
     }
-    invert_m3_m3(td->ext->r_smtx, td->ext->r_mtx);
+    invert_m3_m3(td_ext->r_smtx, td_ext->r_mtx);
   }
 
   pseudoinverse_m3_m3(td->smtx, td->mtx, PSEUDOINVERSE_EPSILON);
@@ -486,7 +496,7 @@ static void add_pose_transdata(TransInfo *t, bPoseChannel *pchan, Object *ob, Tr
     if (pchan->parent) {
       /* Same as `td->smtx` but without `pchan->bone->bone_mat`. */
       td->flag |= TD_PBONE_LOCAL_MTX_C;
-      mul_m3_m3m3(td->ext->l_smtx, pchan->bone->bone_mat, td->smtx);
+      mul_m3_m3m3(td_ext->l_smtx, pchan->bone->bone_mat, td->smtx);
     }
     else {
       td->flag |= TD_PBONE_LOCAL_MTX_P;
@@ -494,13 +504,13 @@ static void add_pose_transdata(TransInfo *t, bPoseChannel *pchan, Object *ob, Tr
   }
 
   /* For `axismtx` we use the bone's own transform. */
-  copy_m3_m4(pmat, pchan->pose_mat);
+  BKE_pose_channel_transform_orientation(arm, pchan, pmat);
   mul_m3_m3m3(td->axismtx, omat, pmat);
   normalize_m3(td->axismtx);
 
   if (t->orient_type_mask & (1 << V3D_ORIENT_GIMBAL)) {
-    if (!gimbal_axis_pose(ob, pchan, td->ext->axismtx_gimbal)) {
-      copy_m3_m3(td->ext->axismtx_gimbal, td->axismtx);
+    if (!gimbal_axis_pose(ob, pchan, td_ext->axismtx_gimbal)) {
+      copy_m3_m3(td_ext->axismtx_gimbal, td->axismtx);
     }
   }
 
@@ -581,7 +591,7 @@ static void createTransPose(bContext * /*C*/, TransInfo *t)
     /* Now count, and check if we have autoIK or have to switch from translate to rotate. */
     LISTBASE_FOREACH (bPoseChannel *, pchan, &ob->pose->chanbase) {
       Bone *bone = pchan->bone;
-      if (!(bone->flag & BONE_TRANSFORM)) {
+      if (!(pchan->runtime.flag & POSE_RUNTIME_TRANSFORM)) {
         continue;
       }
 
@@ -594,7 +604,7 @@ static void createTransPose(bContext * /*C*/, TransInfo *t)
 
         if (has_targetless_ik(pchan) == nullptr) {
           if (pchan->parent && (bone->flag & BONE_CONNECTED)) {
-            if (bone->flag & BONE_HINGE_CHILD_TRANSFORM) {
+            if (pchan->runtime.flag & POSE_RUNTIME_HINGE_CHILD_TRANSFORM) {
               has_translate_rotate[0] = true;
             }
           }
@@ -631,15 +641,15 @@ static void createTransPose(bContext * /*C*/, TransInfo *t)
         /* Clear the MIRROR flag from previous runs. */
         pchan->bone->flag &= ~BONE_TRANSFORM_MIRROR;
 
-        if ((pchan->bone->flag & BONE_TRANSFORM) &&
+        if ((pchan->runtime.flag & POSE_RUNTIME_TRANSFORM) &&
             BKE_pose_channel_get_mirrored(ob->pose, pchan->name))
         {
           total_mirrored++;
         }
       }
 
-      PoseInitData_Mirror *pid = static_cast<PoseInitData_Mirror *>(
-          MEM_mallocN((total_mirrored + 1) * sizeof(PoseInitData_Mirror), "PoseInitData_Mirror"));
+      PoseInitData_Mirror *pid = MEM_malloc_arrayN<PoseInitData_Mirror>((total_mirrored + 1),
+                                                                        "PoseInitData_Mirror");
 
       /* Trick to terminate iteration. */
       pid[total_mirrored].pchan = nullptr;
@@ -656,7 +666,6 @@ static void createTransPose(bContext * /*C*/, TransInfo *t)
     Object *ob = tc->poseobj;
     TransData *td;
     TransDataExtension *tdx;
-    int i;
 
     PoseInitData_Mirror *pid = static_cast<PoseInitData_Mirror *>(tc->custom.type.data);
     int pid_index = 0;
@@ -673,18 +682,12 @@ static void createTransPose(bContext * /*C*/, TransInfo *t)
     tc->poseobj = ob;
 
     /* Initialize trans data. */
-    td = tc->data = static_cast<TransData *>(
-        MEM_callocN(tc->data_len * sizeof(TransData), "TransPoseBone"));
-    tdx = tc->data_ext = static_cast<TransDataExtension *>(
-        MEM_callocN(tc->data_len * sizeof(TransDataExtension), "TransPoseBoneExt"));
-    for (i = 0; i < tc->data_len; i++, td++, tdx++) {
-      td->ext = tdx;
-      td->val = nullptr;
-    }
+    td = tc->data = MEM_calloc_arrayN<TransData>(tc->data_len, "TransPoseBone");
+    tdx = tc->data_ext = MEM_calloc_arrayN<TransDataExtension>(tc->data_len, "TransPoseBoneExt");
 
     if (mirror) {
       LISTBASE_FOREACH (bPoseChannel *, pchan, &pose->chanbase) {
-        if (pchan->bone->flag & BONE_TRANSFORM) {
+        if (pchan->runtime.flag & POSE_RUNTIME_TRANSFORM) {
           bPoseChannel *pchan_mirror = BKE_pose_channel_get_mirrored(ob->pose, pchan->name);
           if (pchan_mirror) {
             pchan_mirror->bone->flag |= BONE_TRANSFORM_MIRROR;
@@ -705,15 +708,19 @@ static void createTransPose(bContext * /*C*/, TransInfo *t)
 
     /* Use pose channels to fill trans data. */
     td = tc->data;
+    tdx = tc->data_ext;
+    tdx->center_no_override[0] = 0;
+    tdx->center_no_override[1] = 0;
+    tdx->center_no_override[2] = 0;
     LISTBASE_FOREACH (bPoseChannel *, pchan, &ob->pose->chanbase) {
-      if (pchan->bone->flag & BONE_TRANSFORM) {
-        add_pose_transdata(t, pchan, ob, td);
-        td++;
+      if (pchan->runtime.flag & POSE_RUNTIME_TRANSFORM) {
+        add_pose_transdata(t, pchan, ob, td++, tdx++);
       }
     }
 
     if (td != (tc->data + tc->data_len)) {
       BKE_report(t->reports, RPT_DEBUG, "Bone selection count error");
+      BLI_assert_unreachable();
     }
   }
 
@@ -747,7 +754,7 @@ static void createTransArmatureVerts(bContext * /*C*/, TransInfo *t)
     LISTBASE_FOREACH (EditBone *, ebo, edbo) {
       const int data_len_prev = tc->data_len;
 
-      if (EBONE_VISIBLE(arm, ebo) && !(ebo->flag & BONE_EDITMODE_LOCKED)) {
+      if (blender::animrig::bone_is_visible(arm, ebo) && !(ebo->flag & BONE_EDITMODE_LOCKED)) {
         if (ELEM(t->mode, TFM_BONESIZE, TFM_BONE_ENVELOPE_DIST)) {
           if (ebo->flag & BONE_SELECTED) {
             tc->data_len++;
@@ -780,8 +787,7 @@ static void createTransArmatureVerts(bContext * /*C*/, TransInfo *t)
     }
 
     if (mirror) {
-      BoneInitData *bid = static_cast<BoneInitData *>(
-          MEM_mallocN((total_mirrored + 1) * sizeof(BoneInitData), "BoneInitData"));
+      BoneInitData *bid = MEM_malloc_arrayN<BoneInitData>((total_mirrored + 1), "BoneInitData");
 
       /* Trick to terminate iteration. */
       bid[total_mirrored].bone = nullptr;
@@ -810,8 +816,7 @@ static void createTransArmatureVerts(bContext * /*C*/, TransInfo *t)
     copy_m3_m4(mtx, tc->obedit->object_to_world().ptr());
     pseudoinverse_m3_m3(smtx, mtx, PSEUDOINVERSE_EPSILON);
 
-    td = tc->data = static_cast<TransData *>(
-        MEM_callocN(tc->data_len * sizeof(TransData), "TransEditBone"));
+    td = tc->data = MEM_calloc_arrayN<TransData>(tc->data_len, "TransEditBone");
     int i = 0;
 
     LISTBASE_FOREACH (EditBone *, ebo, edbo) {
@@ -820,7 +825,7 @@ static void createTransArmatureVerts(bContext * /*C*/, TransInfo *t)
       /* (length == 0.0) on extrude, used for scaling radius of bone points. */
       ebo->oldlength = ebo->length;
 
-      if (EBONE_VISIBLE(arm, ebo) && !(ebo->flag & BONE_EDITMODE_LOCKED)) {
+      if (blender::animrig::bone_is_visible(arm, ebo) && !(ebo->flag & BONE_EDITMODE_LOCKED)) {
         if (t->mode == TFM_BONE_ENVELOPE) {
           if (ebo->flag & BONE_ROOTSEL) {
             td->val = &ebo->rad_head;
@@ -833,7 +838,6 @@ static void createTransArmatureVerts(bContext * /*C*/, TransInfo *t)
             copy_m3_m3(td->mtx, mtx);
 
             td->loc = nullptr;
-            td->ext = nullptr;
 
             td++;
           }
@@ -847,7 +851,6 @@ static void createTransArmatureVerts(bContext * /*C*/, TransInfo *t)
             copy_m3_m3(td->mtx, mtx);
 
             td->loc = nullptr;
-            td->ext = nullptr;
 
             td++;
           }
@@ -876,8 +879,6 @@ static void createTransArmatureVerts(bContext * /*C*/, TransInfo *t)
             copy_m3_m3(td->axismtx, td->mtx);
             normalize_m3(td->axismtx);
 
-            td->ext = nullptr;
-
             td++;
           }
         }
@@ -890,8 +891,6 @@ static void createTransArmatureVerts(bContext * /*C*/, TransInfo *t)
             copy_v3_v3(td->center, ebo->head);
             td->flag = TD_SELECTED;
 
-            td->ext = nullptr;
-
             td++;
           }
         }
@@ -928,7 +927,6 @@ static void createTransArmatureVerts(bContext * /*C*/, TransInfo *t)
               td->ival = ebo->roll;
             }
 
-            td->ext = nullptr;
             td->val = nullptr;
 
             td++;
@@ -950,7 +948,6 @@ static void createTransArmatureVerts(bContext * /*C*/, TransInfo *t)
             td->extra = ebo; /* To fix roll. */
             td->ival = ebo->roll;
 
-            td->ext = nullptr;
             td->val = nullptr;
 
             td++;
@@ -1055,7 +1052,8 @@ static void recalcData_edit_armature(TransInfo *t)
 
       if (ebo_parent) {
         /* If this bone has a parent tip that has been moved. */
-        if (EBONE_VISIBLE(arm, ebo_parent) && (ebo_parent->flag & BONE_TIPSEL)) {
+        if (blender::animrig::bone_is_visible(arm, ebo_parent) && (ebo_parent->flag & BONE_TIPSEL))
+        {
           copy_v3_v3(ebo->head, ebo_parent->tail);
           if (t->mode == TFM_BONE_ENVELOPE) {
             ebo->rad_head = ebo_parent->rad_tail;
@@ -1164,7 +1162,7 @@ static void pose_transform_mirror_update(TransInfo *t, TransDataContainer *tc, O
   TransData *td = tc->data;
   for (int i = tc->data_len; i--; td++) {
     bPoseChannel *pchan_orig = static_cast<bPoseChannel *>(td->extra);
-    BLI_assert(pchan_orig->bone->flag & BONE_TRANSFORM);
+    BLI_assert(pchan_orig->runtime.flag & POSE_RUNTIME_TRANSFORM);
     /* No layer check, correct mirror is more important. */
     bPoseChannel *pchan = BKE_pose_channel_get_mirrored(pose, pchan_orig->name);
     if (pchan == nullptr) {
@@ -1226,7 +1224,7 @@ static void pose_mirror_info_restore(const PoseInitData_Mirror *pid)
 {
   bPoseChannel *pchan = pid->pchan;
   copy_v3_v3(pchan->loc, pid->orig.loc);
-  copy_v3_v3(pchan->size, pid->orig.size);
+  copy_v3_v3(pchan->scale, pid->orig.scale);
   pchan->curve_in_x = pid->orig.curve_in_x;
   pchan->curve_out_x = pid->orig.curve_out_x;
   pchan->roll1 = pid->orig.roll1;
@@ -1262,15 +1260,15 @@ static void restoreMirrorPoseBones(TransDataContainer *tc)
 
 /* Given the transform mode `tmode` return a Vector of RNA paths that were possibly modified during
  * that transformation. */
-static blender::Vector<RNAPath> get_affected_rna_paths_from_transform_mode(
+static Vector<RNAPath> get_affected_rna_paths_from_transform_mode(
     const eTfmMode tmode,
     ToolSettings *toolsettings,
-    const blender::StringRef rotation_path,
+    const StringRef rotation_path,
     const bool targetless_ik,
     const bool is_connected,
     const bool transforming_more_than_one_bone)
 {
-  blender::Vector<RNAPath> rna_paths;
+  Vector<RNAPath> rna_paths;
 
   /* Handle the cases where we always need to key location, regardless of
    * transform mode. */
@@ -1345,17 +1343,17 @@ static void autokeyframe_pose(bContext *C,
 
   bPose *pose = ob->pose;
   LISTBASE_FOREACH (bPoseChannel *, pchan, &pose->chanbase) {
-    if ((pchan->bone->flag & BONE_TRANSFORM) == 0 &&
+    if ((pchan->runtime.flag & POSE_RUNTIME_TRANSFORM) == 0 &&
         !((pose->flag & POSE_MIRROR_EDIT) && (pchan->bone->flag & BONE_TRANSFORM_MIRROR)))
     {
       continue;
     }
 
-    blender::Vector<RNAPath> rna_paths;
-    const blender::StringRef rotation_path = blender::animrig::get_rotation_mode_path(
+    Vector<RNAPath> rna_paths;
+    const StringRef rotation_path = animrig::get_rotation_mode_path(
         eRotationModes(pchan->rotmode));
 
-    if (blender::animrig::is_keying_flag(scene, AUTOKEY_FLAG_INSERTNEEDED)) {
+    if (animrig::is_keying_flag(scene, AUTOKEY_FLAG_INSERTNEEDED)) {
       const bool is_connected = pchan->bone->parent != nullptr &&
                                 (pchan->bone->flag & BONE_CONNECTED);
       rna_paths = get_affected_rna_paths_from_transform_mode(tmode,
@@ -1369,8 +1367,7 @@ static void autokeyframe_pose(bContext *C,
       rna_paths = {{"location"}, {rotation_path}, {"scale"}};
     }
 
-    blender::animrig::autokeyframe_pose_channel(
-        C, scene, ob, pchan, rna_paths.as_span(), targetless_ik);
+    animrig::autokeyframe_pose_channel(C, scene, ob, pchan, rna_paths.as_span(), targetless_ik);
   }
 }
 
@@ -1428,7 +1425,7 @@ static void recalcData_pose(TransInfo *t)
 
       /* TODO: autokeyframe calls need some setting to specify to add samples
        * (FPoints) instead of keyframes? */
-      if ((t->animtimer) && (t->context) && blender::animrig::is_autokey_on(t->scene)) {
+      if ((t->animtimer) && (t->context) && animrig::is_autokey_on(t->scene)) {
 
         /* XXX: this currently doesn't work, since flags aren't set yet! */
         int targetless_ik = (t->flag & T_AUTOIK);
@@ -1460,57 +1457,56 @@ static void recalcData_pose(TransInfo *t)
 /** \name Special After Transform Pose
  * \{ */
 
-static void bone_children_clear_transflag(int mode, short around, ListBase *lb)
+static void pose_channel_children_clear_transflag(bPose &pose,
+                                                  bPoseChannel &pose_bone,
+                                                  const int mode,
+                                                  const short around)
 {
-  Bone *bone = static_cast<Bone *>(lb->first);
-
-  for (; bone; bone = bone->next) {
-    if ((bone->flag & BONE_HINGE) && (bone->flag & BONE_CONNECTED)) {
-      bone->flag |= BONE_HINGE_CHILD_TRANSFORM;
+  blender::animrig::pose_bone_descendent_iterator(pose, pose_bone, [&](bPoseChannel &child) {
+    if (&pose_bone == &child) {
+      return;
     }
-    else if ((bone->flag & BONE_TRANSFORM) && ELEM(mode, TFM_ROTATION, TFM_TRACKBALL) &&
-             (around == V3D_AROUND_LOCAL_ORIGINS))
+    Bone *bone = child.bone;
+    if ((bone->flag & BONE_HINGE) && (bone->flag & BONE_CONNECTED)) {
+      child.runtime.flag |= POSE_RUNTIME_HINGE_CHILD_TRANSFORM;
+    }
+    else if ((child.runtime.flag & POSE_RUNTIME_TRANSFORM) &&
+             ELEM(mode, TFM_ROTATION, TFM_TRACKBALL) && (around == V3D_AROUND_LOCAL_ORIGINS))
     {
-      bone->flag |= BONE_TRANSFORM_CHILD;
+      child.runtime.flag |= POSE_RUNTIME_TRANSFORM_CHILD;
     }
     else {
-      bone->flag &= ~BONE_TRANSFORM;
+      child.runtime.flag &= ~POSE_RUNTIME_TRANSFORM;
     }
-
-    bone_children_clear_transflag(mode, around, &bone->childbase);
-  }
+  });
 }
 
 void transform_convert_pose_transflags_update(Object *ob, const int mode, const short around)
 {
   bArmature *arm = static_cast<bArmature *>(ob->data);
-  Bone *bone;
 
   LISTBASE_FOREACH (bPoseChannel *, pchan, &ob->pose->chanbase) {
-    bone = pchan->bone;
-    if (PBONE_VISIBLE(arm, bone)) {
-      if (bone->flag & BONE_SELECTED) {
-        bone->flag |= BONE_TRANSFORM;
+    if (blender::animrig::bone_is_visible(arm, pchan)) {
+      if (pchan->flag & POSE_SELECTED) {
+        pchan->runtime.flag |= POSE_RUNTIME_TRANSFORM;
       }
       else {
-        bone->flag &= ~BONE_TRANSFORM;
+        pchan->runtime.flag &= ~POSE_RUNTIME_TRANSFORM;
       }
 
-      bone->flag &= ~BONE_HINGE_CHILD_TRANSFORM;
-      bone->flag &= ~BONE_TRANSFORM_CHILD;
+      pchan->runtime.flag &= ~POSE_RUNTIME_HINGE_CHILD_TRANSFORM;
+      pchan->runtime.flag &= ~POSE_RUNTIME_TRANSFORM_CHILD;
     }
     else {
-      bone->flag &= ~BONE_TRANSFORM;
+      pchan->runtime.flag &= ~POSE_RUNTIME_TRANSFORM;
     }
   }
 
   /* Make sure no bone can be transformed when a parent is transformed. */
-  /* Since pchans are depsgraph sorted, the parents are in beginning of list. */
   if (!ELEM(mode, TFM_BONESIZE, TFM_BONE_ENVELOPE_DIST)) {
     LISTBASE_FOREACH (bPoseChannel *, pchan, &ob->pose->chanbase) {
-      bone = pchan->bone;
-      if (bone->flag & BONE_TRANSFORM) {
-        bone_children_clear_transflag(mode, around, &bone->childbase);
+      if (pchan->runtime.flag & POSE_RUNTIME_TRANSFORM) {
+        pose_channel_children_clear_transflag(*ob->pose, *pchan, mode, around);
       }
     }
   }
@@ -1546,7 +1542,6 @@ static short apply_targetless_ik(Object *ob)
         }
       }
       for (; segcount; segcount--) {
-        Bone *bone;
         float mat[4][4];
 
         /* `pose_mat(b) = pose_mat(b-1) * offs_bone * channel * constraint * IK`. */
@@ -1555,8 +1550,8 @@ static short apply_targetless_ik(Object *ob)
         /* `mat = pose_mat(b) * inv(pose_mat(b-1) * offs_bone)`. */
 
         parchan = chanlist[segcount - 1];
-        bone = parchan->bone;
-        bone->flag |= BONE_TRANSFORM; /* Ensures it gets an auto key inserted. */
+        /* Ensures it gets an auto key inserted. */
+        parchan->runtime.flag |= POSE_RUNTIME_TRANSFORM;
 
         BKE_armature_mat_pose_to_bone(parchan, parchan->pose_mat, mat);
         /* Apply and decompose, doesn't work for constraints or non-uniform scale well. */
@@ -1580,7 +1575,7 @@ static short apply_targetless_ik(Object *ob)
             BKE_pchan_rot_to_mat3(parchan, qrmat);
             invert_m3_m3(imat3, qrmat);
             mul_m3_m3m3(smat, rmat3, imat3);
-            mat3_to_size(parchan->size, smat);
+            mat3_to_size(parchan->scale, smat);
           }
 
           /* Causes problems with some constraints (e.g. child-of), so disable this
@@ -1656,7 +1651,7 @@ static void special_aftertrans_update__pose(bContext *C, TransInfo *t)
   else {
     const bool canceled = (t->state == TRANS_CANCEL);
 
-    if (blender::animrig::is_autokey_on(t->scene) && !canceled) {
+    if (animrig::is_autokey_on(t->scene) && !canceled) {
       ANIM_deselect_keys_in_animation_editors(C);
     }
 
@@ -1675,7 +1670,7 @@ static void special_aftertrans_update__pose(bContext *C, TransInfo *t)
         BKE_pose_where_is(t->depsgraph, t->scene, pose_ob);
       }
 
-      /* Set BONE_TRANSFORM flags for auto-key, gizmo draw might have changed them. */
+      /* Set POSE_RUNTIME_TRANSFORM flags for auto-key, gizmo draw might have changed them. */
       if (!canceled && (t->mode != TFM_DUMMY)) {
         transform_convert_pose_transflags_update(ob, t->mode, t->around);
       }
@@ -1741,3 +1736,5 @@ TransConvertTypeInfo TransConvertType_Pose = {
     /*recalc_data*/ recalcData_pose,
     /*special_aftertrans_update*/ special_aftertrans_update__pose,
 };
+
+}  // namespace blender::ed::transform
