@@ -15,7 +15,7 @@
 #include "BLI_listbase.h"
 #include "BLI_math_vector.h"
 #include "BLI_rect.h"
-#include "BLI_string.h"
+#include "BLI_string_utf8.h"
 #include "BLI_utildefines.h"
 
 #include "BLT_translation.hh"
@@ -121,7 +121,7 @@ static void draw_overshoot_triangle(const uint8_t color[4],
                                     const float y)
 {
   const uint shdr_pos_2d = GPU_vertformat_attr_add(
-      immVertexFormat(), "pos", GPU_COMP_F32, 2, GPU_FETCH_FLOAT);
+      immVertexFormat(), "pos", blender::gpu::VertAttrType::SFLOAT_32_32);
   immBindBuiltinProgram(GPU_SHADER_3D_UNIFORM_COLOR);
   GPU_blend(GPU_BLEND_ALPHA);
   GPU_polygon_smooth(true);
@@ -365,10 +365,10 @@ static void slider_draw(const bContext * /*C*/, ARegion *region, void *arg)
   char factor_string[256];
   switch (slider->slider_mode) {
     case SLIDER_MODE_PERCENT:
-      SNPRINTF(factor_string, "%.0f %s", slider->factor * 100, slider->unit_string);
+      SNPRINTF_UTF8(factor_string, "%.0f %s", slider->factor * 100, slider->unit_string);
       break;
     case SLIDER_MODE_FLOAT:
-      SNPRINTF(factor_string, "%.1f %s", slider->factor, slider->unit_string);
+      SNPRINTF_UTF8(factor_string, "%.1f %s", slider->factor, slider->unit_string);
       break;
   }
 
@@ -523,41 +523,41 @@ void ED_slider_status_string_get(const tSlider *slider,
 
   if (slider->allow_overshoot_lower || slider->allow_overshoot_upper) {
     if (slider->overshoot) {
-      STRNCPY(overshoot_str, IFACE_("[E] - Disable overshoot"));
+      STRNCPY_UTF8(overshoot_str, IFACE_("[E] - Disable overshoot"));
     }
     else {
-      STRNCPY(overshoot_str, IFACE_("[E] - Enable overshoot"));
+      STRNCPY_UTF8(overshoot_str, IFACE_("[E] - Enable overshoot"));
     }
   }
   else {
-    STRNCPY(overshoot_str, IFACE_("Overshoot disabled"));
+    STRNCPY_UTF8(overshoot_str, IFACE_("Overshoot disabled"));
   }
 
   if (slider->precision) {
-    STRNCPY(precision_str, IFACE_("[Shift] - Precision active"));
+    STRNCPY_UTF8(precision_str, IFACE_("[Shift] - Precision active"));
   }
   else {
-    STRNCPY(precision_str, IFACE_("Shift - Hold for precision"));
+    STRNCPY_UTF8(precision_str, IFACE_("Shift - Hold for precision"));
   }
 
   if (slider->allow_increments) {
     if (slider->increments) {
-      STRNCPY(increments_str, IFACE_(" | [Ctrl] - Increments active"));
+      STRNCPY_UTF8(increments_str, IFACE_(" | [Ctrl] - Increments active"));
     }
     else {
-      STRNCPY(increments_str, IFACE_(" | Ctrl - Hold for increments"));
+      STRNCPY_UTF8(increments_str, IFACE_(" | Ctrl - Hold for increments"));
     }
   }
   else {
     increments_str[0] = '\0';
   }
 
-  BLI_snprintf(status_string,
-               size_of_status_string,
-               "%s | %s%s",
-               overshoot_str,
-               precision_str,
-               increments_str);
+  BLI_snprintf_utf8(status_string,
+                    size_of_status_string,
+                    "%s | %s%s",
+                    overshoot_str,
+                    precision_str,
+                    increments_str);
 }
 
 void ED_slider_status_get(const tSlider *slider, WorkspaceStatus &status)
@@ -649,7 +649,7 @@ SliderMode ED_slider_mode_get(const tSlider *slider)
 
 void ED_slider_unit_set(tSlider *slider, const char *unit)
 {
-  STRNCPY(slider->unit_string, unit);
+  STRNCPY_UTF8(slider->unit_string, unit);
 }
 
 void ED_slider_property_label_set(tSlider *slider, const char *property_label)
@@ -669,7 +669,7 @@ void ED_region_draw_mouse_line_cb(const bContext *C, ARegion *region, void *arg_
   };
 
   const uint shdr_pos = GPU_vertformat_attr_add(
-      immVertexFormat(), "pos", GPU_COMP_F32, 2, GPU_FETCH_FLOAT);
+      immVertexFormat(), "pos", blender::gpu::VertAttrType::SFLOAT_32_32);
 
   GPU_line_width(1.0f);
 
@@ -744,7 +744,7 @@ static void metadata_custom_draw_fields(const char *field, const char *value, vo
   }
   MetadataCustomDrawContext *ctx = (MetadataCustomDrawContext *)ctx_v;
   char temp_str[MAX_METADATA_STR];
-  SNPRINTF(temp_str, "%s: %s", field, value);
+  SNPRINTF_UTF8(temp_str, "%s: %s", field, value);
   BLF_position(ctx->fontid, ctx->xmin, ctx->ymin + ctx->current_y, 0.0f);
   BLF_draw(ctx->fontid, temp_str, sizeof(temp_str));
   ctx->current_y += ctx->vertical_offset;
@@ -770,14 +770,14 @@ static void metadata_draw_imbuf(const ImBuf *ibuf, const rctf *rect, int fontid,
       /* first line */
       if (i == 0) {
         bool do_newline = false;
-        int len = SNPRINTF_RLEN(temp_str, "%s: ", meta_data_list[0]);
+        int len = SNPRINTF_UTF8_RLEN(temp_str, "%s: ", meta_data_list[0]);
         if (metadata_is_valid(ibuf, temp_str, 0, len)) {
           BLF_position(fontid, xmin, ymax - vertical_offset, 0.0f);
           BLF_draw(fontid, temp_str, sizeof(temp_str));
           do_newline = true;
         }
 
-        len = SNPRINTF_RLEN(temp_str, "%s: ", meta_data_list[1]);
+        len = SNPRINTF_UTF8_RLEN(temp_str, "%s: ", meta_data_list[1]);
         if (metadata_is_valid(ibuf, temp_str, 1, len)) {
           int line_width = BLF_width(fontid, temp_str, sizeof(temp_str));
           BLF_position(fontid, xmax - line_width, ymax - vertical_offset, 0.0f);
@@ -790,7 +790,7 @@ static void metadata_draw_imbuf(const ImBuf *ibuf, const rctf *rect, int fontid,
         }
       } /* Strip */
       else if (ELEM(i, 1, 2)) {
-        int len = SNPRINTF_RLEN(temp_str, "%s: ", meta_data_list[i + 1]);
+        int len = SNPRINTF_UTF8_RLEN(temp_str, "%s: ", meta_data_list[i + 1]);
         if (metadata_is_valid(ibuf, temp_str, i + 1, len)) {
           BLF_position(fontid, xmin, ymax - vertical_offset - ofs_y, 0.0f);
           BLF_draw(fontid, temp_str, sizeof(temp_str));
@@ -798,7 +798,7 @@ static void metadata_draw_imbuf(const ImBuf *ibuf, const rctf *rect, int fontid,
         }
       } /* Note (wrapped) */
       else if (i == 3) {
-        int len = SNPRINTF_RLEN(temp_str, "%s: ", meta_data_list[i + 1]);
+        int len = SNPRINTF_UTF8_RLEN(temp_str, "%s: ", meta_data_list[i + 1]);
         if (metadata_is_valid(ibuf, temp_str, i + 1, len)) {
           ResultBLF info;
           BLF_enable(fontid, BLF_WORD_WRAP);
@@ -811,7 +811,7 @@ static void metadata_draw_imbuf(const ImBuf *ibuf, const rctf *rect, int fontid,
         }
       }
       else {
-        int len = SNPRINTF_RLEN(temp_str, "%s: ", meta_data_list[i + 1]);
+        int len = SNPRINTF_UTF8_RLEN(temp_str, "%s: ", meta_data_list[i + 1]);
         if (metadata_is_valid(ibuf, temp_str, i + 1, len)) {
           int line_width = BLF_width(fontid, temp_str, sizeof(temp_str));
           BLF_position(fontid, xmax - line_width, ymax - vertical_offset - ofs_y, 0.0f);
@@ -832,7 +832,7 @@ static void metadata_draw_imbuf(const ImBuf *ibuf, const rctf *rect, int fontid,
     int ofs_x = 0;
     ofs_y = ctx.current_y;
     for (int i = 5; i < 10; i++) {
-      int len = SNPRINTF_RLEN(temp_str, "%s: ", meta_data_list[i]);
+      int len = SNPRINTF_UTF8_RLEN(temp_str, "%s: ", meta_data_list[i]);
       if (metadata_is_valid(ibuf, temp_str, i, len)) {
         BLF_position(fontid, xmin + ofs_x, ymin + ofs_y, 0.0f);
         BLF_draw(fontid, temp_str, sizeof(temp_str));
@@ -933,7 +933,7 @@ static void text_info_row(const char *text,
   BLF_draw(font_id, IFACE_(text), text_len);
   BLF_position(font_id, col2, row, 0.0f);
   char draw_text[MAX_NAME];
-  SNPRINTF(draw_text, "%d x %d", size_x, size_y);
+  SNPRINTF_UTF8(draw_text, "%d x %d", size_x, size_y);
   BLF_draw(font_id, draw_text, sizeof(draw_text));
 
   BLF_disable(font_id, BLF_SHADOW);
@@ -991,7 +991,7 @@ void ED_region_image_render_region_draw(
   GPU_matrix_scale_2f(zoomx, zoomy);
 
   GPUVertFormat *format = immVertexFormat();
-  uint pos = GPU_vertformat_attr_add(format, "pos", GPU_COMP_F32, 2, GPU_FETCH_FLOAT);
+  uint pos = GPU_vertformat_attr_add(format, "pos", blender::gpu::VertAttrType::SFLOAT_32_32);
 
   immBindBuiltinProgram(GPU_SHADER_3D_UNIFORM_COLOR);
   GPU_blend(GPU_BLEND_ALPHA);
@@ -1056,7 +1056,7 @@ void ED_region_image_metadata_draw(
     BLI_rctf_init(&rect, frame->xmin, frame->xmax, frame->ymax, frame->ymax + box_y);
     /* draw top box */
     GPUVertFormat *format = immVertexFormat();
-    uint pos = GPU_vertformat_attr_add(format, "pos", GPU_COMP_F32, 2, GPU_FETCH_FLOAT);
+    uint pos = GPU_vertformat_attr_add(format, "pos", blender::gpu::VertAttrType::SFLOAT_32_32);
     immBindBuiltinProgram(GPU_SHADER_3D_UNIFORM_COLOR);
     immUniformThemeColorAlpha(TH_METADATA_BG, 1.0f);
     immRectf(pos, rect.xmin, rect.ymin, rect.xmax, rect.ymax);
@@ -1081,7 +1081,7 @@ void ED_region_image_metadata_draw(
     BLI_rctf_init(&rect, frame->xmin, frame->xmax, frame->ymin - box_y, frame->ymin);
     /* draw top box */
     GPUVertFormat *format = immVertexFormat();
-    uint pos = GPU_vertformat_attr_add(format, "pos", GPU_COMP_F32, 2, GPU_FETCH_FLOAT);
+    uint pos = GPU_vertformat_attr_add(format, "pos", blender::gpu::VertAttrType::SFLOAT_32_32);
     immBindBuiltinProgram(GPU_SHADER_3D_UNIFORM_COLOR);
     immUniformThemeColorAlpha(TH_METADATA_BG, 1.0f);
     immRectf(pos, rect.xmin, rect.ymin, rect.xmax, rect.ymax);
