@@ -57,9 +57,7 @@ GPUMatrixState *GPU_matrix_state_create()
 #define MATRIX_4X4_IDENTITY \
   { \
     {1.0f, 0.0f, 0.0f, 0.0f}, {0.0f, 1.0f, 0.0f, 0.0f}, {0.0f, 0.0f, 1.0f, 0.0f}, \
-    { \
-      0.0f, 0.0f, 0.0f, 1.0f \
-    } \
+        {0.0f, 0.0f, 0.0f, 1.0f} \
   }
 
   GPUMatrixState *state = (GPUMatrixState *)MEM_mallocN(sizeof(*state), __func__);
@@ -609,7 +607,7 @@ const float (*GPU_matrix_normal_inverse_get(float m[3][3]))[3]
   return m;
 }
 
-void GPU_matrix_bind(GPUShader *shader)
+void GPU_matrix_bind(blender::gpu::Shader *shader)
 {
   /* set uniform values to matrix stack values
    * call this before a draw call if desired matrices are dirty
