@@ -159,7 +159,7 @@ gpu::MTLBuffer *MTLBufferPool::allocate_aligned(uint64_t size,
       if (found_size >= aligned_alloc_size &&
           found_size <= (aligned_alloc_size * mtl_buffer_size_threshold_factor_))
       {
-        MTL_LOG_INFO(
+        MTL_LOG_DEBUG(
             "[MemoryAllocator] Suitable Buffer of size %lld found, for requested size: %lld",
             found_size,
             aligned_alloc_size);
@@ -171,7 +171,7 @@ gpu::MTLBuffer *MTLBufferPool::allocate_aligned(uint64_t size,
         pool->erase(result);
       }
       else {
-        MTL_LOG_INFO(
+        MTL_LOG_DEBUG(
             "[MemoryAllocator] Buffer of size %lld found, but was incompatible with requested "
             "size: %lld",
             found_size,
@@ -904,10 +904,10 @@ void MTLScratchBufferManager::ensure_increment_scratch_buffer()
     active_scratch_buf = scratch_buffers_[current_scratch_buffer_];
     active_scratch_buf->reset();
     BLI_assert(&active_scratch_buf->own_context_ == &context_);
-    MTL_LOG_INFO("Scratch buffer %d reset - (ctx %p)(Frame index: %d)",
-                 current_scratch_buffer_,
-                 &context_,
-                 context_.get_current_frame_index());
+    MTL_LOG_DEBUG("Scratch buffer %d reset - (ctx %p)(Frame index: %d)",
+                  current_scratch_buffer_,
+                  &context_,
+                  context_.get_current_frame_index());
   }
 }
 
@@ -1005,11 +1005,12 @@ MTLTemporaryBuffer MTLCircularBuffer::allocate_range_aligned(uint64_t alloc_size
          * maximum */
         if (aligned_alloc_size > MTLScratchBufferManager::mtl_scratch_buffer_max_size_) {
           new_size = aligned_alloc_size;
-          MTL_LOG_INFO("Temporarily growing Scratch buffer to %d MB", (int)new_size / 1024 / 1024);
+          MTL_LOG_DEBUG("Temporarily growing Scratch buffer to %d MB",
+                        (int)new_size / 1024 / 1024);
         }
         else {
           new_size = MTLScratchBufferManager::mtl_scratch_buffer_max_size_;
-          MTL_LOG_INFO("Shrinking Scratch buffer back to %d MB", (int)new_size / 1024 / 1024);
+          MTL_LOG_DEBUG("Shrinking Scratch buffer back to %d MB", (int)new_size / 1024 / 1024);
         }
       }
       BLI_assert(aligned_alloc_size <= new_size);
@@ -1061,7 +1062,7 @@ MTLTemporaryBuffer MTLCircularBuffer::allocate_range_aligned(uint64_t alloc_size
     if (G.debug & G_DEBUG_GPU) {
       cbuffer_->set_label(@"Circular Scratch Buffer");
     }
-    MTL_LOG_INFO("Resized Metal circular buffer to %llu bytes", new_size);
+    MTL_LOG_DEBUG("Resized Metal circular buffer to %llu bytes", new_size);
 
     /* Reset allocation Status. */
     aligned_current_offset = 0;
