@@ -43,7 +43,6 @@ struct Collection;
 struct Curve;
 struct FluidsimSettings;
 struct ImageUser;
-struct Ipo;
 struct LightgroupMembership;
 struct Material;
 struct Object;
@@ -213,10 +212,7 @@ typedef struct Object {
   struct Object *proxy DNA_DEPRECATED;
   struct Object *proxy_group DNA_DEPRECATED;
   struct Object *proxy_from DNA_DEPRECATED;
-  /** Old animation system, deprecated for 2.5. */
-  struct Ipo *ipo DNA_DEPRECATED;
   // struct Path *path;
-  struct bAction *action DNA_DEPRECATED;  /* XXX deprecated... old animation system */
   struct bAction *poselib DNA_DEPRECATED; /* Pre-Blender 3.0 pose library, deprecated in 3.5. */
   /** Pose data, armature objects only. */
   struct bPose *pose;
@@ -231,12 +227,10 @@ typedef struct Object {
   bAnimVizSettings avs;
   /** Motion path cache for this object. */
   bMotionPath *mpath;
-  void *_pad0;
 
-  ListBase constraintChannels DNA_DEPRECATED; /* XXX deprecated... old animation system */
-  ListBase effect DNA_DEPRECATED;             /* XXX deprecated... keep for readfile */
-  ListBase defbase DNA_DEPRECATED;            /* Only for versioning, moved to object data. */
-  ListBase fmaps DNA_DEPRECATED;              /* For versioning, moved to generic attributes. */
+  ListBase effect DNA_DEPRECATED;  /* XXX deprecated... keep for readfile */
+  ListBase defbase DNA_DEPRECATED; /* Only for versioning, moved to object data. */
+  ListBase fmaps DNA_DEPRECATED;   /* For versioning, moved to generic attributes. */
   /** List of ModifierData structures. */
   ListBase modifiers;
   /** List of GpencilModifierData structures. */
@@ -255,7 +249,7 @@ typedef struct Object {
   char *matbits;
   /** Copy of mesh, curve & meta struct member of same name (keep in sync). */
   int totcol;
-  /** Currently selected material in the UI. */
+  /** Currently selected material in the UI (one-based). */
   int actcol;
 
   /* rot en drot have to be together! (transform('r' en 's')) */
@@ -347,8 +341,7 @@ typedef struct Object {
 
   /** Object constraints. */
   ListBase constraints;
-  ListBase nlastrips DNA_DEPRECATED; /* XXX deprecated... old animation system */
-  ListBase hooks DNA_DEPRECATED;     /* XXX deprecated... old animation system */
+  ListBase hooks DNA_DEPRECATED;
   /** Particle systems. */
   ListBase particlesystem;
 
@@ -677,9 +670,8 @@ enum {
   /** Unknown state, clear before use. */
   OB_DONE = 1 << 10,
   OB_FLAG_USE_SIMULATION_CACHE = 1 << 11,
-#ifdef DNA_DEPRECATED_ALLOW
-  OB_FLAG_UNUSED_12 = 1 << 12, /* cleared */
-#endif
+  /** Used for the clipboard to mark the active object. */
+  OB_FLAG_ACTIVE_CLIPBOARD = 1 << 12,
 };
 
 /** #Object.visibility_flag */
@@ -698,6 +690,7 @@ enum {
   OB_HIDE_PROBE_VOLUME = 1 << 11,
   OB_HIDE_PROBE_CUBEMAP = 1 << 12,
   OB_HIDE_PROBE_PLANAR = 1 << 13,
+  OB_HIDE_SURFACE_PICK = 1 << 14,
 };
 
 /** #Object.shapeflag */
