@@ -7,20 +7,14 @@
  */
 
 #include <cfloat>
-#include <climits>
 #include <cstdlib>
 
 #include "DNA_boid_types.h"
-#include "DNA_object_types.h"
-#include "DNA_particle_types.h"
-#include "DNA_scene_types.h"
-
-#include "BLI_utildefines.h"
 
 #include "RNA_define.hh"
 #include "RNA_enum_types.hh"
 
-#include "rna_internal.h"
+#include "rna_internal.hh"
 
 #include "WM_api.hh"
 #include "WM_types.hh"
@@ -100,9 +94,12 @@ static const EnumPropertyItem boidruleset_type_items[] = {
 
 #ifdef RNA_RUNTIME
 
-#  include "BLI_math_base.h"
+#  include <fmt/format.h>
 
-#  include "BKE_context.h"
+#  include "BLI_math_base.h"
+#  include "BLI_string.h"
+
+#  include "BKE_context.hh"
 #  include "BKE_particle.h"
 
 #  include "DEG_depsgraph.hh"
@@ -163,14 +160,12 @@ static StructRNA *rna_BoidRule_refine(PointerRNA *ptr)
   }
 }
 
-static char *rna_BoidRule_path(const PointerRNA *ptr)
+static std::optional<std::string> rna_BoidRule_path(const PointerRNA *ptr)
 {
   const BoidRule *rule = (BoidRule *)ptr->data;
   char name_esc[sizeof(rule->name) * 2];
-
   BLI_str_escape(name_esc, rule->name, sizeof(name_esc));
-
-  return BLI_sprintfN("rules[\"%s\"]", name_esc); /* XXX not unique */
+  return fmt::format("rules[\"{}\"]", name_esc); /* XXX not unique */
 }
 
 static PointerRNA rna_BoidState_active_boid_rule_get(PointerRNA *ptr)
@@ -180,10 +175,10 @@ static PointerRNA rna_BoidState_active_boid_rule_get(PointerRNA *ptr)
 
   for (; rule; rule = rule->next) {
     if (rule->flag & BOIDRULE_CURRENT) {
-      return rna_pointer_inherit_refine(ptr, &RNA_BoidRule, rule);
+      return RNA_pointer_create_with_parent(*ptr, &RNA_BoidRule, rule);
     }
   }
-  return rna_pointer_inherit_refine(ptr, &RNA_BoidRule, nullptr);
+  return RNA_pointer_create_with_parent(*ptr, &RNA_BoidRule, nullptr);
 }
 static void rna_BoidState_active_boid_rule_index_range(
     PointerRNA *ptr, int *min, int *max, int * /*softmin*/, int * /*softmax*/)
@@ -230,7 +225,7 @@ static int particle_id_check(const PointerRNA *ptr)
   return (GS(id->name) == ID_PA);
 }
 
-static char *rna_BoidSettings_path(const PointerRNA *ptr)
+static std::optional<std::string> rna_BoidSettings_path(const PointerRNA *ptr)
 {
   const BoidSettings *boids = (BoidSettings *)ptr->data;
 
@@ -238,10 +233,10 @@ static char *rna_BoidSettings_path(const PointerRNA *ptr)
     ParticleSettings *part = (ParticleSettings *)ptr->owner_id;
 
     if (part->boids == boids) {
-      return BLI_strdup("boids");
+      return "boids";
     }
   }
-  return nullptr;
+  return std::nullopt;
 }
 
 static PointerRNA rna_BoidSettings_active_boid_state_get(PointerRNA *ptr)
@@ -251,10 +246,10 @@ static PointerRNA rna_BoidSettings_active_boid_state_get(PointerRNA *ptr)
 
   for (; state; state = state->next) {
     if (state->flag & BOIDSTATE_CURRENT) {
-      return rna_pointer_inherit_refine(ptr, &RNA_BoidState, state);
+      return RNA_pointer_create_with_parent(*ptr, &RNA_BoidState, state);
     }
   }
-  return rna_pointer_inherit_refine(ptr, &RNA_BoidState, nullptr);
+  return RNA_pointer_create_with_parent(*ptr, &RNA_BoidState, nullptr);
 }
 static void rna_BoidSettings_active_boid_state_index_range(
     PointerRNA *ptr, int *min, int *max, int * /*softmin*/, int * /*softmax*/)
