@@ -8,15 +8,11 @@
 
 #include <cmath>
 
-#include "BLI_assert.h"
 #include "BLI_math_base.hh"
 #include "BLI_math_vector.hh"
 #include "BLI_math_vector_types.hh"
 
-#include "RNA_access.hh"
-
-#include "UI_interface.hh"
-#include "UI_resources.hh"
+#include "RNA_types.hh"
 
 #include "IMB_colormanagement.hh"
 
@@ -28,67 +24,98 @@
 
 namespace blender::nodes::node_composite_tonemap_cc {
 
-NODE_STORAGE_FUNCS(NodeTonemap)
+static const EnumPropertyItem type_items[] = {
+    {CMP_NODE_TONE_MAP_PHOTORECEPTOR,
+     "RD_PHOTORECEPTOR",
+     0,
+     N_("R/D Photoreceptor"),
+     N_("More advanced algorithm based on eye physiology, by Reinhard and Devlin")},
+    {CMP_NODE_TONE_MAP_SIMPLE,
+     "RH_SIMPLE",
+     0,
+     N_("Rh Simple"),
+     N_("Simpler photographic algorithm by Reinhard")},
+    {0, nullptr, 0, nullptr, nullptr},
+};
 
 static void cmp_node_tonemap_declare(NodeDeclarationBuilder &b)
 {
+  b.use_custom_socket_order();
+  b.allow_any_socket_order();
+
   b.add_input<decl::Color>("Image")
       .default_value({1.0f, 1.0f, 1.0f, 1.0f})
-      .compositor_domain_priority(0);
-  b.add_output<decl::Color>("Image");
+      .hide_value()
+      .structure_type(StructureType::Dynamic);
+
+  b.add_output<decl::Color>("Image").structure_type(StructureType::Dynamic).align_with_previous();
+
+  b.add_input<decl::Menu>("Type")
+      .default_value(CMP_NODE_TONE_MAP_PHOTORECEPTOR)
+      .static_items(type_items)
+      .optional_label();
+
+  b.add_input<decl::Float>("Key")
+      .default_value(0.18f)
+      .min(0.0f)
+      .usage_by_single_menu(CMP_NODE_TONE_MAP_SIMPLE)
+      .description(
+          "The luminance that will be mapped to the log average luminance, typically set to the "
+          "middle gray value");
+  b.add_input<decl::Float>("Balance")
+      .default_value(1.0f)
+      .min(0.0f)
+      .usage_by_single_menu(CMP_NODE_TONE_MAP_SIMPLE)
+      .description(
+          "Balances low and high luminance areas. Lower values emphasize details in shadows, "
+          "while higher values compress highlights more smoothly");
+  b.add_input<decl::Float>("Gamma")
+      .default_value(1.0f)
+      .min(0.0f)
+      .usage_by_single_menu(CMP_NODE_TONE_MAP_SIMPLE)
+      .description("Gamma correction factor applied after tone mapping");
+
+  b.add_input<decl::Float>("Intensity")
+      .default_value(0.0f)
+      .usage_by_single_menu(CMP_NODE_TONE_MAP_PHOTORECEPTOR)
+      .description(
+          "Controls the intensity of the image, lower values makes it darker while higher values "
+          "makes it lighter");
+  b.add_input<decl::Float>("Contrast")
+      .default_value(0.0f)
+      .min(0.0f)
+      .usage_by_single_menu(CMP_NODE_TONE_MAP_PHOTORECEPTOR)
+      .description(
+          "Controls the contrast of the image. Zero automatically sets the contrast based on its "
+          "global range for better luminance distribution");
+  b.add_input<decl::Float>("Light Adaptation")
+      .default_value(0.0f)
+      .subtype(PROP_FACTOR)
+      .min(0.0f)
+      .max(1.0f)
+      .usage_by_single_menu(CMP_NODE_TONE_MAP_PHOTORECEPTOR)
+      .description(
+          "Specifies if tone mapping operates on the entire image or per pixel, 0 means the "
+          "entire image, 1 means it is per pixel, and values in between blends between both");
+  b.add_input<decl::Float>("Chromatic Adaptation")
+      .default_value(0.0f)
+      .subtype(PROP_FACTOR)
+      .min(0.0f)
+      .max(1.0f)
+      .usage_by_single_menu(CMP_NODE_TONE_MAP_PHOTORECEPTOR)
+      .description(
+          "Specifies if tone mapping operates on the luminance or on each channel independently, "
+          "0 means it uses luminance, 1 means it is per channel, and values in between blends "
+          "between both");
 }
 
 static void node_composit_init_tonemap(bNodeTree * /*ntree*/, bNode *node)
 {
-  NodeTonemap *ntm = MEM_cnew<NodeTonemap>(__func__);
-  ntm->type = 1;
-  ntm->key = 0.18;
-  ntm->offset = 1;
-  ntm->gamma = 1;
-  ntm->f = 0;
-  ntm->m = 0; /* Actual value is set according to input. */
-  /* Default a of 1 works well with natural HDR images, but not always so for CGI.
-   * Maybe should use 0 or at least lower initial value instead. */
-  ntm->a = 1;
-  ntm->c = 0;
+  /* Unused, but still allocated for forward compatibility. */
+  NodeTonemap *ntm = MEM_callocN<NodeTonemap>(__func__);
   node->storage = ntm;
 }
 
-static void node_composit_buts_tonemap(uiLayout *layout, bContext * /*C*/, PointerRNA *ptr)
-{
-  uiLayout *col;
-
-  col = uiLayoutColumn(layout, false);
-  uiItemR(col, ptr, "tonemap_type", UI_ITEM_R_SPLIT_EMPTY_NAME, "", ICON_NONE);
-  if (RNA_enum_get(ptr, "tonemap_type") == 0) {
-    uiItemR(
-        col, ptr, "key", UI_ITEM_R_SPLIT_EMPTY_NAME | UI_ITEM_R_SLIDER, std::nullopt, ICON_NONE);
-    uiItemR(col, ptr, "offset", UI_ITEM_R_SPLIT_EMPTY_NAME, std::nullopt, ICON_NONE);
-    uiItemR(col, ptr, "gamma", UI_ITEM_R_SPLIT_EMPTY_NAME, std::nullopt, ICON_NONE);
-  }
-  else {
-    uiItemR(col, ptr, "intensity", UI_ITEM_R_SPLIT_EMPTY_NAME, std::nullopt, ICON_NONE);
-    uiItemR(col,
-            ptr,
-            "contrast",
-            UI_ITEM_R_SPLIT_EMPTY_NAME | UI_ITEM_R_SLIDER,
-            std::nullopt,
-            ICON_NONE);
-    uiItemR(col,
-            ptr,
-            "adaptation",
-            UI_ITEM_R_SPLIT_EMPTY_NAME | UI_ITEM_R_SLIDER,
-            std::nullopt,
-            ICON_NONE);
-    uiItemR(col,
-            ptr,
-            "correction",
-            UI_ITEM_R_SPLIT_EMPTY_NAME | UI_ITEM_R_SLIDER,
-            std::nullopt,
-            ICON_NONE);
-  }
-}
-
 using namespace blender::compositor;
 
 class ToneMapOperation : public NodeOperation {
@@ -97,10 +124,10 @@ class ToneMapOperation : public NodeOperation {
 
   void execute() override
   {
-    Result &input_image = get_input("Image");
-    Result &output_image = get_result("Image");
+    const Result &input_image = this->get_input("Image");
+    Result &output_image = this->get_result("Image");
     if (input_image.is_single_value()) {
-      input_image.pass_through(output_image);
+      output_image.share_data(input_image);
       return;
     }
 
@@ -111,10 +138,9 @@ class ToneMapOperation : public NodeOperation {
       case CMP_NODE_TONE_MAP_PHOTORECEPTOR:
         execute_photoreceptor();
         return;
-      default:
-        BLI_assert_unreachable();
-        return;
     }
+
+    output_image.share_data(input_image);
   }
 
   /* Tone mapping based on equation (3) from Reinhard, Erik, et al. "Photographic tone reproduction
@@ -134,10 +160,10 @@ class ToneMapOperation : public NodeOperation {
   {
     const float luminance_scale = compute_luminance_scale();
     const float luminance_scale_blend_factor = compute_luminance_scale_blend_factor();
-    const float gamma = node_storage(bnode()).gamma;
+    const float gamma = this->get_gamma();
     const float inverse_gamma = gamma != 0.0f ? 1.0f / gamma : 0.0f;
 
-    GPUShader *shader = context().get_shader("compositor_tone_map_simple");
+    gpu::Shader *shader = context().get_shader("compositor_tone_map_simple");
     GPU_shader_bind(shader);
 
     GPU_shader_uniform_1f(shader, "luminance_scale", luminance_scale);
@@ -163,7 +189,7 @@ class ToneMapOperation : public NodeOperation {
   {
     const float luminance_scale = compute_luminance_scale();
     const float luminance_scale_blend_factor = compute_luminance_scale_blend_factor();
-    const float gamma = node_storage(bnode()).gamma;
+    const float gamma = this->get_gamma();
     const float inverse_gamma = gamma != 0.0f ? 1.0f / gamma : 0.0f;
 
     const Result &image = get_input("Image");
@@ -195,7 +221,7 @@ class ToneMapOperation : public NodeOperation {
   float compute_luminance_scale()
   {
     const float geometric_mean = compute_geometric_mean_of_luminance();
-    return geometric_mean != 0.0 ? node_storage(bnode()).key / geometric_mean : 0.0f;
+    return geometric_mean != 0.0 ? this->get_key() / geometric_mean : 0.0f;
   }
 
   /* Computes equation (1) from Reinhard's 2002 paper. However, note that the equation in the paper
@@ -208,6 +234,11 @@ class ToneMapOperation : public NodeOperation {
     return std::exp(compute_average_log_luminance());
   }
 
+  float get_key()
+  {
+    return math::max(0.0f, this->get_input("Key").get_single_value_default(0.18f));
+  }
+
   /* Equation (3) from Reinhard's 2002 paper blends between high luminance scaling for high
    * luminance values and low luminance scaling for low luminance values. This is done by adding 1
    * to the denominator, since for low luminance values, the denominator will be close to 1 and for
@@ -216,7 +247,12 @@ class ToneMapOperation : public NodeOperation {
    * a parameter to the user for more flexibility. */
   float compute_luminance_scale_blend_factor()
   {
-    return node_storage(bnode()).offset;
+    return math::max(0.0f, this->get_input("Balance").get_single_value_default(1.0f));
+  }
+
+  float get_gamma()
+  {
+    return math::max(0.0f, this->get_input("Gamma").get_single_value_default(1.0f));
   }
 
   /* Tone mapping based on equation (1) and the trilinear interpolation between equations (6) and
@@ -240,7 +276,7 @@ class ToneMapOperation : public NodeOperation {
     const float chromatic_adaptation = get_chromatic_adaptation();
     const float light_adaptation = get_light_adaptation();
 
-    GPUShader *shader = context().get_shader("compositor_tone_map_photoreceptor");
+    gpu::Shader *shader = context().get_shader("compositor_tone_map_photoreceptor");
     GPU_shader_bind(shader);
 
     GPU_shader_uniform_4fv(shader, "global_adaptation_level", global_adaptation_level);
@@ -345,15 +381,15 @@ class ToneMapOperation : public NodeOperation {
   /* Computes equation (5) from Reinhard's 2005 paper. */
   float compute_intensity()
   {
-    return std::exp(-node_storage(bnode()).f);
+    return std::exp(-this->get_intensity());
   }
 
   /* If the contrast is not zero, return it, otherwise, a zero contrast denote automatic derivation
    * of the contrast value based on equations (2) and (4) from Reinhard's 2005 paper. */
   float compute_contrast()
   {
-    if (node_storage(bnode()).m != 0.0f) {
-      return node_storage(bnode()).m;
+    if (this->get_contrast() != 0.0f) {
+      return this->get_contrast();
     }
 
     const float log_maximum_luminance = compute_log_maximum_luminance();
@@ -399,19 +435,34 @@ class ToneMapOperation : public NodeOperation {
     return std::log(math::max(minimum, 1e-5f));
   }
 
+  float get_intensity()
+  {
+    return this->get_input("Intensity").get_single_value_default(0.0f);
+  }
+
+  float get_contrast()
+  {
+    return math::max(0.0f, this->get_input("Contrast").get_single_value_default(0.0f));
+  }
+
   float get_chromatic_adaptation()
   {
-    return node_storage(bnode()).c;
+    return math::clamp(
+        this->get_input("Chromatic Adaptation").get_single_value_default(0.0f), 0.0f, 1.0f);
   }
 
   float get_light_adaptation()
   {
-    return node_storage(bnode()).a;
+    return math::clamp(
+        this->get_input("Light Adaptation").get_single_value_default(0.0f), 0.0f, 1.0f);
   }
 
   CMPNodeToneMapType get_type()
   {
-    return static_cast<CMPNodeToneMapType>(node_storage(bnode()).type);
+    const Result &input = this->get_input("Type");
+    const MenuValue default_menu_value = MenuValue(CMP_NODE_TONE_MAP_PHOTORECEPTOR);
+    const MenuValue menu_value = input.get_single_value_default(default_menu_value);
+    return static_cast<CMPNodeToneMapType>(menu_value.value);
   }
 };
 
@@ -422,7 +473,7 @@ static NodeOperation *get_compositor_operation(Context &context, DNode node)
 
 }  // namespace blender::nodes::node_composite_tonemap_cc
 
-void register_node_type_cmp_tonemap()
+static void register_node_type_cmp_tonemap()
 {
   namespace file_ns = blender::nodes::node_composite_tonemap_cc;
 
@@ -436,11 +487,11 @@ void register_node_type_cmp_tonemap()
   ntype.enum_name_legacy = "TONEMAP";
   ntype.nclass = NODE_CLASS_OP_COLOR;
   ntype.declare = file_ns::cmp_node_tonemap_declare;
-  ntype.draw_buttons = file_ns::node_composit_buts_tonemap;
   ntype.initfunc = file_ns::node_composit_init_tonemap;
   blender::bke::node_type_storage(
-      &ntype, "NodeTonemap", node_free_standard_storage, node_copy_standard_storage);
+      ntype, "NodeTonemap", node_free_standard_storage, node_copy_standard_storage);
   ntype.get_compositor_operation = file_ns::get_compositor_operation;
 
-  blender::bke::node_register_type(&ntype);
+  blender::bke::node_register_type(ntype);
 }
+NOD_REGISTER_NODE(register_node_type_cmp_tonemap)
