@@ -13,18 +13,18 @@
 
 #include <Python.h>
 
-#include "BLI_string.h"
+#include "BLI_string_utf8.h"
 
 #include "GPU_context.hh"
 #include "GPU_uniform_buffer.hh"
 
-#include "../generic/python_compat.hh"
+#include "../generic/python_compat.hh" /* IWYU pragma: keep. */
 
 #include "gpu_py.hh"
 #include "gpu_py_uniformbuffer.hh" /* own include */
 
 /* -------------------------------------------------------------------- */
-/** \name GPUUniformBuf Common Utilities
+/** \name blender::gpu::UniformBuf Common Utilities
  * \{ */
 
 static int pygpu_uniformbuffer_valid_check(BPyGPUUniformBuf *bpygpu_ub)
@@ -53,7 +53,7 @@ static int pygpu_uniformbuffer_valid_check(BPyGPUUniformBuf *bpygpu_ub)
 /** \} */
 
 /* -------------------------------------------------------------------- */
-/** \name GPUUniformBuf Type
+/** \name blender::gpu::UniformBuf Type
  * \{ */
 
 static PyObject *pygpu_uniformbuffer__tp_new(PyTypeObject * /*self*/,
@@ -62,7 +62,7 @@ static PyObject *pygpu_uniformbuffer__tp_new(PyTypeObject * /*self*/,
 {
   BPYGPU_IS_INIT_OR_ERROR_OBJ;
 
-  GPUUniformBuf *ubo = nullptr;
+  blender::gpu::UniformBuf *ubo = nullptr;
   PyObject *pybuffer_obj;
   char err_out[256] = "unknown error. See console";
 
@@ -79,7 +79,7 @@ static PyObject *pygpu_uniformbuffer__tp_new(PyTypeObject * /*self*/,
   }
 
   if (!GPU_context_active_get()) {
-    STRNCPY(err_out, "No active GPU context found");
+    STRNCPY_UTF8(err_out, "No active GPU context found");
   }
   else {
     Py_buffer pybuffer;
@@ -89,7 +89,7 @@ static PyObject *pygpu_uniformbuffer__tp_new(PyTypeObject * /*self*/,
     }
 
     if ((pybuffer.len % 16) != 0) {
-      STRNCPY(err_out, "UBO is not padded to size of vec4");
+      STRNCPY_UTF8(err_out, "UBO is not padded to size of vec4");
     }
     else {
       ubo = GPU_uniformbuf_create_ex(pybuffer.len, pybuffer.buf, "python_uniformbuffer");
@@ -231,7 +231,7 @@ PyTypeObject BPyGPUUniformBuf_Type = {
 /** \name Public API
  * \{ */
 
-PyObject *BPyGPUUniformBuf_CreatePyObject(GPUUniformBuf *ubo)
+PyObject *BPyGPUUniformBuf_CreatePyObject(blender::gpu::UniformBuf *ubo)
 {
   BPyGPUUniformBuf *self;
 
