@@ -14,7 +14,7 @@
 #include "MEM_guardedalloc.h"
 
 #include "BLI_ghash.h"
-#include "BLI_kdopbvh.h"
+#include "BLI_kdopbvh.hh"
 #include "BLI_math_geom.h"
 #include "BLI_math_matrix.h"
 #include "BLI_math_vector.h"
@@ -22,34 +22,32 @@
 #include "BLI_polyfill_2d.h"
 #include "BLI_utildefines.h"
 
-#include "BKE_bvhutils.h"
+#include "BKE_bvhutils.hh"
 
-#include "../generic/py_capi_utils.h"
-#include "../generic/python_utildefines.h"
+#include "../generic/py_capi_utils.hh"
+#include "../generic/python_utildefines.hh"
 
-#include "mathutils.h"
-#include "mathutils_bvhtree.h" /* own include */
+#include "mathutils.hh"
+#include "mathutils_bvhtree.hh" /* own include */
 
 #ifndef MATH_STANDALONE
 #  include "DNA_mesh_types.h"
-#  include "DNA_meshdata_types.h"
 #  include "DNA_object_types.h"
 
-#  include "BKE_customdata.h"
-#  include "BKE_editmesh_bvh.h"
-#  include "BKE_lib_id.h"
+#  include "BKE_customdata.hh"
+#  include "BKE_lib_id.hh"
 #  include "BKE_mesh.hh"
 #  include "BKE_mesh_runtime.hh"
 #  include "BKE_object.hh"
 
 #  include "DEG_depsgraph_query.hh"
 
-#  include "bmesh.h"
+#  include "bmesh.hh"
 
-#  include "../bmesh/bmesh_py_types.h"
+#  include "../bmesh/bmesh_py_types.hh"
 #endif /* MATH_STANDALONE */
 
-#include "BLI_strict_flags.h"
+#include "BLI_strict_flags.h" /* IWYU pragma: keep. Keep last. */
 
 /* -------------------------------------------------------------------- */
 /** \name Documentation String (snippets)
@@ -60,15 +58,13 @@
   "   :type distance: float\n"
 
 #define PYBVH_FIND_GENERIC_RETURN_DOC \
-  "   :return: Returns a tuple\n" \
-  "      (:class:`Vector` location, :class:`Vector` normal, int index, float distance),\n" \
+  "   :return: Returns a tuple: (position, normal, index, distance),\n" \
   "      Values will all be None if no hit is found.\n" \
-  "   :rtype: :class:`tuple`\n"
+  "   :rtype: tuple[:class:`Vector` | None, :class:`Vector` | None, int | None, float | None]\n"
 
 #define PYBVH_FIND_GENERIC_RETURN_LIST_DOC \
-  "   :return: Returns a list of tuples\n" \
-  "      (:class:`Vector` location, :class:`Vector` normal, int index, float distance),\n" \
-  "   :rtype: :class:`list`\n"
+  "   :return: Returns a list of tuples (position, normal, index, distance)\n" \
+  "   :rtype: list[tuple[:class:`Vector`, :class:`Vector`, int, float]]\n"
 
 #define PYBVH_FROM_GENERIC_EPSILON_DOC \
   "   :arg epsilon: Increase the threshold for detecting overlap and raycast hits.\n" \
@@ -267,16 +263,17 @@ static void py_bvhtree_raycast_cb(void *userdata,
 {
   const PyBVHTree *self = static_cast<const PyBVHTree *>(userdata);
 
-  const float(*coords)[3] = (const float(*)[3])self->coords;
+  const float (*coords)[3] = self->coords;
   const uint *tri = self->tris[index];
   const float *tri_co[3] = {coords[tri[0]], coords[tri[1]], coords[tri[2]]};
   float dist;
 
   if (self->epsilon == 0.0f) {
-    dist = bvhtree_ray_tri_intersection(ray, hit->dist, UNPACK3(tri_co));
+    dist = blender::bke::bvhtree_ray_tri_intersection(ray, hit->dist, UNPACK3(tri_co));
   }
   else {
-    dist = bvhtree_sphereray_tri_intersection(ray, self->epsilon, hit->dist, UNPACK3(tri_co));
+    dist = blender::bke::bvhtree_sphereray_tri_intersection(
+        ray, self->epsilon, hit->dist, UNPACK3(tri_co));
   }
 
   if (dist >= 0 && dist < hit->dist) {
@@ -299,7 +296,7 @@ static void py_bvhtree_nearest_point_cb(void *userdata,
 {
   PyBVHTree *self = static_cast<PyBVHTree *>(userdata);
 
-  const float(*coords)[3] = (const float(*)[3])self->coords;
+  const float (*coords)[3] = (const float (*)[3])self->coords;
   const uint *tri = self->tris[index];
   const float *tri_co[3] = {coords[tri[0]], coords[tri[1]], coords[tri[2]]};
   float nearest_tmp[3], dist_sq;
@@ -320,16 +317,18 @@ static void py_bvhtree_nearest_point_cb(void *userdata,
   }
 }
 
-PyDoc_STRVAR(py_bvhtree_ray_cast_doc,
-             ".. method:: ray_cast(origin, direction, distance=sys.float_info.max)\n"
-             "\n"
-             "   Cast a ray onto the mesh.\n"
-             "\n"
-             "   :arg origin: Start location of the ray in object space.\n"
-             "   :type origin: :class:`Vector`\n"
-             "   :arg direction: Direction of the ray in object space.\n"
-             "   :type direction: :class:`Vector`\n" PYBVH_FIND_GENERIC_DISTANCE_DOC
-                 PYBVH_FIND_GENERIC_RETURN_DOC);
+PyDoc_STRVAR(
+    /* Wrap. */
+    py_bvhtree_ray_cast_doc,
+    ".. method:: ray_cast(origin, direction, distance=sys.float_info.max, /)\n"
+    "\n"
+    "   Cast a ray onto the mesh.\n"
+    "\n"
+    "   :arg origin: Start location of the ray in object space.\n"
+    "   :type origin: :class:`Vector`\n"
+    "   :arg direction: Direction of the ray in object space.\n"
+    "   :type direction: :class:`Vector`\n" PYBVH_FIND_GENERIC_DISTANCE_DOC
+        PYBVH_FIND_GENERIC_RETURN_DOC);
 static PyObject *py_bvhtree_ray_cast(PyBVHTree *self, PyObject *args)
 {
   const char *error_prefix = "ray_cast";
@@ -360,7 +359,8 @@ static PyObject *py_bvhtree_ray_cast(PyBVHTree *self, PyObject *args)
   /* may fail if the mesh has no faces, in that case the ray-cast misses */
   if (self->tree) {
     if (BLI_bvhtree_ray_cast(self->tree, co, direction, 0.0f, &hit, py_bvhtree_raycast_cb, self) !=
-        -1) {
+        -1)
+    {
       return py_bvhtree_raycast_to_py(&hit);
     }
   }
@@ -368,15 +368,17 @@ static PyObject *py_bvhtree_ray_cast(PyBVHTree *self, PyObject *args)
   return py_bvhtree_raycast_to_py_none();
 }
 
-PyDoc_STRVAR(py_bvhtree_find_nearest_doc,
-             ".. method:: find_nearest(origin, distance=" PYBVH_MAX_DIST_STR
-             ")\n"
-             "\n"
-             "   Find the nearest element (typically face index) to a point.\n"
-             "\n"
-             "   :arg co: Find nearest element to this point.\n"
-             "   :type co: :class:`Vector`\n" PYBVH_FIND_GENERIC_DISTANCE_DOC
-                 PYBVH_FIND_GENERIC_RETURN_DOC);
+PyDoc_STRVAR(
+    /* Wrap. */
+    py_bvhtree_find_nearest_doc,
+    ".. method:: find_nearest(origin, distance=" PYBVH_MAX_DIST_STR
+    ", /)\n"
+    "\n"
+    "   Find the nearest element (typically face index) to a point.\n"
+    "\n"
+    "   :arg co: Find nearest element to this point.\n"
+    "   :type co: :class:`Vector`\n" PYBVH_FIND_GENERIC_DISTANCE_DOC
+        PYBVH_FIND_GENERIC_RETURN_DOC);
 static PyObject *py_bvhtree_find_nearest(PyBVHTree *self, PyObject *args)
 {
   const char *error_prefix = "find_nearest";
@@ -404,7 +406,8 @@ static PyObject *py_bvhtree_find_nearest(PyBVHTree *self, PyObject *args)
   /* may fail if the mesh has no faces, in that case the ray-cast misses */
   if (self->tree) {
     if (BLI_bvhtree_find_nearest(self->tree, co, &nearest, py_bvhtree_nearest_point_cb, self) !=
-        -1) {
+        -1)
+    {
       return py_bvhtree_nearest_to_py(&nearest);
     }
   }
@@ -426,7 +429,7 @@ static void py_bvhtree_nearest_point_range_cb(void *userdata,
   PyBVH_RangeData *data = static_cast<PyBVH_RangeData *>(userdata);
   PyBVHTree *self = data->self;
 
-  const float(*coords)[3] = self->coords;
+  const float (*coords)[3] = self->coords;
   const uint *tri = self->tris[index];
   const float *tri_co[3] = {coords[tri[0]], coords[tri[1]], coords[tri[2]]};
   float nearest_tmp[3], dist_sq;
@@ -451,9 +454,10 @@ static void py_bvhtree_nearest_point_range_cb(void *userdata,
 }
 
 PyDoc_STRVAR(
+    /* Wrap. */
     py_bvhtree_find_nearest_range_doc,
     ".. method:: find_nearest_range(origin, distance=" PYBVH_MAX_DIST_STR
-    ")\n"
+    ", /)\n"
     "\n"
     "   Find the nearest elements (typically face index) to a point in the distance range.\n"
     "\n"
@@ -544,8 +548,9 @@ static bool py_bvhtree_overlap_cb(void *userdata, int index_a, int index_b, int
 }
 
 PyDoc_STRVAR(
+    /* Wrap. */
     py_bvhtree_overlap_doc,
-    ".. method:: overlap(other_tree)\n"
+    ".. method:: overlap(other_tree, /)\n"
     "\n"
     "   Find overlapping indices between 2 trees.\n"
     "\n"
@@ -553,7 +558,7 @@ PyDoc_STRVAR(
     "   :type other_tree: :class:`BVHTree`\n"
     "   :return: Returns a list of unique index pairs,"
     "      the first index referencing this tree, the second referencing the **other_tree**.\n"
-    "   :rtype: :class:`list`\n");
+    "   :rtype: list[tuple[int, int]]\n");
 static PyObject *py_bvhtree_overlap(PyBVHTree *self, PyBVHTree *other)
 {
   PyBVHTree_OverlapData data;
@@ -629,15 +634,16 @@ static PyObject *py_bvhtree_overlap(PyBVHTree *self, PyBVHTree *other)
  * \{ */
 
 PyDoc_STRVAR(
+    /* Wrap. */
     C_BVHTree_FromPolygons_doc,
-    ".. classmethod:: FromPolygons(vertices, polygons, all_triangles=False, epsilon=0.0)\n"
+    ".. classmethod:: FromPolygons(vertices, polygons, *, all_triangles=False, epsilon=0.0)\n"
     "\n"
     "   BVH tree constructed geometry passed in as arguments.\n"
     "\n"
     "   :arg vertices: float triplets each representing ``(x, y, z)``\n"
-    "   :type vertices: float triplet sequence\n"
-    "   :arg polygons: Sequence of polyugons, each containing indices to the vertices argument.\n"
-    "   :type polygons: Sequence of sequences containing ints\n"
+    "   :type vertices: Sequence[Sequence[float]]\n"
+    "   :arg polygons: Sequence of polygons, each containing indices to the vertices argument.\n"
+    "   :type polygons: Sequence[Sequence[int]]\n"
     "   :arg all_triangles: Use when all **polygons** are triangles for more efficient "
     "conversion.\n"
     "   :type all_triangles: bool\n" PYBVH_FROM_GENERIC_EPSILON_DOC);
@@ -652,7 +658,7 @@ static PyObject *C_BVHTree_FromPolygons(PyObject * /*cls*/, PyObject *args, PyOb
   MemArena *poly_arena = nullptr;
   MemArena *pf_arena = nullptr;
 
-  float(*coords)[3] = nullptr;
+  float (*coords)[3] = nullptr;
   uint(*tris)[3] = nullptr;
   uint coords_len, tris_len;
   float epsilon = 0.0f;
@@ -660,7 +666,7 @@ static PyObject *C_BVHTree_FromPolygons(PyObject * /*cls*/, PyObject *args, PyOb
 
   /* when all_triangles is False */
   int *orig_index = nullptr;
-  float(*orig_normal)[3] = nullptr;
+  float (*orig_normal)[3] = nullptr;
 
   uint i;
   bool valid = true;
@@ -688,7 +694,7 @@ static PyObject *C_BVHTree_FromPolygons(PyObject * /*cls*/, PyObject *args, PyOb
   if (valid) {
     PyObject **py_coords_fast_items = PySequence_Fast_ITEMS(py_coords_fast);
     coords_len = uint(PySequence_Fast_GET_SIZE(py_coords_fast));
-    coords = static_cast<float(*)[3]>(MEM_mallocN(size_t(coords_len) * sizeof(*coords), __func__));
+    coords = MEM_malloc_arrayN<float[3]>(size_t(coords_len), __func__);
 
     for (i = 0; i < coords_len; i++) {
       PyObject *py_vert = py_coords_fast_items[i];
@@ -707,7 +713,7 @@ static PyObject *C_BVHTree_FromPolygons(PyObject * /*cls*/, PyObject *args, PyOb
     /* all triangles, simple case */
     PyObject **py_tris_fast_items = PySequence_Fast_ITEMS(py_tris_fast);
     tris_len = uint(PySequence_Fast_GET_SIZE(py_tris_fast));
-    tris = static_cast<uint(*)[3]>(MEM_mallocN(size_t(tris_len) * sizeof(*tris), __func__));
+    tris = MEM_malloc_arrayN<uint[3]>(size_t(tris_len), __func__);
 
     for (i = 0; i < tris_len; i++) {
       PyObject *py_tricoords = py_tris_fast_items[i];
@@ -784,7 +790,7 @@ static PyObject *C_BVHTree_FromPolygons(PyObject * /*cls*/, PyObject *args, PyOb
       plink = static_cast<PolyLink *>(BLI_memarena_alloc(
           poly_arena, sizeof(*plink) + (sizeof(int) * size_t(py_tricoords_len))));
 
-      plink->len = uint(py_tricoords_len);
+      plink->len = py_tricoords_len;
       *p_plink_prev = plink;
       p_plink_prev = &plink->next;
 
@@ -813,11 +819,10 @@ static PyObject *C_BVHTree_FromPolygons(PyObject * /*cls*/, PyObject *args, PyOb
     /* All NGON's are parsed, now tessellate. */
 
     pf_arena = BLI_memarena_new(BLI_POLYFILL_ARENA_SIZE, __func__);
-    tris = static_cast<uint(*)[3]>(MEM_mallocN(sizeof(*tris) * size_t(tris_len), __func__));
+    tris = MEM_malloc_arrayN<uint[3]>(size_t(tris_len), __func__);
 
-    orig_index = static_cast<int *>(MEM_mallocN(sizeof(*orig_index) * size_t(tris_len), __func__));
-    orig_normal = static_cast<float(*)[3]>(
-        MEM_mallocN(sizeof(*orig_normal) * size_t(polys_len), __func__));
+    orig_index = MEM_malloc_arrayN<int>(size_t(tris_len), __func__);
+    orig_normal = MEM_malloc_arrayN<float[3]>(size_t(polys_len), __func__);
 
     for (plink = plink_first, poly_index = 0, i = 0; plink; plink = plink->next, poly_index++) {
       if (plink->len == 3) {
@@ -828,7 +833,7 @@ static PyObject *C_BVHTree_FromPolygons(PyObject * /*cls*/, PyObject *args, PyOb
         i++;
       }
       else if (plink->len > 3) {
-        float(*proj_coords)[2] = static_cast<float(*)[2]>(
+        float (*proj_coords)[2] = static_cast<float (*)[2]>(
             BLI_memarena_alloc(pf_arena, sizeof(*proj_coords) * plink->len));
         float *normal = orig_normal[poly_index];
         const float *co_prev;
@@ -920,26 +925,27 @@ static PyObject *C_BVHTree_FromPolygons(PyObject * /*cls*/, PyObject *args, PyOb
 
 #ifndef MATH_STANDALONE
 
-PyDoc_STRVAR(C_BVHTree_FromBMesh_doc,
-             ".. classmethod:: FromBMesh(bmesh, epsilon=0.0)\n"
-             "\n"
-             "   BVH tree based on :class:`BMesh` data.\n"
-             "\n"
-             "   :arg bmesh: BMesh data.\n"
-             "   :type bmesh: :class:`BMesh`\n" PYBVH_FROM_GENERIC_EPSILON_DOC);
+PyDoc_STRVAR(
+    /* Wrap. */
+    C_BVHTree_FromBMesh_doc,
+    ".. classmethod:: FromBMesh(bmesh, *, epsilon=0.0)\n"
+    "\n"
+    "   BVH tree based on :class:`BMesh` data.\n"
+    "\n"
+    "   :arg bmesh: BMesh data.\n"
+    "   :type bmesh: :class:`BMesh`\n" PYBVH_FROM_GENERIC_EPSILON_DOC);
 static PyObject *C_BVHTree_FromBMesh(PyObject * /*cls*/, PyObject *args, PyObject *kwargs)
 {
   const char *keywords[] = {"bmesh", "epsilon", nullptr};
 
   BPy_BMesh *py_bm;
 
-  float(*coords)[3] = nullptr;
+  float (*coords)[3] = nullptr;
   uint(*tris)[3] = nullptr;
   uint coords_len, tris_len;
   float epsilon = 0.0f;
 
   BMesh *bm;
-  BMLoop *(*looptris)[3];
 
   if (!PyArg_ParseTupleAndKeywords(args,
                                    kwargs,
@@ -955,18 +961,15 @@ static PyObject *C_BVHTree_FromBMesh(PyObject * /*cls*/, PyObject *args, PyObjec
   bm = py_bm->bm;
 
   /* Get data for tessellation */
-  {
-    coords_len = uint(bm->totvert);
-    tris_len = uint(poly_to_tri_count(bm->totface, bm->totloop));
 
-    coords = static_cast<float(*)[3]>(MEM_mallocN(sizeof(*coords) * size_t(coords_len), __func__));
-    tris = static_cast<uint(*)[3]>(MEM_mallocN(sizeof(*tris) * size_t(tris_len), __func__));
+  coords_len = uint(bm->totvert);
+  tris_len = uint(poly_to_tri_count(bm->totface, bm->totloop));
 
-    looptris = static_cast<BMLoop *(*)[3]>(
-        MEM_mallocN(sizeof(*looptris) * size_t(tris_len), __func__));
+  coords = MEM_malloc_arrayN<float[3]>(size_t(coords_len), __func__);
+  tris = MEM_malloc_arrayN<uint[3]>(size_t(tris_len), __func__);
 
-    BM_mesh_calc_tessellation(bm, looptris);
-  }
+  blender::Array<std::array<BMLoop *, 3>> corner_tris(tris_len);
+  BM_mesh_calc_tessellation(bm, corner_tris);
 
   {
     BMIter iter;
@@ -974,17 +977,15 @@ static PyObject *C_BVHTree_FromBMesh(PyObject * /*cls*/, PyObject *args, PyObjec
     uint i;
 
     int *orig_index = nullptr;
-    float(*orig_normal)[3] = nullptr;
+    float (*orig_normal)[3] = nullptr;
 
     tree = BLI_bvhtree_new(int(tris_len), epsilon, PY_BVH_TREE_TYPE_DEFAULT, PY_BVH_AXIS_DEFAULT);
     if (tree) {
       BMFace *f;
       BMVert *v;
 
-      orig_index = static_cast<int *>(
-          MEM_mallocN(sizeof(*orig_index) * size_t(tris_len), __func__));
-      orig_normal = static_cast<float(*)[3]>(
-          MEM_mallocN(sizeof(*orig_normal) * size_t(bm->totface), __func__));
+      orig_index = MEM_malloc_arrayN<int>(size_t(tris_len), __func__);
+      orig_normal = MEM_malloc_arrayN<float[3]>(size_t(bm->totface), __func__);
 
       BM_ITER_MESH_INDEX (v, &iter, bm, BM_VERTS_OF_MESH, i) {
         copy_v3_v3(coords[i], v->co);
@@ -999,29 +1000,27 @@ static PyObject *C_BVHTree_FromBMesh(PyObject * /*cls*/, PyObject *args, PyObjec
       for (i = 0; i < tris_len; i++) {
         float co[3][3];
 
-        tris[i][0] = uint(BM_elem_index_get(looptris[i][0]->v));
-        tris[i][1] = uint(BM_elem_index_get(looptris[i][1]->v));
-        tris[i][2] = uint(BM_elem_index_get(looptris[i][2]->v));
+        tris[i][0] = uint(BM_elem_index_get(corner_tris[i][0]->v));
+        tris[i][1] = uint(BM_elem_index_get(corner_tris[i][1]->v));
+        tris[i][2] = uint(BM_elem_index_get(corner_tris[i][2]->v));
 
         copy_v3_v3(co[0], coords[tris[i][0]]);
         copy_v3_v3(co[1], coords[tris[i][1]]);
         copy_v3_v3(co[2], coords[tris[i][2]]);
 
         BLI_bvhtree_insert(tree, int(i), co[0], 3);
-        orig_index[i] = BM_elem_index_get(looptris[i][0]->f);
+        orig_index[i] = BM_elem_index_get(corner_tris[i][0]->f);
       }
 
       BLI_bvhtree_balance(tree);
     }
 
-    MEM_freeN(looptris);
-
     return bvhtree_CreatePyObject(
         tree, epsilon, coords, coords_len, tris, tris_len, orig_index, orig_normal);
   }
 }
 
-/* return various derived meshes based on requested settings */
+/** Return various evaluated meshes based on requested settings. */
 static const Mesh *bvh_get_mesh(const char *funcname,
                                 Depsgraph *depsgraph,
                                 Scene *scene,
@@ -1030,11 +1029,12 @@ static const Mesh *bvh_get_mesh(const char *funcname,
                                 const bool use_cage,
                                 bool *r_free_mesh)
 {
-  Object *ob_eval = DEG_get_evaluated_object(depsgraph, ob);
+  Object *ob_eval = DEG_get_evaluated(depsgraph, ob);
   /* we only need minimum mesh data for topology and vertex locations */
   const CustomData_MeshMasks data_masks = CD_MASK_BAREMESH;
   const bool use_render = DEG_get_mode(depsgraph) == DAG_EVAL_RENDER;
   *r_free_mesh = false;
+  Mesh *mesh;
 
   /* Write the display mesh into the dummy mesh */
   if (use_deform) {
@@ -1047,15 +1047,33 @@ static const Mesh *bvh_get_mesh(const char *funcname,
         return nullptr;
       }
 
+      mesh = blender::bke::mesh_create_eval_final(depsgraph, scene, ob, &data_masks);
+      if (mesh == nullptr) {
+        PyErr_Format(PyExc_ValueError,
+                     "%s(...): Cannot get a mesh from object '%s'",
+                     ob->id.name + 2,
+                     funcname);
+        return nullptr;
+      }
+
       *r_free_mesh = true;
-      return mesh_create_eval_final(depsgraph, scene, ob, &data_masks);
+      return mesh;
     }
     if (ob_eval != nullptr) {
       if (use_cage) {
-        return mesh_get_eval_deform(depsgraph, scene, ob_eval, &data_masks);
+        mesh = blender::bke::mesh_get_eval_deform(depsgraph, scene, ob_eval, &data_masks);
       }
-
-      return BKE_object_get_evaluated_mesh(ob_eval);
+      else {
+        mesh = BKE_object_get_evaluated_mesh(ob_eval);
+      }
+      if (mesh == nullptr) {
+        PyErr_Format(PyExc_ValueError,
+                     "%s(...): Cannot get a mesh from object '%s'",
+                     ob->id.name + 2,
+                     funcname);
+        return nullptr;
+      }
+      return mesh;
     }
 
     PyErr_Format(PyExc_ValueError,
@@ -1073,9 +1091,16 @@ static const Mesh *bvh_get_mesh(const char *funcname,
           funcname);
       return nullptr;
     }
-
+    mesh = blender::bke::mesh_create_eval_no_deform_render(depsgraph, scene, ob, &data_masks);
+    if (mesh == nullptr) {
+      PyErr_Format(PyExc_ValueError,
+                   "%s(...): Cannot get a mesh from object '%s'",
+                   ob->id.name + 2,
+                   funcname);
+      return nullptr;
+    }
     *r_free_mesh = true;
-    return mesh_create_eval_no_deform_render(depsgraph, scene, ob, &data_masks);
+    return mesh;
   }
 
   if (use_cage) {
@@ -1086,24 +1111,34 @@ static const Mesh *bvh_get_mesh(const char *funcname,
     return nullptr;
   }
 
+  mesh = blender::bke::mesh_create_eval_no_deform(depsgraph, scene, ob, &data_masks);
+  if (mesh == nullptr) {
+    PyErr_Format(PyExc_ValueError,
+                 "%s(...): Cannot get a mesh from object '%s'",
+                 ob->id.name + 2,
+                 funcname);
+    return nullptr;
+  }
   *r_free_mesh = true;
-  return mesh_create_eval_no_deform(depsgraph, scene, ob, &data_masks);
+  return mesh;
 }
 
-PyDoc_STRVAR(C_BVHTree_FromObject_doc,
-             ".. classmethod:: FromObject(object, depsgraph, deform=True, render=False, "
-             "cage=False, epsilon=0.0)\n"
-             "\n"
-             "   BVH tree based on :class:`Object` data.\n"
-             "\n"
-             "   :arg object: Object data.\n"
-             "   :type object: :class:`Object`\n"
-             "   :arg depsgraph: Depsgraph to use for evaluating the mesh.\n"
-             "   :type depsgraph: :class:`Depsgraph`\n"
-             "   :arg deform: Use mesh with deformations.\n"
-             "   :type deform: bool\n"
-             "   :arg cage: Use modifiers cage.\n"
-             "   :type cage: bool\n" PYBVH_FROM_GENERIC_EPSILON_DOC);
+PyDoc_STRVAR(
+    /* Wrap. */
+    C_BVHTree_FromObject_doc,
+    ".. classmethod:: FromObject(object, depsgraph, *, deform=True, render=False, "
+    "cage=False, epsilon=0.0)\n"
+    "\n"
+    "   BVH tree based on :class:`Object` data.\n"
+    "\n"
+    "   :arg object: Object data.\n"
+    "   :type object: :class:`Object`\n"
+    "   :arg depsgraph: Depsgraph to use for evaluating the mesh.\n"
+    "   :type depsgraph: :class:`Depsgraph`\n"
+    "   :arg deform: Use mesh with deformations.\n"
+    "   :type deform: bool\n"
+    "   :arg cage: Use modifiers cage.\n"
+    "   :type cage: bool\n" PYBVH_FROM_GENERIC_EPSILON_DOC);
 static PyObject *C_BVHTree_FromObject(PyObject * /*cls*/, PyObject *args, PyObject *kwargs)
 {
   /* NOTE: options here match #bpy_bmesh_from_object. */
@@ -1146,18 +1181,16 @@ static PyObject *C_BVHTree_FromObject(PyObject * /*cls*/, PyObject *args, PyObje
   }
 
   const blender::Span<int> corner_verts = mesh->corner_verts();
-  const blender::Span<MLoopTri> looptris = mesh->looptris();
-  const blender::Span<int> looptri_faces = mesh->looptri_faces();
+  const blender::Span<blender::int3> corner_tris = mesh->corner_tris();
+  const blender::Span<int> tri_faces = mesh->corner_tri_faces();
 
   /* Get data for tessellation */
 
-  const uint coords_len = uint(mesh->totvert);
+  const uint coords_len = uint(mesh->verts_num);
 
-  float(*coords)[3] = static_cast<float(*)[3]>(
-      MEM_mallocN(sizeof(*coords) * size_t(coords_len), __func__));
-  uint(*tris)[3] = static_cast<uint(*)[3]>(
-      MEM_mallocN(sizeof(*tris) * size_t(looptris.size()), __func__));
-  memcpy(coords, mesh->vert_positions().data(), sizeof(float[3]) * size_t(mesh->totvert));
+  float (*coords)[3] = MEM_malloc_arrayN<float[3]>(size_t(coords_len), __func__);
+  uint(*tris)[3] = MEM_malloc_arrayN<uint[3]>(size_t(corner_tris.size()), __func__);
+  memcpy(coords, mesh->vert_positions().data(), sizeof(float[3]) * size_t(mesh->verts_num));
 
   BVHTree *tree;
 
@@ -1165,30 +1198,28 @@ static PyObject *C_BVHTree_FromObject(PyObject * /*cls*/, PyObject *args, PyObje
   blender::float3 *orig_normal = nullptr;
 
   tree = BLI_bvhtree_new(
-      int(looptris.size()), epsilon, PY_BVH_TREE_TYPE_DEFAULT, PY_BVH_AXIS_DEFAULT);
+      int(corner_tris.size()), epsilon, PY_BVH_TREE_TYPE_DEFAULT, PY_BVH_AXIS_DEFAULT);
   if (tree) {
-    orig_index = static_cast<int *>(
-        MEM_mallocN(sizeof(*orig_index) * size_t(looptris.size()), __func__));
+    orig_index = MEM_malloc_arrayN<int>(size_t(corner_tris.size()), __func__);
     if (!BKE_mesh_face_normals_are_dirty(mesh)) {
       const blender::Span<blender::float3> face_normals = mesh->face_normals();
-      orig_normal = static_cast<blender::float3 *>(
-          MEM_malloc_arrayN(size_t(mesh->faces_num), sizeof(blender::float3), __func__));
+      orig_normal = MEM_malloc_arrayN<blender::float3>(size_t(mesh->faces_num), __func__);
       blender::MutableSpan(orig_normal, face_normals.size()).copy_from(face_normals);
     }
 
-    for (const int64_t i : looptris.index_range()) {
+    for (const int64_t i : corner_tris.index_range()) {
       float co[3][3];
 
-      tris[i][0] = uint(corner_verts[looptris[i].tri[0]]);
-      tris[i][1] = uint(corner_verts[looptris[i].tri[1]]);
-      tris[i][2] = uint(corner_verts[looptris[i].tri[2]]);
+      tris[i][0] = uint(corner_verts[corner_tris[i][0]]);
+      tris[i][1] = uint(corner_verts[corner_tris[i][1]]);
+      tris[i][2] = uint(corner_verts[corner_tris[i][2]]);
 
       copy_v3_v3(co[0], coords[tris[i][0]]);
       copy_v3_v3(co[1], coords[tris[i][1]]);
       copy_v3_v3(co[2], coords[tris[i][2]]);
 
       BLI_bvhtree_insert(tree, int(i), co[0], 3);
-      orig_index[i] = int(looptri_faces[i]);
+      orig_index[i] = int(tri_faces[i]);
     }
 
     BLI_bvhtree_balance(tree);
@@ -1203,9 +1234,9 @@ static PyObject *C_BVHTree_FromObject(PyObject * /*cls*/, PyObject *args, PyObje
                                 coords,
                                 coords_len,
                                 tris,
-                                uint(looptris.size()),
+                                uint(corner_tris.size()),
                                 orig_index,
-                                reinterpret_cast<float(*)[3]>(orig_normal));
+                                reinterpret_cast<float (*)[3]>(orig_normal));
 }
 #endif /* MATH_STANDALONE */
 
@@ -1215,9 +1246,14 @@ static PyObject *C_BVHTree_FromObject(PyObject * /*cls*/, PyObject *args, PyObje
 /** \name Module & Type definition
  * \{ */
 
-#if (defined(__GNUC__) && !defined(__clang__))
-#  pragma GCC diagnostic push
-#  pragma GCC diagnostic ignored "-Wcast-function-type"
+#ifdef __GNUC__
+#  ifdef __clang__
+#    pragma clang diagnostic push
+#    pragma clang diagnostic ignored "-Wcast-function-type"
+#  else
+#    pragma GCC diagnostic push
+#    pragma GCC diagnostic ignored "-Wcast-function-type"
+#  endif
 #endif
 
 static PyMethodDef py_bvhtree_methods[] = {
@@ -1253,8 +1289,12 @@ static PyMethodDef py_bvhtree_methods[] = {
     {nullptr, nullptr, 0, nullptr},
 };
 
-#if (defined(__GNUC__) && !defined(__clang__))
-#  pragma GCC diagnostic pop
+#ifdef __GNUC__
+#  ifdef __clang__
+#    pragma clang diagnostic pop
+#  else
+#    pragma GCC diagnostic pop
+#  endif
 #endif
 
 PyTypeObject PyBVHTree_Type = {
@@ -1312,8 +1352,10 @@ PyTypeObject PyBVHTree_Type = {
 /* -------------------------------------------------------------------- */
 /* Module definition */
 
-PyDoc_STRVAR(py_bvhtree_doc,
-             "BVH tree structures for proximity searches and ray casts on geometry.");
+PyDoc_STRVAR(
+    /* Wrap. */
+    py_bvhtree_doc,
+    "BVH tree structures for proximity searches and ray casts on geometry.");
 static PyModuleDef bvhtree_moduledef = {
     /*m_base*/ PyModuleDef_HEAD_INIT,
     /*m_name*/ "mathutils.bvhtree",
